package com.hanyun.hip.mrqc.jms.api;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.setting.Setting;
import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.hanyun.hip.mrqc.common.EmrETLApi;
import com.hanyun.hip.mrqc.common.PageRequest;
import com.hanyun.hip.mrqc.rule.util.EngrUtil;
import com.hanyun.hip.mrqc.service.entity.EmrContent;
import com.hanyun.hip.mrqc.service.entity.EmrRecord;
import com.hanyun.hip.mrqc.service.service.IMrqcEtlService;
import com.hanyun.hip.mrqc.service.util.CommonUtil;
import com.hanyun.hip.mrqc.service.util.DateUtils;
import com.hanyun.hip.mrqc.service.util.EncryptUtil;
import com.hanyun.hip.mrqc.service.util.XmlTool;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.util.EntityUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowCallbackHandler;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.net.ssl.SSLContext;
import java.io.File;
import java.nio.charset.Charset;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.*;
import java.util.concurrent.ConcurrentMap;
import java.util.regex.Matcher;
import java.util.stream.Collectors;

@Service("JmsEmrETLApi")
public class JmsEmrETLApi implements EmrETLApi {

    private static final Logger LOGGER = LogManager.getLogger(JmsEmrETLApi.class);

    private static final JdbcTemplate jyJdbcTemplate = new JdbcTemplate();
    private static final JdbcTemplate vteJdbcTemplate = new JdbcTemplate();
    private static final String CDATA_START = "<!\\[CDATA\\[";
    private static final String CDATA_END = "]]>";

    private static final String HOSPITAL_CODE = "41427642-9";

    private static final String HOSPITAL_CODE_0 = "41427642-0";
    private static final String HOSPITAL_NAME = "佳木斯市中心医院";

    private static final String WEBSERVICE_URL = "http://172.17.0.206/csp/hsb/DHC.Published.PUB0010.BS.PUB0010.CLS";

    private static final String WEBSERVICE_SOAPACTION = "http://www.dhcc.com.cn/DHC.Published.PUB0010.BS.PUB0010.GetHISInfo";

    private static final String WEBSERVICE_XML = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:dhcc=\"http://www.dhcc.com.cn\">\n" +
            "<soapenv:Header/>\n" +
            "<soapenv:Body>\n" +
            "    <dhcc:GetHISInfo>\n" +
            "        <dhcc:action>%s</dhcc:action>\n" +
            "        <dhcc:Input><![CDATA[%s]]></dhcc:Input>\n" +
            "    </dhcc:GetHISInfo>\n" +
            "</soapenv:Body>\n" +
            "</soapenv:Envelope>";

    private static final String YZ_HTML = "<!DOCTYPE html>\r\n"
            + "<html lang=\"zh\" xmlns:th=\"http://www.thymeleaf.org\" xmlns:shiro=\"http://www.pollix.at/thymeleaf/shiro\">\r\n"
            + "<head>\r\n" + "	<th:block th:include=\"include :: header('医嘱列表')\" />\r\n" + "</head>\r\n"
            + "<body class=\"gray-bg\">" + " \r\n" + "<!-- 你的HTML代码 -->${yzTableHtml}\r\n" + " \r\n"
            + "<th:block th:include=\"include :: footer\" />\r\n" + "</body>\r\n" + "</html>";
    private static String XMLNS_FLAG = "xmlns=\"http://www.abisource.com/awml.dtd\"";

    private static String html = "<!DOCTYPE html>\r\n"
            + "<html lang=\"zh\" xmlns:th=\"http://www.thymeleaf.org\" xmlns:shiro=\"http://www.pollix.at/thymeleaf/shiro\">\r\n"
            + "<head>\r\n" + "	<th:block th:include=\"include :: header('医嘱列表')\" />\r\n" + "</head>\r\n"
            + "<body class=\"gray-bg\">" + " \r\n" + "<!-- 你的HTML代码 -->${yzTableHtml}\r\n" + " \r\n"
            + "<th:block th:include=\"include :: footer\" />\r\n" + "</body>\r\n" + "</html>";

    private static String vteHtml = "<!DOCTYPE html>\n" +
            "<html lang=\"zh\" xmlns:th=\"http://www.thymeleaf.org\" xmlns:shiro=\"http://www.pollix.at/thymeleaf/shiro\">\n" +
            "  <head>\n" +
            "    <th:block th:include=\"include :: header('VTE评估表')\" />\n" +
            "    <style>\n" +
            "      table {\n" +
            "        width: 100%;\n" +
            "        border-collapse: collapse;\n" +
            "      }\n" +
            "      table tr {\n" +
            "      }\n" +
            "      table td {\n" +
            "\t\t\t\tfont-size: 14px;\n" +
            "        border-bottom: 1px solid #ccc;\n" +
            "        padding: 6px 8px;\n" +
            "        line-height: 2em;\n" +
            "        width: 16.66%;\n" +
            "        text-align: center;\n" +
            "        border: 1px solid #2f4050 !important;\n" +
            "      }\n" +
            "    </style>\n" +
            "  </head>\n" +
            "  <body class=\"gray-bg\">\n" +
            "    <!-- 你的HTML代码 -->\n" +
            "    <div class=\"fixed-table-body\">\n" +
            "      <table id=\"bootstrap-table\" class=\"table table-hover\">${yzTableHtml}\n" +
            "      </table>\n" +
            "    </div>\n" +
            "    <th:block th:include=\"include :: footer\" />\n" +
            "  </body>\n" +
            "</html>";

    private static class StaticParams {
        // MES0422 获取病案首页信息  入参格式：<Request><AdmRowId>3</AdmRowId></Request>
        private static final String SY_CODE = "MES0001";
        private static final String SY_PARAM = "<Request><AdmRowId>%s</AdmRowId></Request>";
        // MES0431	区间病历质控-获取出院范围内的就诊号  <Request><StDate>2022-09-01</StDate><EnDate>2022-09-01</EnDate></Request>
        private static final String QJBL_CODE = "MES0004";

        private static final String IJBL_CODE = "MES0010";
        private static final String QJBL_PARAM = "<Request><StDate>%S</StDate><EnDate>%S</EnDate></Request>";
        // MES0002 获取病程记录信息
        private static final String BC_CODE = "MES0002";
        // MES0006 获取入院记录信息  入参格式：就诊号
        private static final String RY_CODE = "MES0006";
        // MES0008 获取知情同意书信息 入参格式：就诊号
        private static final String ZQTYS_CODE = "MES0008";
        // MES0003 获取出院记录信息   入参格式：就诊号
        private static final String CY_CODE = "MES0003";
        // MES0007 获取手术信息      入参格式：就诊号
        private static final String SS_CODE = "MES0007";
        // MES0007 文档检索-病历质控
        private static final String JS_CODE = "MES0428";
        // MES0429 文档调阅-病历质控
        private static final String DY_CODE = "MES0429";
        // MES0430 获取医嘱信息-病历质控    入参格式：就诊号
        private static final String YZ_CODE = "MES0005";
        // MES0517	获取手术信息-麻醉术前访视记录    入参格式：就诊号
        private static final String SQFSJL_CODE = "MES0517";
        // MES0518	获取手术信息-麻醉术后访视记录    入参格式：就诊号
        private static final String SHFSJL_CODE = "MES0518";
        // MES0519	获取手术信息-手术安全核查表       入参格式：就诊号
        private static final String AQHCB_CODE = "MES0519";

        //MES0428 检验检查报告
        private static final String JYJCBG_CODE = "MES0428";
        private static final String JYJCBG_PARAM = "<Request><Header><SourceSystem></SourceSystem><MessageID></MessageID></Header><Body><DocumentRetrievalRt><PATPatientID></PATPatientID><PAADMVisitNumber>%s</PAADMVisitNumber><DocumentType>%s</DocumentType><StartDate></StartDate><EndDate></EndDate><DocumentFormat>xml</DocumentFormat></DocumentRetrievalRt></Body></Request>";
        //MES0429 检验检查明细
        private static final String JYJCMX_CODE = "MES0429";
        private static final String JYJCMX_PARAM = "<Request><Header><SourceSystem></SourceSystem><MessageID></MessageID></Header><Body><DocumentSearchRt><PATPatientID></PATPatientID><PAADMVisitNumber>%s</PAADMVisitNumber><DocumentType>%s</DocumentType><DocumentID>%s</DocumentID></DocumentSearchRt></Body></Request>";

        //检查报告类型
        private static final String INSPECT_TYPES = "02001,02002,02003,02004,02005,02006";
        //检验报告类型
        private static final String TEST_TYPES = "01001,01002";

        private static String HTML = "<!DOCTYPE html>\r\n"
                + "<html lang=\"zh\" xmlns:th=\"http://www.thymeleaf.org\" xmlns:shiro=\"http://www.pollix.at/thymeleaf/shiro\">\r\n"
                + "<head>\r\n" + "	<th:block th:include=\"include :: header('医嘱列表')\" />\r\n" + "</head>\r\n"
                + "<body class=\"gray-bg\">" + " \r\n" + "<!-- 你的HTML代码 -->${yzTableHtml}\r\n" + " \r\n"
                + "<th:block th:include=\"include :: footer\" />\r\n" + "</body>\r\n" + "</html>";

        private static String YZ_TABLE_HTML = "<div class=\"fixed-table-body\">\r\n"
                + "	<table id=\"bootstrap-table\" class=\"table table-hover\">\r\n" + "		<thead>\r\n"
                + "			<tr>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" colspan=\"5\" tabindex=\"0\"><div\r\n"
                + "						class=\"th-inner \">开始</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" colspan=\"3\" tabindex=\"0\"><div\r\n"
                + "						class=\"th-inner \">结束</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n" + "			</tr>"
                + "			<tr>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userName\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" nowrap>类型</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" style='width:100px' nowrap>日期时间</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" nowrap>医嘱</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userName\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" nowrap>医生</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userPhone\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" nowrap>护士</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userEmail\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" style='width:100px'  nowrap>日期时间</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userBalance\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" nowrap>医生</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n"
                + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userBalance\" tabindex=\"0\" nowrap><div\r\n"
                + "						class=\"th-inner \" nowrap>护士</div>\r\n"
                + "					<div class=\"fht-cell\"></div></th>\r\n" + "			</tr>\r\n"
                + "		</thead>" + "		<tbody>";

        private static String jcSql = "SELECT * from web_test.JYJCST_HDICLIEXAINFO(\"%s\",\"%s\") WHERE business_id= ? ";

        private static String jySql = "SELECT * from web_test.JYJCST_HDILAREXADETLIST(\"%s\",\"%s\") WHERE business_id= ? ";

        private static String vteSql = "select *  from  view_single_disease_vte_reporting  where ass_type != 'meTaboo' and patient_no = ? and sub_trigger_type is not null group by ass_type,risk_id order by  sub_trigger_type";

    }

    static {
        Setting poolSetting = new Setting();
        poolSetting.put("initialSize", "20");
        poolSetting.put("maxActive", "50");
        poolSetting.put("maxWait", "60000");
        poolSetting.put("validationQuery", "SELECT 1");

        jyJdbcTemplate.setDataSource(createDataSource("jdbc:Cache://172.17.0.34:1972/dhc-app",
                "com.intersys.jdbc.CacheDriver", "JSHYUsers", "JSHYUsers", poolSetting));
        poolSetting.put("validationQuery", "SELECT 1 FROM DUAL");
        vteJdbcTemplate.setDataSource(createDataSource("jdbc:mysql://172.17.0.218:3307/vte3_cdss?useUnicode=true&characterEncoding=utf8&useSSL=true&allowMultiQueries=true&serverTimezone=GMT%2B8", "com.mysql.cj.jdbc.Driver", "root", "vte_m_2021", poolSetting));

    }

    protected static DruidDataSource createDataSource(String jdbcUrl, String driver, String user, String pass,
                                                      Setting poolSetting) {
        final DruidDataSource ds = new DruidDataSource();
        ds.setUrl(jdbcUrl);
        ds.setDriverClassName(driver);
        ds.setUsername(user);
        ds.setPassword(pass);
        // 规范化属性名
        Properties druidProps = new Properties();
        String keyStr;
        for (Map.Entry<String, String> entry : poolSetting.entrySet()) {
            keyStr = StrUtil.addPrefixIfNot(entry.getKey(), "druid.");
            druidProps.put(keyStr, entry.getValue());
        }
        // 连接池信息
        ds.configFromPropety(druidProps);
        // 检查关联配置，在用户未设置某项配置时，
        if (null == ds.getValidationQuery()) {
            // 在validationQuery未设置的情况下，以下三项设置都将无效
            ds.setTestOnBorrow(false);
            ds.setTestOnReturn(false);
            ds.setTestWhileIdle(false);
        }
        return ds;
    }

    @Override
    public List<EmrRecord> selectEmrRecordList(String hospitalCode, String recordId, String hospitalNum, Date beginDate, Date endDate, PageRequest pager, List<String> logMsg, Boolean notExtract) throws Exception {
        LinkedList<EmrRecord> emrRecords = Lists.newLinkedList();
        if (StringUtils.isNotBlank(recordId)) {
            List<String> recordIds = Lists.newArrayList();
            recordIds.add(recordId);
            return assembleRecordIndex(recordIds);
        } else if (null != beginDate && null != endDate) {
            List<String> recordIds = assembleTimeInterval(DateUtils.dateToString(beginDate, DateUtils.DATE_FORMAT), DateUtils.dateToString(endDate, DateUtils.DATE_FORMAT));
            return assembleRecordIndex(recordIds);
        }
        return emrRecords;
    }

    @Override
    public List<EmrContent> selectEmrContentList(EmrRecord record, List<String> logMsg) throws Exception {
        String recordId = record.getRecordId();
        List<EmrContent> result = new ArrayList<>();
        if (StringUtils.isNotBlank(recordId)) {
            //首页
            EmrContent basy = assembleRecordIndex(record);
            result.add(basy);
            //入院记录
            List<EmrContent> ryEmrContents = assembleOtherContent(StaticParams.RY_CODE, recordId);
            if (!CollectionUtils.isEmpty(ryEmrContents)) result.addAll(ryEmrContents);

            //出院记录
            List<EmrContent> cyEmrContents = assembleOtherContent(StaticParams.CY_CODE, recordId);
            if (!CollectionUtils.isEmpty(cyEmrContents)) result.addAll(cyEmrContents);

            //手术记录
            List<EmrContent> ssEmrContents = assembleOtherContent(StaticParams.SS_CODE, recordId);
            if (!CollectionUtils.isEmpty(ssEmrContents)) result.addAll(ssEmrContents);

            //知情同意书记录
            List<EmrContent> zqEmrContents = assembleOtherContent(StaticParams.ZQTYS_CODE, recordId);
            if (!CollectionUtils.isEmpty(zqEmrContents)) result.addAll(zqEmrContents);

            //病程记录
            List<EmrContent> bcEmrContents = assembleOtherContent(StaticParams.BC_CODE, recordId);
            if (!CollectionUtils.isEmpty(bcEmrContents)) result.addAll(bcEmrContents);

           /* //术前访视记录
            LinkedList<EmrContent> sqEmrContents = assembleOtherContent(StaticParams.SQFSJL_CODE,  recordId);
            if(!CollectionUtils.isEmpty(sqEmrContents)) result.addAll(sqEmrContents);

            //术后访视记录
            LinkedList<EmrContent> shEmrContents = assembleOtherContent(StaticParams.SHFSJL_CODE,  recordId);
            if(!CollectionUtils.isEmpty(shEmrContents)) result.addAll(shEmrContents);*/

            //安全核查记录
            EmrContent anhcEmrContents = assembleSafetyChecklist(StaticParams.AQHCB_CODE, recordId);
            if (null != anhcEmrContents) result.add(anhcEmrContents);

            //医嘱
            LinkedHashMap<String, JSONArray> advice = findDoctorAdvice(StaticParams.YZ_CODE, recordId);
            JSONArray lsJsonArray = advice.get("临时医嘱");
            EmrContent lsContent = assembleDoctorAdvice("临时医嘱", "1", lsJsonArray);
            if (null != lsContent && StringUtils.isNotBlank(lsContent.getContentId())) result.add(lsContent);
            JSONArray cqJsonArray = advice.get("长期医嘱");
            EmrContent cqContent = assembleDoctorAdvice("长期医嘱", "0", cqJsonArray);
            if (null != cqContent && StringUtils.isNotBlank(cqContent.getContentId())) result.add(cqContent);
            // //检验检查
            // List<EmrContent> inspectionContent = getAndInspectionContent(record);
            // result.addAll(inspectionContent);

            EmrContent vteContent = assembleVTE(recordId);
            if (null != vteContent && StringUtils.isNotBlank(vteContent.getContentId())) result.add(vteContent);

            result = result.stream().sorted(Comparator.comparing(emr -> {
                if (ObjectUtil.isNotEmpty(emr.getContentTime())) {
                    return emr.getContentTime();
                } else {
                    return new Date();
                }
            })).collect(Collectors.toList());
        }
        return result;
    }

    private static List<EmrContent> getAndInspectionContent(EmrRecord record) {
        List<EmrContent> emrContentList = new ArrayList<EmrContent>();
        String recordId = record.getRecordId();
        Date inTime = record.getInTime();
        Date outTime = null == record.getOutTime() ? Calendar.getInstance().getTime() : record.getOutTime();
        List<HashMap<String, Object>> jcList = new ArrayList<>();
        ConcurrentMap<String, HashMap<String, Object>> jcDistinctMap = Maps.newConcurrentMap();

        try {
            /** 检查 */
            jyJdbcTemplate.query(String.format(StaticParams.jcSql, DateUtil.format(inTime, "yyyy-MM-dd"), DateUtil.format(outTime, "yyyy-MM-dd")), new Object[]{recordId}, new RowCallbackHandler() {
                @Override
                public void processRow(ResultSet rs) throws SQLException {
                    HashMap<String, Object> blwsJsonMap = Maps.newHashMap();
                    ResultSetMetaData rsMeta = rs.getMetaData();
                    for (int i = 0, size = rsMeta.getColumnCount(); i < size; ++i) {
                        String columName = rsMeta.getColumnLabel(i + 1);
                        Object value = rs.getObject(i + 1);
                        blwsJsonMap.put(columName, value);
                    }
                    jcList.add(blwsJsonMap);
                }
            });
        } catch (Exception e) {
        }

        EmrContent jcEmrContent = getJcEmrContent(jcList, record);
        if (null != jcEmrContent) {
            emrContentList.add(jcEmrContent);
        }

        /** 检验 */
        List<HashMap<String, Object>> jyList = new ArrayList<>();
        try {
            jyJdbcTemplate.query(String.format(StaticParams.jySql, DateUtil.format(inTime, "yyyyMMdd"), DateUtil.format(outTime, "yyyyMMdd")), new Object[]{recordId}, new RowCallbackHandler() {
                @Override
                public void processRow(ResultSet rs) throws SQLException {
                    HashMap<String, Object> blwsJsonMap = Maps.newHashMap();
                    ResultSetMetaData rsMeta = rs.getMetaData();
                    for (int i = 0, size = rsMeta.getColumnCount(); i < size; ++i) {
                        String columName = rsMeta.getColumnLabel(i + 1);
                        Object value = rs.getObject(i + 1);
                        blwsJsonMap.put(columName, value);
                    }
                    jyList.add(blwsJsonMap);
                }
            });
        } catch (Exception  e) {

        }

        EmrContent jyEmrContent = getJyEmrContent(jyList, record);

        if (null != jyEmrContent) {
            emrContentList.add(jyEmrContent);
        }
        return emrContentList;
    }

    private static EmrContent getJyEmrContent(List<HashMap<String, Object>> yzLists, EmrRecord emrRecord) {
        EmrContent emrContent = null;
        String recordId = emrRecord.getRecordId();
        Date inTime = emrRecord.getInTime();
        Date outTime = null == emrRecord.getOutTime() ? Calendar.getInstance().getTime() : emrRecord.getOutTime();
        if (!CollectionUtils.isEmpty(yzLists)) {
            emrContent = new EmrContent();
            emrContent.isAutoJson = false;
            emrContent.setContentId("jy" + recordId);
            emrContent.setTitle("检验报告单");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(new Date());
//            emrContent.setEmrId("emr06");

            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(new Date());
            Map<String, List<HashMap<String, Object>>> map = new HashMap<String, List<HashMap<String, Object>>>();
            map.put("datas", yzLists);
            String structJson = JSON.toJSONString(map);
            emrContent.setStructJson(structJson);
            int j = 0;
            StringBuffer jyTableHtml = new StringBuffer();
            jyTableHtml.append("<div class=\"fixed-table-body\">\r\n");
            jyTableHtml.append("	<table id=\"bootstrap-table\" class=\"table table-hover\">\r\n" + "		<thead>\r\n");
            jyTableHtml.append("			<tr>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验类别</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验项目</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验结果</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>参考值</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>单位</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>执行医生</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>异常提示</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
            jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验日期时间</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");

            jyTableHtml.append("			</tr>\r\n		</thead>\r\n		<tbody>\r\n");
            for (Map<String, Object> tempMap : yzLists) {
                String exeDate = mapNullValueReplace(tempMap.get("AP090002500"));

                Date jyTime = DateUtils.stringToDate(exeDate, "yyyyMMddHHmmss");
                jyTableHtml.append("			<tr data-index=\"" + j + "\">\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(mapNullValueReplace(tempMap.get("TestSetName")));
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(mapNullValueReplace(tempMap.get("TestName")));
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(mapNullValueReplace(tempMap.get("Result")));
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(mapNullValueReplace(tempMap.get("Ranges")));
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(mapNullValueReplace(tempMap.get("Unit")));
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(mapNullValueReplace(tempMap.get("AuthUserName")));
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(mapNullValueReplace(tempMap.get("ResFlag")));
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append(exeDate);
                jyTableHtml.append("</td>\r\n");

                jyTableHtml.append("			</tr>");

                j++;
            }
            jyTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(html.replaceAll("\\$\\{yzTableHtml\\}", Matcher.quoteReplacement(jyTableHtml.toString())));
        }

        return emrContent;
    }

    private EmrContent assembleVTE(String recordId) {
        EmrContent emrContent = null;

        try {
            List<HashMap<String, Object>> vteList = new ArrayList<>();
            /** VTE */
            vteJdbcTemplate.query(StaticParams.vteSql, new Object[]{recordId}, new RowCallbackHandler() {
                @Override
                public void processRow(ResultSet rs) throws SQLException {
                    HashMap<String, Object> blwsJsonMap = Maps.newHashMap();
                    ResultSetMetaData rsMeta = rs.getMetaData();
                    for (int i = 0, size = rsMeta.getColumnCount(); i < size; ++i) {
                        String columName = rsMeta.getColumnLabel(i + 1);
                        Object value = rs.getObject(i + 1);
                        blwsJsonMap.put(columName, value);
                    }
                    vteList.add(blwsJsonMap);
                }
            });
            if (CollUtil.isNotEmpty(vteList)) {
                emrContent = new EmrContent();
                StringBuffer tableStr = new StringBuffer();
                emrContent.isAutoJson = false;
                emrContent.setContentId("vte" + recordId);
                emrContent.setTitle("VTE相关风险动态评估表");
                emrContent.setRecordId(recordId);
                emrContent.setCreateBy("admin");
                emrContent.setCreateTime(new Date());
    //            emrContent.setEmrId("emr06");

                emrContent.setShowOrder("1");
                emrContent.setUpdateBy("admin");
                emrContent.setUpdateTime(new Date());
                Map<String, List<HashMap<String, Object>>> map = new HashMap<String, List<HashMap<String, Object>>>();
                map.put("datas", vteList);
                String structJson = JSON.toJSONString(map);
                emrContent.setStructJson(structJson);
                Map<Object, List<HashMap<String, Object>>> typeMap = vteList.stream().collect(Collectors.groupingBy(o -> o.get("sub_trigger_type")));
                typeMap.forEach((key, list) -> {
                    tableStr.append("<tr><td>评估时间</td><td colspan=\"5\">" + list.get(0).get("ass_time") + "</td></tr>");
                    tableStr.append("<tr><td>评估科室</td><td colspan=\"5\">" + list.get(0).get("emergency_dept") + "</td></tr>");
                    Map<Object, List<HashMap<String, Object>>> ass_type = list.stream().collect(Collectors.groupingBy(o -> o.get("ass_type")));
                    ass_type.forEach((assKey, dataList) -> {
                        tableStr.append("<tr><td>量表名称</td><td>" + assKey + "</td> <td>评估时机</td><td>" + key + "</td><td>评估分数</td><td>" + dataList.get(0).get("scores") + "</td></tr>");
                        String itemName = "";
                        for (int i = 0; i < dataList.size(); i++) {
                            int n = i + 1;
                            Object ass_item_name = dataList.get(i).get("ass_item_name");
                            if (ObjectUtil.isEmpty(ass_item_name)) break;
                            itemName += n + "." + ass_item_name + "(" + dataList.get(i).get("score") + ")   ";
                        }
                        tableStr.append("<tr><td>危险因素</td><td colspan=\"3\">" + itemName + "</td> <td>风险评估等级</td><td>" + dataList.get(0).get("risk_level") + "</td></tr>");
                    });
                    tableStr.append("<tr><td>护士签名</td><td colspan=\"1\">" + list.get(0).get("nurse_confirm_name") + "</td><td>医生签名</td><td >" + list.get(0).get("doc_confirm_name") + "</td><td>签名时间</td><td >" + list.get(0).get("doc_confirm_time") + "</td></tr>");
                });
                emrContent.setContentText(vteHtml.replaceAll("\\$\\{yzTableHtml\\}", tableStr.toString()));

            }
        } catch (DataAccessException e) {
            LOGGER.error("assembleVTE error", e);
        }
        return emrContent;
    }

    private static EmrContent getJcEmrContent(List<HashMap<String, Object>> jyList, EmrRecord emrRecord) {
        EmrContent emrContent = null;
        String recordId = emrRecord.getRecordId();
        Date inTime = emrRecord.getInTime();
        Date outTime = null == emrRecord.getOutTime() ? Calendar.getInstance().getTime() : emrRecord.getOutTime();
        if (!CollectionUtils.isEmpty(jyList)) {
            emrContent = new EmrContent();
            emrContent.isAutoJson = false;
            emrContent.setContentId("jc" + recordId);
            emrContent.setTitle("检查报告单");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(new Date());
//            emrContent.setEmrId("emr06");

            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(new Date());
            Map<String, List<HashMap<String, Object>>> map = new HashMap<String, List<HashMap<String, Object>>>();
            map.put("datas", jyList);
            String structJson = JSON.toJSONString(map);
            emrContent.setStructJson(structJson);

            int j = 0;
            StringBuffer jcTableHtml = new StringBuffer();
            jcTableHtml.append("<div class=\"fixed-table-body\">\r\n");
            jcTableHtml.append("	<table id=\"bootstrap-table\" class=\"table table-hover\">\r\n");
            jcTableHtml.append("        <thead>\r\n");
            jcTableHtml.append("			<tr>\r\n");
            jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查类别</div>\r\n");
            jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
            jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查项目</div>\r\n");
            jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
            jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查医生</div>\r\n");
            jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
            jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查结果</div>\r\n");
            jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
            jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查日期时间</div>\r\n");
            jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
            jcTableHtml.append("				</tr>\r\n");

            for (HashMap<String, Object> tempMap : jyList) {
                String exeDate = CommonUtil.getMap(tempMap, "WS06_00_924_01", "yyyy-MM-dd'T'HH:mm:ss");
                Date jyTime = DateUtils.stringToDate(exeDate, "yyyy-MM-dd HH:mm:ss");
                jcTableHtml.append("		<tr data-index=\"" + j + "\">\r\n");

                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(mapNullValueReplace(tempMap.get("DE04_30_019_01")));
                jcTableHtml.append("</td>\r\n");

                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(mapNullValueReplace(tempMap.get("DE04_30_020_00")));
                jcTableHtml.append("</td>\r\n");

                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(mapNullValueReplace(tempMap.get("DE02_01_039_064")));
                jcTableHtml.append("</td>\r\n");

                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(mapNullValueReplace(tempMap.get("DE04_50_131_00")));
                jcTableHtml.append("</td>\r\n");

                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(exeDate);
                jcTableHtml.append("</td>\r\n");

                jcTableHtml.append("		</tr>\r\n");

                j++;
            }
            jcTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(html.replaceAll("\\$\\{yzTableHtml\\}", Matcher.quoteReplacement(jcTableHtml.toString())));
        }

        return emrContent;
    }

    @Override
    public void writeEndDate(Date endDate) {

    }

    @Override
    public void deleteEmrContentList(IMrqcEtlService mrqcEtlService, EmrRecord record, List<String> logMsg)
            throws Exception {
        mrqcEtlService.deleteEmrContentsByRecord(record);
        logMsg.add("删除病历文书成功： [" + record.getRecordId() + "]");
    }

    /**
     * 组装首页及Record表数据
     *
     * @return 质控病历对象
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/14
     */
    private LinkedList<EmrRecord> assembleRecordIndex(List<String> recordIds) {
        LinkedList<EmrRecord> emrRecords = Lists.newLinkedList();
        LOGGER.info("开始获取病例并组装，共：{} 条。", recordIds.size());
        for (String recordId : recordIds) {
            String response = doPostSoap(WEBSERVICE_URL, StaticParams.SY_CODE, recordId, WEBSERVICE_SOAPACTION);
            try {
                JSONObject jsonObject = XmlTool.documentToJSONObject(response);
                JSONObject serviceResponse = jsonObject.getJSONObject("Body").getJSONObject("GetHISInfoResponse");
                if (null != serviceResponse && serviceResponse.size() != 0) {
                    String resultResponse = serviceResponse.getString("GetHISInfoResult");
                    JSONObject admFirstPage = XmlTool.documentToJSONObject(resultResponse);
//                    JSONObject admFirstPage = resultResponse;
                    EmrRecord emrRecord = new EmrRecord();
                    String record_id = mapNullValueReplace(admFirstPage.getString("AdmNo"));
                    emrRecord.setStructDataJson(admFirstPage.toJSONString());
                    emrRecord.setRecordId(record_id);// 记录id
                    emrRecord.setPatientNum(mapNullValueReplace(admFirstPage.getString("HealthCardId")));// 就诊号

                    emrRecord.setHospitNum(mapNullValueReplace(admFirstPage.getString("DocumentID")));// 门诊号/住院号
                    emrRecord.setPatientName(mapNullValueReplace(admFirstPage.getString("PatName"))); // 患者姓名
                    emrRecord.setHospitCode(HOSPITAL_CODE);
                    // if (StrUtil.startWith(emrRecord.getHospitNum(), "9")) {
                    //     emrRecord.setHospitCode(HOSPITAL_CODE);// 医院编码
                    // }else {
                    //     emrRecord.setHospitCode(HOSPITAL_CODE_0);// 医院编码
                    // }
                    emrRecord.setHospitName(HOSPITAL_NAME);// 医院名称
                    emrRecord.setPatientSex(mapNullValueReplace(admFirstPage.getString("Sex"))); // 患者性别
                    emrRecord.setBirthday(DateUtils.stringToDate(mapNullValueReplace(admFirstPage.getString("Birthday")), "YYYYmmdd"));// 出生日期
                    emrRecord.setAge(mapNullValueReplace(admFirstPage.getString("AgeYear")));
                    //主治
                    String zzys = mapNullValueReplace(admFirstPage.getString("AesculapiusDesc"));
                    String zzysCode = mapNullValueReplace(admFirstPage.getString("AttendingCode"));
                    if (EngrUtil.isNone(zzys)) {
                        zzys = mapNullValueReplace(admFirstPage.getString("DirectorDesc"));
                        zzysCode = mapNullValueReplace(admFirstPage.getString("DirectorCode"));
                    }
                    // 诊疗医生 --取主治医师
                    emrRecord.setHospitDoctor(zzys);
                    emrRecord.setDoctorCode(zzysCode);

                    // 病历评分
                    emrRecord.setRecordScore(null);
                    emrRecord.setHomepageScore(null);

                    emrRecord.setCreater("admin"); // 创建者

                    emrRecord.setCreateTime(Calendar.getInstance().getTime());// 修改时间
                    emrRecord.setModifyer("admin"); // 修改者
                    emrRecord.setModifyTime(Calendar.getInstance().getTime()); // 修改时间
                    emrRecord.setInTime(DateUtils.stringToDate(mapNullValueReplace(admFirstPage.getString("AdmSDate")), DateUtils.DATE_TIME_MINUTE));//入院时间
                    emrRecord.setOutTime(DateUtils.stringToDate(mapNullValueReplace(admFirstPage.getString("AdmDDate")), DateUtils.DATE_TIME_MINUTE));//出院时间
                    // emrRecord.setEtlTime(Calendar.getInstance().getTime());
                    if (null != emrRecord.getOutTime()) {
                        // 诊疗时间--对应出院院时间
                        emrRecord.setHospitTime(DateUtils.stringToDate(mapNullValueReplace(admFirstPage.getString("AdmDDate")), DateUtils.DATE_TIME_FORMAT));
                        emrRecord.setDepartmentCode(mapNullValueReplace(admFirstPage.getString("AdmDWardCode"))); // 病区编码
                        emrRecord.setDepartmentName(mapNullValueReplace(admFirstPage.getString("AdmDWard"))); // 病区名称

                        emrRecord.setInpatientCode(mapNullValueReplace(admFirstPage.getString("AdmDDeptCode")));
                        emrRecord.setInpatientArea(mapNullValueReplace(admFirstPage.getString("AdmDDept"))); //科室
                        emrRecord.setEmrStatus("4");
                    } else {
                        // 诊疗时间--对应入院院时间
                        emrRecord.setHospitTime(DateUtils.stringToDate(mapNullValueReplace(admFirstPage.getString("AdmSDate")), DateUtils.DATE_TIME_FORMAT));
                        emrRecord.setDepartmentCode(mapNullValueReplace(admFirstPage.getString("AdmSWardCode"))); // 病区编码
                        emrRecord.setDepartmentName(mapNullValueReplace(admFirstPage.getString("AdmSWard"))); // 病区名称

                        emrRecord.setInpatientCode(mapNullValueReplace(admFirstPage.getString("AdmSDeptCode")));
                        emrRecord.setInpatientArea(mapNullValueReplace(admFirstPage.getString("AdmSDept"))); // 科室名称
                        emrRecord.setEmrStatus("1");
                    }
                    JSONArray diagnoseInfos = new JSONArray();
                    if (admFirstPage.getJSONObject("DiagnoseInfo").get("Diagnose") instanceof JSONObject) {
                        diagnoseInfos.add(admFirstPage.getJSONObject("DiagnoseInfo").getJSONObject("Diagnose"));
                    }
                    if (admFirstPage.getJSONObject("DiagnoseInfo").get("Diagnose") instanceof JSONArray) {
                        diagnoseInfos.addAll(admFirstPage.getJSONObject("DiagnoseInfo").getJSONArray("Diagnose"));
                    }
                    if (!CollectionUtils.isEmpty(diagnoseInfos)) {
                        for (Object info : diagnoseInfos) {
                            JSONObject diagnose = (JSONObject) info;
                            String diagnoseMarker = diagnose.getString("DiagnoseMarker");
                            if ("1".equals(diagnoseMarker)) {
                                emrRecord.setDiagnosisCode(diagnose.getString("DiagnoseCode"));
                                emrRecord.setDiagnosisName(diagnose.getString("DiagnoseDesc"));
                            }
                        }
                    }
                    emrRecords.add(emrRecord);

                }
            } catch (DocumentException e) {
                LOGGER.error("数据抽提, 病历recordId: {} ,病案数据XML转换异常! ExceptionMsg: {}", recordId, e.getMessage());
            }
        }
        LOGGER.info("病历Record组装完成，共{} 份。", emrRecords.size());
        return emrRecords;
    }

    /**
     * 根据时间区间获取病历ID
     *
     * @param beginDate 起始时间[YYYYmmdd]
     * @param endDate   截止时间[YYYYmmdd]
     * @return 病历唯一标识集合
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/14
     */
    private List<String> assembleTimeInterval(String beginDate, String endDate) {
        List<String> result = new ArrayList<>();
        String response = doPostSoap(WEBSERVICE_URL, StaticParams.QJBL_CODE, String.format(StaticParams.QJBL_PARAM, beginDate, endDate), WEBSERVICE_SOAPACTION);
        String responseIn = doPostSoap(WEBSERVICE_URL, StaticParams.IJBL_CODE, String.format(StaticParams.QJBL_PARAM, beginDate, endDate), WEBSERVICE_SOAPACTION);
        try {
            getRecordIds(result, response);
            getRecordIds(result, responseIn);
        } catch (DocumentException e) {
            LOGGER.error("数据抽提, 病历起始时间: {} ,截止时间: {} ,病案数据XML转换异常! ExceptionMsg: {}", beginDate, endDate, ExceptionUtils.getStackTrace(e));
        }
        return result;
    }

    private static void getRecordIds(List<String> result, String response) throws DocumentException {
        JSONObject jsonObject = XmlTool.documentToJSONObject(response);
        JSONObject serviceResponse = jsonObject.getJSONObject("Body").getJSONObject("GetHISInfoResponse");
        if (null != serviceResponse && serviceResponse.size() != 0) {
            String resultResponse = serviceResponse.getString("GetHISInfoResult");
            if (StringUtils.isNotBlank(resultResponse)) {
                String[] split = resultResponse.split("\\^");
                result.addAll(Arrays.asList(split));
            }
        }
    }

    /**
     * 组装病案首页
     *
     * @param record 病历对象
     * @return 首页文书对象
     */
    private EmrContent assembleRecordIndex(EmrRecord record) {
        String recordId = record.getRecordId();
        EmrContent basy = new EmrContent();
        basy.setContentId(recordId);
        basy.setTitle("病案首页");
        basy.setEmrId("emr01");
        basy.setShowOrder("1");
        basy.setContentTime(record.getOutTime());
        basy.setRecordId(recordId);
        String dataJson = record.getStructDataJson();
        if (StringUtils.isNotBlank(dataJson)) {
            try {
                JSONObject jsonObject = JSON.parseObject(dataJson);
                String kezhuren = jsonObject.getString("kezhuren");
                if (StrUtil.isNotEmpty(kezhuren)&&!"^^".equals(kezhuren)) {
                    String[] split = kezhuren.split("\\^");
                    if (split.length>=3) {
                        String timeStr = split[1];
                        Date date = DateUtils.stringToDate(timeStr, DateUtils.DATE_TIME_FORMAT);
                        basy.setContentTime(date);
                    }
                }
            } catch (Exception e) {
                LOGGER.error("病案首页组装异常；病历：{}，异常：{}", recordId, e.getMessage());
            }
        }
        basy.setStructJson(dataJson);
        return basy;
    }

    /**
     * 组装文书对象
     *
     * @return LinkedList<EmrContent>
     */
    private List<EmrContent> assembleOtherContent(String code, String recordId) throws Exception {
        List<EmrContent> emrContents = new ArrayList<>();
        try {

            Element GetHISInfoResult = findGetHISInfoResult(code, recordId);
            if (null != GetHISInfoResult) {
                Element root = GetHISInfoResult.element("root");
                if (null != root) {
                    List<Element> elements = root.elements();
                    for (Element element : elements) {
                        EmrContent content = new EmrContent();
                        String contentId = element.elementText("content_id");
                        contentId = contentId.replaceAll("\\|\\|", "_");
                        content.setContentId(contentId);
                        content.setTitle(element.elementText("title"));
//                        content.setEmrId(element.elementText("emr_id"));
                        content.setShowUrl(element.elementText("content_html").replaceFirst(CDATA_START, "").replaceAll(CDATA_END, ""));
                        content.setShowOrder(element.elementText("show_order"));
                        String contentTime = element.elementText("content_time");
                        content.setContentTime(DateUtils.stringToDate(contentTime, DateUtils.DATE_TIME_FORMAT));
                        content.setRecordId(element.elementText("record_id"));
                        JSONObject object = setContentTimeByOrderTime(element, content);

                        try {
                            Element NInstanceData = element.element("content_xml").element("NInstanceData");
                            if (null == NInstanceData) {
                                NInstanceData = element.element("content_xml").element("Document").element("StructuredBody").element("NInstanceData");
                            }
                            if (null != NInstanceData) {
                                String xml = getXmlContent(NInstanceData);
                                content.setContentXml(xml);
                                JSONObject jsonObject = XmlTool.documentToJSONObject(xml);
                                jsonObject.put("object", object);
                                content.setStructJson(jsonObject.toJSONString());
                            } else {
                                LOGGER.error("获取文书正文【NInstanceData】异常! recordId: {},code: {}, Title: {}", recordId, code, content.getTitle());
                            }
                        } catch (Exception e) {
                            LOGGER.error("获取文书正文section异常! recordId: {},code: {}, Title: {}", recordId, code, content.getTitle());
                            LOGGER.error(ExceptionUtils.getStackTrace(e));
                            continue;
                        }
                        if (StringUtils.isNotBlank(content.getContentId())) {
                            emrContents.add(content);
                        }
                    }
                    return emrContents;
                }
            }

        } catch (Exception e) {
            LOGGER.error("获取文书异常! recordId: {},code: {}, param: {}; ExceptionMsg: {}", recordId, code, ExceptionUtils.getStackTrace(e));
            return Lists.newLinkedList();
        }
        return Lists.newLinkedList();
    }

    private static JSONObject setContentTimeByOrderTime(Element element, EmrContent content) {
        JSONObject object = new JSONObject();
        //主任医生
        String resident = element.elementText("Resident");
        object.put("Resident", resident);
        //主治医生
        String attending = element.elementText("Attending");
        object.put("Attending", attending);
        //住院医生
        String chief = element.elementText("Chief");
        object.put("Chief", chief);
        List<Date> dateList = new ArrayList<>();
        if (StrUtil.isNotEmpty(resident)&&!"^^".equals(resident)) {
            String[] split = resident.split("\\^");
            if (split.length>=3) {
                String timeStr = split[2];
                Date date = DateUtils.stringToDate(timeStr, DateUtils.DATE_TIME_FORMAT);
                dateList.add(date);
            }
        }
        if (StrUtil.isNotEmpty(attending)&&!"^^".equals(attending)) {
            String[] split = attending.split("\\^");
            if (split.length>=3) {
                String timeStr = split[2];
                Date date = DateUtils.stringToDate(timeStr, DateUtils.DATE_TIME_FORMAT);
                dateList.add(date);
            }
        }
        if (StrUtil.isNotEmpty(chief)&&!"^^".equals(chief)) {
            String[] split = chief.split("\\^");
            if (split.length>=3) {
                String timeStr = split[2];
                Date date = DateUtils.stringToDate(timeStr, DateUtils.DATE_TIME_FORMAT);
                dateList.add(date);
            }
        }
        Date maxDate = null;
        if (CollUtil.isNotEmpty(dateList)) {
            maxDate = dateList.stream()
                    .max(Date::compareTo)
                    .orElse(null);
        }
        if (ObjectUtil.isNotNull(maxDate)) {
            content.setContentTime(maxDate);
        }
        return object;
    }

    private static String getXmlContent(Element nInstanceData) {
        String xml = "";
        Element scatterData = nInstanceData.element("ScatterData");
        if (null != scatterData) {
            xml = xml + scatterData.asXML();
            xml = replaceXmlFlag(xml);
            return xml;
        }
        Element abiword = nInstanceData.element("abiword");
        if (null != abiword) {
            List<Element> abiwords = abiword.elements();
            for (Element e : abiwords) {
                if ("section".equals(e.getName())) {
                    //仅取第一个 section
                    Element data = abiword.element("data");
                    if (null != data) {
                        Element data1 = e.addElement("data");
                        List<Element> elementsB = data.elements("d");
                        Element elB = elementsB.get(0);
                        data1.setText("data:image/jpeg;base64," + elB.getText());
                    }
                    xml = e.asXML();
                    xml = replaceXmlFlag(xml);
                    break;
                }
            }
            return xml;
        }
        Element iEditor = nInstanceData.element("iEditor");
        if (null != iEditor) {
            xml = replaceXmlFlag(xml + iEditor.asXML());
            return xml;
        }
        return xml;
    }

    private static String replaceXmlFlag(String xml) {
        if (StrUtil.isNotBlank(xml) && xml.contains("xmlns")) {
            xml = xml.replace(XMLNS_FLAG, "");
        }
        return xml;
    }

    /**
     * 根据接口代码，获取结果元素
     *
     * @author ouyang0810@foxmail.com
     * @Date 2023/3/23
     */
    private Element findGetHISInfoResult(String code, String recordId) throws Exception {
        String response = doPostSoapNew(WEBSERVICE_URL, code, recordId, WEBSERVICE_SOAPACTION);
        Document document = XmlTool.strToDocument(response);
        Element rootElement = document.getRootElement();
        Element GetHISInfoResponse = rootElement.element("Body").element("GetHISInfoResponse");
        return GetHISInfoResponse.element("GetHISInfoResult");
    }

    /**
     * 获取医嘱集
     *
     * @param
     * @return LinkedHashMap
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/25
     */
    private LinkedHashMap<String, JSONArray> findDoctorAdvice(String code, String recordId) throws Exception {
        LinkedHashMap<String, JSONArray> hashMap = Maps.newLinkedHashMap();
        try {
            String response = doPostSoap(WEBSERVICE_URL, code, recordId, WEBSERVICE_SOAPACTION);


            JSONObject jsonObject = XmlTool.documentToJSONObject(response);
            JSONObject body = jsonObject.getJSONObject("Body");
            JSONObject GetHISInfoResponse = body.getJSONObject("GetHISInfoResponse");
            String GetHISInfoResult = GetHISInfoResponse.getString("GetHISInfoResult");
            if (!CollectionUtils.isEmpty(GetHISInfoResponse) && StrUtil.isNotEmpty(GetHISInfoResult)) {
                JSONObject orderList = XmlTool.documentToJSONObject(GetHISInfoResult);
                if (!CollectionUtils.isEmpty(orderList)) {
                    JSONArray orderInfo = new JSONArray();
                    if (orderList.get("OrderInfo") instanceof JSONArray) {
                        orderInfo = orderList.getJSONArray("OrderInfo");
                    }
                    if (orderList.get("OrderInfo") instanceof JSONObject) {
                        orderInfo.add(orderList.getJSONObject("OrderInfo"));
                    }
                    for (Object o : orderInfo) {
                        JSONObject order = (JSONObject) o;
                        EmrContent content = new EmrContent();
                        String title = order.getString("HDSD0015022");
                        if (hashMap.containsKey(title)) {
                            JSONArray jsonValues = hashMap.get(title);
                            jsonValues.add(order);
                            hashMap.put(title, jsonValues);
                        } else {
                            JSONArray jsonValues = new JSONArray();
                            jsonValues.add(order);
                            hashMap.put(title, jsonValues);
                        }
                    }
                }
            }
        } catch (Exception e) {
            LOGGER.error("获取医嘱异常! recordId: {},code: {}, param: {}; ExceptionMsg: {}", recordId, code, ExceptionUtils.getStackTrace(e));
            return Maps.newLinkedHashMap();
        }
        return hashMap;
    }

    /**
     * 组装医嘱
     *
     * @param
     * @return EmrContent
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/25
     */
    private EmrContent assembleDoctorAdvice(String title, String xh, JSONArray jsonArray) throws Exception {
        EmrContent content = new EmrContent();
        try {
            if (CollectionUtils.isEmpty(jsonArray)) {
                return null;
            }
            content.isAutoJson = false;
            JSONObject jsonObject = (JSONObject) jsonArray.get(0);
            content.setContentId(jsonObject.getString("RECORDID") + "_yz" + xh);
            content.setTitle(title);
            content.setRecordId(jsonObject.getString("RECORDID"));
            content.setCreateBy("admin");
            String startDateTime = CommonUtil.getMap(jsonObject.getInnerMap(), "HDSD0015018", "yyyy-MM-dd HH:mm:ss");
            content.setCreateTime(DateUtils.stringToDate(startDateTime, "yyyy-MM-dd HH:mm:ss"));
//            content.setEmrId("emr03");

            content.setShowOrder("1");
            content.setUpdateBy("admin");
            content.setUpdateTime(Calendar.getInstance().getTime());
            String yzTableHtml = StaticParams.YZ_TABLE_HTML;
            int j = 0;
            for (Object o : jsonArray) {
                JSONObject object = (JSONObject) o;
                Map<String, Object> map = object.getInnerMap();
                String BeginDate = (String) CommonUtil.getMap(map, "HDSD0015018", "yyyy-MM-dd HH:mm:ss");
                String EndDate = (String) CommonUtil.getMap(map, "HDSD0015026", "yyyy-MM-dd HH:mm:ss");

                String InputDoctorName = (String) CommonUtil.getMap(map, "HDSD0015021", null);
                String StopDoctorName = (String) CommonUtil.getMap(map, "HDSD0015013", null);
                String startStaffName = (String) CommonUtil.getMap(map, "HDSD0015006", null);
                String ExecStaffName = (String) CommonUtil.getMap(map, "HDSD0015014", null);

                String OrderContent = (String) CommonUtil.getMap(map, "HDSD0015028", null);

                String classType = (String) CommonUtil.getMap(map, "HDSD0015027", null);
                yzTableHtml += "			<tr data-index=\"" + j + "\">\r\n"
                        + "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">"
                        + classType + "</td>\r\n"
                        + "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">"
                        + BeginDate + "</td>\r\n"
                        + "				<td style=\"border: 1px solid #2f4050!important;\" name = \"contentText\">"
                        + OrderContent + "</td>\r\n"
                        + "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">"
                        + InputDoctorName + "</td>\r\n"
                        + "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">"
                        + startStaffName + "</td>\r\n"
                        + "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">"
                        + EndDate + "</td>\r\n"
                        + "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">"
                        + StopDoctorName + "</td>\r\n"
                        + "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">"
                        + ExecStaffName + "</td>\r\n" + "			</tr>";
                j++;
            }
            yzTableHtml += "		</tbody>\r\n" + "	</table>\r\n" + "</div>";
            content.setContentText(StaticParams.HTML.replaceAll("\\$\\{yzTableHtml\\}", Matcher.quoteReplacement(yzTableHtml)));
            JSONObject object = new JSONObject();
            object.put("datas", jsonArray);
            content.setStructJson(object.toJSONString());
        } catch (Exception e) {
            LOGGER.error("组装医嘱异常! title: {}, xh: {}; ExceptionMsg: {}", title, xh, ExceptionUtils.getStackTrace(e));
            return null;
        }
        return content;
    }

    /**
     * 组装检查
     *
     * @param
     * @return EmrContent
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/25
     */
    private EmrContent assembleExamination(String recordId) throws Exception {
        EmrContent emrContent = new EmrContent();
        //检查
        try {
            List<String> inspectTypes = Arrays.asList(StaticParams.INSPECT_TYPES.split(","));
            JSONArray resultJSONArray = new JSONArray();
            for (String s : inspectTypes) {
                JSONArray examination = findExamination(StaticParams.JYJCBG_CODE, StaticParams.JYJCBG_PARAM, s, "", recordId);
                if (!CollectionUtils.isEmpty(examination)) resultJSONArray.addAll(examination);
            }

            JSONArray details = findExaminationDetail(resultJSONArray, recordId);
            if (!CollectionUtils.isEmpty(details)) {
                emrContent = new EmrContent();
                emrContent.isAutoJson = false;
                emrContent.setContentId("jc_" + recordId);
                emrContent.setTitle("检查报告单");
                emrContent.setRecordId(recordId);
                emrContent.setCreateBy("admin");
                emrContent.setCreateTime(Calendar.getInstance().getTime());
                emrContent.setEmrId("emr06");

                emrContent.setShowOrder("1");
                emrContent.setUpdateBy("admin");
                emrContent.setUpdateTime(Calendar.getInstance().getTime());
                JSONObject object1 = new JSONObject();
                object1.put("datas", details);
                emrContent.setStructJson(object1.toJSONString());

                StringBuffer jcTableHtml = new StringBuffer();
                jcTableHtml.append("<div class=\"fixed-table-body\">\r\n");
                jcTableHtml.append("	<table id=\"bootstrap-table\" class=\"table table-hover\">\r\n");
                jcTableHtml.append("        <thead>\r\n");
                jcTableHtml.append("			<tr>\r\n");
                jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查项目</div>\r\n");
                jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
                jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查医生</div>\r\n");
                jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
                jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查所见</div>\r\n");
                jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
                jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查结果</div>\r\n");
                jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
                jcTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userCode\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检查日期时间</div>\r\n");
                jcTableHtml.append("				<div class=\"fht-cell\"></div></th>\r\n");
                jcTableHtml.append("				</tr>\r\n");

                for (int i = 0; i < details.size(); i++) {

                    JSONObject object = details.getJSONObject(i);

                    jcTableHtml.append("		<tr data-index=\"" + i + "\">\r\n");

                    jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                    jcTableHtml.append(mapNullValueReplace(object.getJSONObject("E21").getString("nodeText")));
                    jcTableHtml.append(")</td>\r\n");

                    jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                    jcTableHtml.append(mapNullValueReplace(object.getJSONObject("E11").getString("nodeText")));
                    jcTableHtml.append("</td>\r\n");

                    jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                    jcTableHtml.append(mapNullValueReplace(object.getJSONObject("E08").getString("nodeText")));
                    jcTableHtml.append("</td>\r\n");

                    jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                    jcTableHtml.append(mapNullValueReplace(object.getJSONObject("E09").getString("nodeText")));
                    jcTableHtml.append("</td>\r\n");

                    jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                    jcTableHtml.append(object.getJSONObject("E23").getString("nodeText") + " " + object.getJSONObject("E24").getString("nodeText"));
                    jcTableHtml.append("</td>\r\n");

                    jcTableHtml.append("		</tr>\r\n");

                }
                jcTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
                emrContent.setContentText(StaticParams.HTML.replaceAll("\\$\\{yzTableHtml\\}", Matcher.quoteReplacement(jcTableHtml.toString())));
            }
        } catch (Exception e) {
            LOGGER.error("组装检查异常! recordId: {}; ExceptionMsg: {}", recordId, ExceptionUtils.getStackTrace(e));
            return null;
        }
        return emrContent;
    }

    /**
     * 组装检验
     *
     * @param
     * @return EmrContent
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/25
     */
    private EmrContent assembleProof(String recordId) throws Exception {
        EmrContent emrContent = null;
        //检验
        try {
            List<String> tests = Arrays.asList(StaticParams.TEST_TYPES.split(","));
            JSONArray resultJSONArray = new JSONArray();
            for (String s : tests) {
                JSONArray examination = findExamination(StaticParams.JYJCBG_CODE, StaticParams.JYJCBG_PARAM, s, "", recordId);
                if (!CollectionUtils.isEmpty(examination)) resultJSONArray.addAll(examination);
            }

            JSONArray details = findExaminationDetail(resultJSONArray, recordId);
            if (!CollectionUtils.isEmpty(details)) {
                emrContent = new EmrContent();
                emrContent.isAutoJson = false;
                emrContent.setContentId("jy_" + recordId);
                emrContent.setTitle("检验报告单");
                emrContent.setRecordId(recordId);
                emrContent.setCreateBy("admin");
                emrContent.setCreateTime(Calendar.getInstance().getTime());
                emrContent.setEmrId("emr06");

                emrContent.setShowOrder("1");
                emrContent.setUpdateBy("admin");
                emrContent.setUpdateTime(Calendar.getInstance().getTime());

                StringBuffer jyTableHtml = new StringBuffer();
                jyTableHtml.append("<div class=\"fixed-table-body\">\r\n");
                jyTableHtml.append("	<table id=\"bootstrap-table\" class=\"table table-hover\">\r\n" + "		<thead>\r\n");
                jyTableHtml.append("			<tr>\r\n");
                jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验项目</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
                jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验子项</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
                jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验结果</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
                jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>参考值</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
                jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>单位</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
                jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>审核医生</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");
                jyTableHtml.append("				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div class=\"th-inner \" nowrap>检验日期时间</div>\r\n					<div class=\"fht-cell\"></div></th>\r\n");


                JSONArray detailJSONArray = new JSONArray();
                jyTableHtml.append("			</tr>\r\n		</thead>\r\n		<tbody>\r\n");
                for (int i = 0; i < details.size(); i++) {
                    JSONObject object = details.getJSONObject(i);

                    JSONArray sections = new JSONArray();
                    if (object.get("section") instanceof JSONObject) {
                        sections.add(object.getJSONObject("section"));
                    }
                    if (object.get("section") instanceof JSONArray) {
                        sections.addAll(object.getJSONArray("section"));
                    }
                    detailJSONArray.addAll(sections);
                    for (int j = 0; j < sections.size(); j++) {
                        JSONObject section = sections.getJSONObject(j);
                        jyTableHtml.append("			<tr data-index=\"" + j + "\">\r\n");
                        if (j == 0) {
                            jyTableHtml.append("				<td rowspan=\"" + sections.size() + "\" style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                            jyTableHtml.append(mapNullValueReplace(object.getJSONObject("E02").getString("nodeText")));
                            jyTableHtml.append("</td>\r\n");
                        }

                        jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                        jyTableHtml.append(mapNullValueReplace(section.getJSONObject("E02").getString("nodeText")));
                        jyTableHtml.append("</td>\r\n");

                        jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                        jyTableHtml.append(mapNullValueReplace(section.getJSONObject("E03").getString("nodeText")));
                        jyTableHtml.append("</td>\r\n");

                        jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                        jyTableHtml.append(mapNullValueReplace(section.getJSONObject("E06").getString("nodeText")));
                        jyTableHtml.append("</td>\r\n");

                        jyTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                        jyTableHtml.append(mapNullValueReplace(section.getJSONObject("E04").getString("nodeText")));
                        jyTableHtml.append("</td>\r\n");

                        if (j == 0) {
                            jyTableHtml.append("				<td rowspan=\"" + sections.size() + "\" style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                            jyTableHtml.append(mapNullValueReplace(object.getJSONObject("E20").getString("nodeText")));
                            jyTableHtml.append("</td>\r\n");

                            jyTableHtml.append("				<td rowspan=\"" + sections.size() + "\" style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                            jyTableHtml.append(object.getJSONObject("E03").getString("nodeText") + " " + object.getJSONObject("E04").getString("nodeText"));
                            jyTableHtml.append("</td>\r\n");
                        }

                        jyTableHtml.append("			</tr>");
                    }

                }
                jyTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");

                emrContent.setContentText(StaticParams.HTML.replaceAll("\\$\\{yzTableHtml\\}", Matcher.quoteReplacement(jyTableHtml.toString())));
                JSONObject structJson = new JSONObject();
                structJson.put("datas", detailJSONArray);
                emrContent.setStructJson(structJson.toJSONString());

            }

        } catch (Exception e) {
            LOGGER.error("组装检验异常! recordId: {}; ExceptionMsg: {}", recordId, ExceptionUtils.getStackTrace(e));
            return null;
        }
        return emrContent;
    }

    /**
     * 获取检验检查报告
     *
     * @param
     * @return LinkedHashMap
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/25
     */
    private JSONArray findExamination(String code, String param, String type, String documentId, String recordId) throws Exception {
        String response = "";
        JSONArray resultArray = new JSONArray();
        try {
            if (StringUtils.isNotBlank(documentId)) {
                response = doPostSoap(WEBSERVICE_URL, StaticParams.JYJCMX_CODE, String.format(param, recordId, type, documentId), WEBSERVICE_SOAPACTION);
                response = response.replaceFirst(CDATA_START, "").replace("]]]]><![CDATA[>", "");
            } else {
                response = doPostSoap(WEBSERVICE_URL, code, String.format(param, recordId, type), WEBSERVICE_SOAPACTION);
            }
            JSONObject jsonObject = XmlTool.documentToJSONObject(response);
            JSONObject body1 = jsonObject.getJSONObject("Body");
            JSONObject GetHISInfoResponse = body1.getJSONObject("GetHISInfoResponse");
            if (null == GetHISInfoResponse || !GetHISInfoResponse.containsKey("GetHISInfoResult")) return resultArray;
            JSONObject GetHISInfoResult = GetHISInfoResponse.getJSONObject("GetHISInfoResult");

            if (!CollectionUtils.isEmpty(GetHISInfoResponse) && !CollectionUtils.isEmpty(GetHISInfoResult)) {
                JSONObject response1 = GetHISInfoResult.getJSONObject("Response");
                //Body DocumentRetrievalRp Documents
                if (response1.containsKey("Body")) {
                    JSONObject body = response1.getJSONObject("Body");
                    String rpKey = "";
                    if (StringUtils.isNotBlank(documentId)) {
                        rpKey = "DocumentSearchRp";
                    } else {
                        rpKey = "DocumentRetrievalRp";
                    }
                    if (null == body || !body.containsKey(rpKey)) return resultArray;

                    JSONObject documentRetrievalRp = body.getJSONObject(rpKey);
                    if (null == documentRetrievalRp || !documentRetrievalRp.containsKey("Documents"))
                        return resultArray;

                    JSONObject documents = documentRetrievalRp.getJSONObject("Documents");
                    if (null == documents || !documents.containsKey("Document")) return resultArray;

                    if (documents.get("Document") instanceof JSONObject) {
                        JSONObject document = documents.getJSONObject("Document");
                        resultArray.add(document);
                    }
                    if (documents.get("Document") instanceof JSONArray) {
                        resultArray = documents.getJSONArray("Document");
                    }
                    return resultArray;
                }
            }
        } catch (Exception e) {
            LOGGER.error("获取检验检查报告! recordId: {}, code: {}, type: {}, documentId: {}; ExceptionMsg: {}", recordId, code, type, documentId, ExceptionUtils.getStackTrace(e));
            return new JSONArray();
        }
        return resultArray;
    }

    /**
     * 获取检验检查报告明细
     *
     * @param
     * @return LinkedHashMap
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/25
     */
    private JSONArray findExaminationDetail(JSONArray resultJSONArray, String recordId) throws Exception {
        //明细结果
        JSONArray resultMxJSONArray = new JSONArray();
        String documentID = "";
        String documentType = "";
        try {
            //明细
            JSONArray resultMxs = new JSONArray();
            if (!CollectionUtils.isEmpty(resultJSONArray)) {
                for (Object o : resultJSONArray) {
                    JSONObject jsonObject = (JSONObject) o;
                    documentID = jsonObject.getString("DocumentID");
                    documentType = jsonObject.getString("DocumentType");
                    JSONArray examination = findExamination(StaticParams.JYJCMX_CODE, StaticParams.JYJCMX_PARAM, documentType, documentID, recordId);
                    if (!CollectionUtils.isEmpty(examination)) resultMxs.addAll(examination);
                }
            }

            if (!CollectionUtils.isEmpty(resultMxs)) {
                for (Object o : resultMxs) {
                    JSONObject jsonObject = (JSONObject) o;
                    String documentContent = jsonObject.getString("DocumentContent");
                    documentID = jsonObject.getString("DocumentID");

                    EncryptUtil instance = EncryptUtil.getInstance();
                    String content = instance.Base64Decode(documentContent);

                    JSONObject documentToJSONObject = XmlTool.documentToJSONObject(content);
                    JSONObject structuredBody = documentToJSONObject.getJSONObject("structuredBody");
                    if (null == structuredBody) {
                        LOGGER.warn("病历: {},报告ID: {}, 返回数据结构异常, 无[structuredBody]节点。", recordId, documentID);
                    }
                    if (!structuredBody.containsKey("section")) {
                        LOGGER.warn("病历: {},报告ID: {}, 返回数据结构异常, [section]节点数据异常。", recordId, documentID);
                    }
                    JSONArray sections = new JSONArray();
                    if (structuredBody.get("section") instanceof JSONArray) {
                        sections = structuredBody.getJSONArray("section");
                    }
                    if (structuredBody.get("section") instanceof JSONObject) {
                        sections.add(structuredBody.getJSONObject("section"));
                    }
                    resultMxJSONArray.add(sections.get(1));
                }
            }
        } catch (Exception e) {
            LOGGER.error("获取检验检查报告明细! recordId: {}, 报告ID：{}， 类型：{}; ExceptionMsg: {}", recordId, documentID, documentType, ExceptionUtils.getStackTrace(e));
            return new JSONArray();
        }
        return resultMxJSONArray;
    }

    /**
     * 组装安全核查表
     *
     * @param
     * @return EmrContent
     * @author ouyang0810@foxmail.com
     * @Date 2022/10/25
     */
    private EmrContent assembleSafetyChecklist(String code, String recordId) throws Exception {
        EmrContent scEmrContent = null;
        Element GetHISInfoResult = findGetHISInfoResult(code, recordId);
        if (null != GetHISInfoResult) {
            Element anData = GetHISInfoResult.element("AnData");
            if (null != anData) {
                Element row = anData.element("Row");
                if (null != row) {
                    scEmrContent = new EmrContent();
                    String contentId = row.elementText("content_id");
                    String title = row.elementText("title");
                    Element contentXml = row.element("content_xml");
                    if (null != contentXml) {
                        scEmrContent.setContentXml(contentXml.asXML());
                    }
                    scEmrContent.setContentId(contentId);
                    scEmrContent.setTitle(title);
                    scEmrContent.setEmrId("emr07");
                    scEmrContent.setShowOrder(row.elementText("show_order"));
                    String contentTime = row.elementText("content_time");
                    scEmrContent.setContentTime(DateUtils.stringToDate(contentTime, DateUtils.DATE_TIME_FORMAT));
                    scEmrContent.setRecordId(row.elementText("record_id"));
                }
            }
        }
        return scEmrContent;
    }

    /**
     * map null值转换
     *
     * @return void
     */
    private static String mapNullValueReplace(Object value) {
        return null == value ? "" : String.valueOf(value);
    }


    /**
     * 使用SOAP发送消息（HttpClient方式）
     *
     * @param postUrl 请求地址
     * @return
     */
    private static String doPostSoap(String postUrl, String requestCode, String requestParam, String soapAction) {
        int socketTimeout = 60000;// 请求超时时间
        int connectTimeout = 60000;// 传输超时时间
        String retStr = "";
        // 创建HttpClientBuilder
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        try {
            //忽略SSL证书(https时需要证书)
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, (arg0, arg1) -> true).build();
            httpClientBuilder.setSSLContext(sslContext);
            httpClientBuilder.setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE);
        } catch (KeyManagementException | NoSuchAlgorithmException | KeyStoreException e) {
            throw new RuntimeException("Could not disable SSL on HttpClient builder.", e);
        }
        // HttpClient
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httpPost = new HttpPost(postUrl);
        //  设置请求和传输超时时间
        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(socketTimeout)
                .setConnectTimeout(connectTimeout).build();
        httpPost.setConfig(requestConfig);
        try {
            httpPost.setHeader("Content-Type", "text/xml;charset=UTF-8");
            httpPost.setHeader("SOAPAction", soapAction);
            StringEntity data = new StringEntity(String.format(WEBSERVICE_XML, requestCode, requestParam), Charset.forName("UTF-8"));
            httpPost.setEntity(data);
            CloseableHttpResponse response = closeableHttpClient
                    .execute(httpPost);
            HttpEntity httpEntity = response.getEntity();
            if (httpEntity != null) {
                // 打印响应内容
                retStr = EntityUtils.toString(httpEntity, "UTF-8");
                //LOGGER.info("response:" + retStr);
            }
            // 释放资源
            closeableHttpClient.close();
        } catch (Exception e) {
            LOGGER.error("文书编号：{}；获取文书请求出错！ 异常信息：{}", e.getMessage());
        }
//        if (retStr.contains(CDATA_START.replaceAll("\\\\", ""))) {
//            retStr = retStr.replaceAll(CDATA_START, "");
//        }
//        if (retStr.contains("]]>")) {
//            retStr = retStr.replaceAll("]]>", "");
//        }
//        if (retStr.contains("<content_html>")) {
//            retStr = retStr.replaceAll("<content_html>", "<content_html><![CDATA[");
//            retStr = retStr.replaceAll("</content_html>", "]]></content_html>");
//        }
        return retStr;
    }


    private static String doPostSoapNew(String postUrl, String requestCode, String requestParam, String soapAction) {
        int socketTimeout = 60000;// 请求超时时间
        int connectTimeout = 60000;// 传输超时时间
        String retStr = "";
        // 创建HttpClientBuilder
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        try {
            //忽略SSL证书(https时需要证书)
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, (arg0, arg1) -> true).build();
            httpClientBuilder.setSSLContext(sslContext);
            httpClientBuilder.setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE);
        } catch (KeyManagementException | NoSuchAlgorithmException | KeyStoreException e) {
            throw new RuntimeException("Could not disable SSL on HttpClient builder.", e);
        }
        // HttpClient
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httpPost = new HttpPost(postUrl);
        //  设置请求和传输超时时间
        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(socketTimeout)
                .setConnectTimeout(connectTimeout).build();
        httpPost.setConfig(requestConfig);
        try {
            httpPost.setHeader("Content-Type", "text/xml;charset=UTF-8");
            httpPost.setHeader("SOAPAction", soapAction);
            StringEntity data = new StringEntity(String.format(WEBSERVICE_XML, requestCode, requestParam), Charset.forName("UTF-8"));
            httpPost.setEntity(data);
            CloseableHttpResponse response = closeableHttpClient
                    .execute(httpPost);
            HttpEntity httpEntity = response.getEntity();
            if (httpEntity != null) {
                // 打印响应内容
                retStr = EntityUtils.toString(httpEntity, "UTF-8");
                //LOGGER.info("response:" + retStr);
            }
            // 释放资源
            closeableHttpClient.close();
        } catch (Exception e) {
            LOGGER.error("文书编号：{}；获取文书请求出错！ 异常信息：{}", e.getMessage());
        }
        if (retStr.contains(CDATA_START.replaceAll("\\\\", ""))) {
            retStr = retStr.replaceFirst(CDATA_START, "");
            int i = retStr.lastIndexOf(CDATA_END);
            retStr = retStr.substring(0, i) + retStr.substring(i + CDATA_END.length(), retStr.length());
        }
        if (retStr.contains("<content_html>")) {
            retStr = retStr.replaceAll("<content_html>", "<content_html><![CDATA[");
            retStr = retStr.replaceAll("</content_html>", "]]></content_html>");
        }
        return retStr;
    }

    @Override
    public List<EmrContent> selectEmrContentList(String contentId, EmrRecord record, List<String> logMsg) throws Exception {
        return null;
    }

    public static void main(String[] args) throws Exception {
        String filePath = "C:/Users/Administrator/Desktop/emrFile/";
        String ryjr = "MES0006根据就诊号获取入院记录.txt";
        String bcjl = "MES0002根据就诊号获取患者病程记录.txt";
        File file = new File(filePath + bcjl);
        StringBuffer stringBuffer = new StringBuffer();
        Scanner scanner = new Scanner(file);
        while (scanner.hasNextLine()) {
            String line = scanner.nextLine();
            stringBuffer.append(line);
        }
        scanner.close();

        String retStr = stringBuffer.toString();

        if (retStr.contains(CDATA_START.replaceAll("\\\\", ""))) {
            retStr = retStr.replaceFirst(CDATA_START, "");
            int i = retStr.lastIndexOf(CDATA_END);
            retStr = retStr.substring(0, i) + retStr.substring(i + CDATA_END.length(), retStr.length());
        }
        if (retStr.contains("<content_html>")) {
            retStr = retStr.replaceAll("<content_html>", "<content_html><![CDATA[");
            retStr = retStr.replaceAll("</content_html>", "]]></content_html>");
        }
        Document document = XmlTool.strToDocument(retStr);
        Element rootElement = document.getRootElement();
        Element GetHISInfoResponse = rootElement.element("Body").element("GetHISInfoResponse");
        Element getHISInfoResult = GetHISInfoResponse.element("GetHISInfoResult");
        LinkedList<EmrContent> emrContents = Lists.newLinkedList();
        try {
            if (null != getHISInfoResult) {
                Element root = getHISInfoResult.element("root");
                if (null != root) {
                    List<Element> elements = root.elements();
                    for (Element element : elements) {
                        EmrContent content = new EmrContent();
                        String contentId = element.elementText("content_id");
                        contentId = contentId.replaceAll("\\|\\|", "_");
                        content.setContentId(contentId);
                        content.setTitle(element.elementText("title"));
                        content.setEmrId(element.elementText("emr_id"));
                        content.setShowUrl(element.elementText("content_html").replaceFirst(CDATA_START, "").replaceAll(CDATA_END, ""));
                        content.setShowOrder(element.elementText("show_order"));
                        String contentTime = element.elementText("content_time");
                        content.setContentTime(DateUtils.stringToDate(contentTime, DateUtils.DATE_TIME_FORMAT));
                        content.setRecordId(element.elementText("record_id"));
                        try {
                            Element NInstanceData = element.element("content_xml").element("NInstanceData");
                            if (null == NInstanceData) {
                                NInstanceData = element.element("content_xml").element("Document").element("StructuredBody").element("NInstanceData");
                            }
                            if (null != NInstanceData) {
                                String xml = "";
                                Element abiword = NInstanceData.element("abiword");
                                if (null != abiword) {
                                    List<Element> abiwords = abiword.elements();
                                    for (Element e : abiwords) {
                                        if ("section".equals(e.getName())) {
                                            //仅取第一个 section
                                            Element data = abiword.element("data");
                                            if (null != data) {
                                                Element data1 = e.addElement("data");
                                                List<Element> elementsB = data.elements("d");
                                                Element elB = elementsB.get(0);
                                                data1.setText("data:image/jpeg;base64," + elB.getText());
                                            }
                                            xml = e.asXML();
                                            if (xml.contains("xmlns")) {
                                                xml = xml.replace("xmlns=\"http://www.abisource.com/awml.dtd\"", "");
                                            }
                                            content.setContentXml(xml);
                                            break;
                                        }
                                    }
                                } else {
                                    Element iEditor = NInstanceData.element("iEditor");
                                    if (null != iEditor) {
                                        xml = xml + iEditor.asXML();
                                        if (xml.contains("xmlns")) {
                                            xml = xml.replace("xmlns=\"http://www.abisource.com/awml.dtd\"", "");
                                        }
                                        content.setContentXml(xml);
                                    }
                                }
                            } else {
                            }
                        } catch (Exception e) {
                            continue;
                        }
                        if (StringUtils.isNotBlank(content.getContentId())) {
                            emrContents.add(content);
                        }
                    }
                }
            }

        } catch (Exception e) {
        }
        int size = emrContents.size();
    }

}
