package com.hanyun.hip.mrqc.jms.api;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import cn.hutool.setting.Setting;
import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.alibaba.druid.util.JdbcUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.hanyun.hip.mrqc.common.EmrETLApi;
import com.hanyun.hip.mrqc.common.PageRequest;
import com.hanyun.hip.mrqc.common.impl.Wx9yEmrETLApi;
import com.hanyun.hip.mrqc.service.constant.ContentConstant;
import com.hanyun.hip.mrqc.service.entity.*;
import com.hanyun.hip.mrqc.service.entity.mdb.MdbObject;
import com.hanyun.hip.mrqc.service.entity.mdb.MdbObjectAttr;
import com.hanyun.hip.mrqc.service.service.IMrqcEtlService;
import com.hanyun.hip.mrqc.service.util.CommonUtil;
import com.hanyun.hip.mrqc.service.util.DateUtils;
import com.hanyun.hip.mrqc.service.util.HtmlUtils;
import oracle.sql.BLOB;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.beanutils.converters.BigDecimalConverter;
import org.apache.commons.beanutils.converters.DateConverter;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.sql.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.*;
import java.util.regex.Matcher;

/**
 * @author yrl
 * @version V1.0
 * @Title: Wx9yEmrETLApi.java
 * @Package com.hanyun.hip.mrqc.common.impl
 * @Description: 无锡九院病历抽提服务
 * @date 2021年1月26日
 */
@Service("Wx9yEmrETLApiNew")
public class Wx9yEmrETLApiNew implements EmrETLApi {

    private static JdbcTemplate wxjyJdbcTemplate = new JdbcTemplate();

    private static JdbcTemplate wxjyBasyJdbcTemplate = new JdbcTemplate();
    private static final Logger LOGGER = LoggerFactory.getLogger(Wx9yEmrETLApi.class);

    static {
        Setting poolSetting = new Setting();
        poolSetting.put("initialSize", "20");
        poolSetting.put("maxActive", "100");
        poolSetting.put("maxWait", "60000");
        poolSetting.put("validationQuery", "SELECT 1 FROM DUAL");
        wxjyJdbcTemplate.setDataSource(createDataSource("jdbc:oracle:thin:@129.1.1.20:1521/dbserver",
                "oracle.jdbc.driver.OracleDriver", "hanyun", "hanyun", poolSetting));

        // 最新无锡九院提交的His病案首页数据库连接
        wxjyBasyJdbcTemplate.setDataSource(createDataSource("jdbc:oracle:thin:@129.1.8.178:1521/dbserver",
                "oracle.jdbc.driver.OracleDriver", "ek_thdhanyun", "ek_thdhanyun123", poolSetting));
    }

    protected static DruidDataSource createDataSource(String jdbcUrl, String driver, String user, String pass,
                                                      Setting poolSetting) {
        final DruidDataSource ds = new DruidDataSource();
        ds.setUrl(jdbcUrl);
        ds.setDriverClassName(driver);
        ds.setUsername(user);
        ds.setPassword(pass);

        // 规范化属性名
        Properties druidProps = new Properties();
        String keyStr;
        for (Map.Entry<String, String> entry : poolSetting.entrySet()) {
            keyStr = StrUtil.addPrefixIfNot(entry.getKey(), "druid.");
            druidProps.put(keyStr, entry.getValue());
        }
        // 连接池信息
        ds.configFromPropety(druidProps);

        // 检查关联配置，在用户未设置某项配置时，
        if (null == ds.getValidationQuery()) {
            // 在validationQuery未设置的情况下，以下三项设置都将无效
            ds.setTestOnBorrow(false);
            ds.setTestOnReturn(false);
            ds.setTestWhileIdle(false);
        }

        return ds;
    }

    List<MrqcDatasource> mrqcDatasourceList = new ArrayList<MrqcDatasource>() {
        private static final long serialVersionUID = 1L;
        {
            add(new MrqcDatasource("EmrRecord", "EmrRecord", "oracle",
                    "oracle.jdbc.driver.OracleDriver",
                    "jdbc:oracle:thin:@129.1.8.178:1521/dbserver", "ek_thdhanyun", "ek_thdhanyun123", "电子病历记录数据源",
                    "EmrRecord",
                    "select HDSD00_11_006||'_'||HDSD00_11_139 as record_id, HDSD00_11_006 as patient_num\n" +
                            "'1001500' as hospit_code,'无锡市第九人民医院' as hospit_name,HDSD00_11_006||HDSD00_11_139 as hospit_num,HDSD00_11_110 as patient_name,\n" +
                            " decode(HDSD00_11_109,'1','男'，'女' )as patient_sex,HDSD00_11_014 as birthday,HDSD00_11_079 as age , '1' as auto,decode(HDSD00_11_019,NULL,'1','4') as emr_status,HDSD00_11_075 as  diagnosis_code,\n" +
                            " HDSD00_11_076 as  diagnosis_name,HDSD00_11_141 as hospit_doctor,  '' as record_score,\n" +
                            "(select ITEMNAME from ek_thdhanyun.his_dict where ITEMCODE =  HDSD00_11_083) as department_name ,\n" +
                            "HDSD00_11_083 AS department_code,(select ITEMNAME from ek_thdhanyun.his_dict where ITEMCODE =  HDSD00_11_084) as inpatient_area,\n" +
                            "HDSD00_11_084 AS inpatient_code,HDSD00_11_019 as hospit_time,HDSD00_11_085 as in_time,HDSD00_11_019 as out_time\n" +
                            "    from ek_thdhanyun.his_ddinp_basy where HDSD00_11_019 is not null",

                    "select HDSD00_11_006||'_'||HDSD00_11_139 as record_id, HDSD00_11_006 as patient_num, \n" +
                            "'1001500' as hospit_code,'无锡市第九人民医院' as hospit_name,HDSD00_11_006||HDSD00_11_139 as hospit_num,HDSD00_11_110 as patient_name,\n" +
                            " decode(HDSD00_11_109,'1','男'，'女' )as patient_sex,HDSD00_11_014 as birthday,HDSD00_11_079 as age , '1' as auto,decode(HDSD00_11_019,NULL,'1','4') as emr_status,HDSD00_11_075 as  diagnosis_code,\n" +
                            " HDSD00_11_076 as  diagnosis_name,HDSD00_11_141 as hospit_doctor,  '' as record_score,\n" +
                            "(select ITEMNAME from ek_thdhanyun.his_dict where ITEMCODE =  HDSD00_11_084) as inpatient_area,\n" +
                            "HDSD00_11_084 AS inpatient_code,(select ITEMNAME from ek_thdhanyun.his_dict where ITEMCODE =  HDSD00_11_083) as department_name,\n" +
                            "HDSD00_11_083 AS department_code,HDSD00_11_019 as hospit_time,HDSD00_11_085 as in_time,HDSD00_11_019 as out_time\n" +
                            "    from ek_thdhanyun.his_ddinp_basy where record_id = ?",
                    ""));
            add(new MrqcDatasource("EmrContent", "EmrContent", "oracle",
                    "oracle.jdbc.driver.OracleDriver",
                    "jdbc:oracle:thin:@129.1.1.103:1521:jhemr", "EMRZK", "EMRZK", "电子病历文书数据源",
                    "EmrContent",
                    "SELECT EFCH.FILE_UNIQUE_ID || '__' || replace(EFCH.mr_code,'.','_') AS content_id, NULL AS emr_id, EFCH.PATIENT_ID || '_' || EFCH.VISIT_ID AS record_id, EFCH.TOPIC AS title, EFCH.MR_CONTENT AS content_text, EFCC.MR_CONTENT AS content_xml, EFCH.FILE_NO AS show_order, EFCH.first_mr_sign_date_time AS content_time FROM EMRZK_FILE_CONTENT_HTM EFCH LEFT JOIN EMRZK_FILE_CONTENT_CDA EFCC ON EFCH.FILE_UNIQUE_ID = EFCC.FILE_UNIQUE_ID",
                    "SELECT EFCH.FILE_UNIQUE_ID || '__' || replace(EFCH.mr_code,'.','_') AS content_id, NULL AS emr_id, EFCH.PATIENT_ID || '_' || EFCH.VISIT_ID AS record_id, EFCH.TOPIC AS title, EFCH.MR_CONTENT AS content_text, EFCC.MR_CONTENT AS content_xml, EFCH.FILE_NO AS show_order, EFCH.first_mr_sign_date_time AS content_time FROM EMRZK_FILE_CONTENT_HTM EFCH LEFT JOIN EMRZK_FILE_CONTENT_CDA EFCC ON EFCH.FILE_UNIQUE_ID = EFCC.FILE_UNIQUE_ID WHERE EFCH.PATIENT_ID = ? and EFCH.VISIT_ID = ?",
                    ""));
            add(new MrqcDatasource("EmrPageRecord", "EmrPageRecord", "oracle",
                    "oracle.jdbc.driver.OracleDriver",
                    "jdbc:oracle:thin:@129.1.8.178:1521/dbserver", "ek_thdhanyun", "ek_thdhanyun123", "电子病历病案首页数据源",
                    "EmrPageRecord",
                    "select * from ek_thdhanyun.his_ddinp_basy ",
                    "select * from ek_thdhanyun.his_ddinp_basy where record_id = ?",
                    ""));
            add(new MrqcDatasource("EmrMedicalRecord", "EmrMedicalRecord", "oracle",
                    "oracle.jdbc.driver.OracleDriver",
                    "jdbc:oracle:thin:@129.1.8.178:1521/dbserver", "ek_thdhanyun", "ek_thdhanyun123", "病程记录--医嘱"
                    , "EmrMedicalRecord"
                    , "SELECT * FROM his_ddinp_order t ORDER BY t.\"医嘱主号\" "
                    , "SELECT * FROM his_ddinp_order t WHERE t.\"住院号\"= ? and t.就诊次数 = ? ORDER BY t.\"医嘱主号\" ",
                    ""));
//            // 检验申请单
//            add(new MrqcDatasource("EmrLisApplyRecord", "EmrLisApplyRecord", "oracle",
//                    "oracle.jdbc.driver.OracleDriver",
//                    "jdbc:oracle:thin:@129.1.1.20:1521/dbserver", "hanyun", "hanyun", "病程记录--检验申请单"
//                    , "EmrLisApplyRecord"
//                    , "select * from hanyun.his_hhlab_req t order by t.申请时间"
//                    , "select * from hanyun.his_hhlab_req t where t.住院号 = ? and t.就诊次数 = ? order by t.申请时间", ""));

//            // 检查申请单
//            add(new MrqcDatasource("EmrPacsApplyRecord", "EmrPacsApplyRecord", "oracle",
//                    "oracle.jdbc.driver.OracleDriver",
//                    "jdbc:oracle:thin:@129.1.1.20:1521/dbserver", "hanyun", "hanyun", "病程记录--检查申请单"
//                    , "EmrPacsApplyRecord"
//                    , "select * from hanyun.his_hhexam_req t order by t.申请时间"
//                    , "select * from hanyun.his_hhexam_req t where t.住院号 = ? and t.就诊次数 = ? order by t.申请时间", ""));

//            // 手术通知单
//            add(new MrqcDatasource("EmrOpeNoticeRecord", "EmrOpeNoticeRecord", "oracle",
//                    "oracle.jdbc.driver.OracleDriver",
//                    "jdbc:oracle:thin:@129.1.1.20:1521/dbserver", "hanyun", "hanyun", "知情同意书--手术通知单"
//                    , "EmrOpeNoticeRecord"
//                    , "select * from hanyun.his_hhinp_operation_schedule t order by t.结束时间"
//                    , "select * from hanyun.his_hhinp_operation_schedule t where t.住院号 = ? and 住院次数 = ? order by t.结束时间", ""));

//            // 会诊记录
//            add(new MrqcDatasource("EmrConsultationRecord", "EmrConsultationRecord", "oracle",
//                    "oracle.jdbc.driver.OracleDriver",
//                    "jdbc:oracle:thin:@129.1.1.20:1521/dbserver", "hanyun", "hanyun", "病程记录--会诊申请"
//                    , "EmrConsultationRecord"
//                    , "SELECT * FROM hanyun.hhapply_diagnosis_record t order by t.会诊时间"
//                    , "SELECT * FROM hanyun.hhapply_diagnosis_record t where t.住院号 = ? and 就诊次数 = ? order by t.会诊时间", ""));

//            add(new MrqcDatasource("EmrNursingRecord", "EmrNursingRecord", "oracle",
//                    "oracle.jdbc.driver.OracleDriver",
//                    "jdbc:oracle:thin:@129.1.1.218:1521:WXJYKYEE", "hanyun", "hanyun", "电子病历护理数据源",
//                    "EmrNursingRecord",
//                    "SELECT * FROM V_KYEE_RECORDS",
//                    "SELECT * FROM V_KYEE_RECORDS v WHERE v.HDSD00_08_095 = ? and v.visit_id = ?",
//                    ""));

            add(new MrqcDatasource("EmrLisRecord", "EmrLisRecord", "oracle",
                    "oracle.jdbc.driver.OracleDriver",
                    "jdbc:oracle:thin:@129.1.1.71:1521/oraclelis", "nis", "admin123", "病程记录--检验",
                    "EmrLisRecord",
                    "SELECT * FROM dbo.hy_lis_result t",
                    "select b.结果, c.* from (SELECT a.报告单号, a.病人编号, wmsys.wm_concat(a.结果) 结果 FROM (SELECT T.*, '</br>' || t.检验结果名称 || ':' || t.检验结果 || '(' || t.检验计量单位 || ')' 结果 FROM dbo.hy_lis_result t) a group by a.报告单号, a.病人编号) b left join (SELECT distinct T.报告单号, t.病人编号, t.就诊次数, t.标本, t.检验项目名称, t.检验人员, t.检验日期 FROM dbo.hy_lis_result t) c on b.报告单号 = c.报告单号 where b.病人编号 = ? and c.就诊次数 = ? order by c.检验日期",
                    ""));

            add(new MrqcDatasource("EmrPacsRecord", "EmrPacsRecord", "sqlserver"
                    , "com.microsoft.sqlserver.jdbc.SQLServerDriver"
                    , "jdbc:sqlserver://129.1.1.2:1433;databasename=eBL", "sa", "eWorldPACS", "病程记录--检查"
                    , "EmrPacsRecord"
                    , "select * from view_1 a order by a.exam_date_time"//-------------------------------------缺少第几次就诊
                    , "select * from view_1 a where a.patient_id = ? and 1 = ? order by a.exam_date_time", ""));

            // 手麻系统
            add(new MrqcDatasource("EmrSurgeryRecord", "EmrSurgeryRecord", "oracle",
                    "oracle.jdbc.driver.OracleDriver",
                    "jdbc:oracle:thin:@129.1.1.213:1521/docare", "jx", "jx", "手术记录--麻醉记录",
                    "EmrSurgeryRecord",
                    "select * from medsurgery.v_mzd",
                    "select * from medsurgery.v_mzd t where t.会员号 = ? and t.就诊次数 = ? ",
                    ""));
        }
    };

    @Override
    public List<EmrRecord> selectEmrRecordList(String hospitalCode, String recordId, String hospitalNum, Date beginDate, Date endDate,
                                               PageRequest pager, List<String> logMsg, Boolean notExtract) throws Exception {
        List<EmrRecord> result = new ArrayList<EmrRecord>();
        if (!CollectionUtils.isEmpty(mrqcDatasourceList)) {
            for (MrqcDatasource mrqcDatasource : mrqcDatasourceList) {

                String emrType = mrqcDatasource.getEmrType();
                String emrSql = mrqcDatasource.getEmrSql();
                String oneEmrSql = mrqcDatasource.getOneEmrSql();
                if ("EmrRecord".equalsIgnoreCase(emrType)) {
                    DruidDataSource druidDataSource = CommonUtil.getDruidDataSource(mrqcDatasource);
                    if (druidDataSource == null) {
                        continue;
                    }

                    List<EmrRecord> temp = null;
                    if (!StringUtils.isEmpty(recordId)) {   // 查询一条记录
                        temp = executeSql(druidDataSource, oneEmrSql, EmrRecord.class, recordId);
                    } else if (beginDate != null && endDate != null) {
                        emrSql = "select * from (" + emrSql + ") a where a.out_time>=? and a.out_time<=?";
                        emrSql = emrSql + " union all \n" +
                                "\tselect HDSD00_11_006||'_'||HDSD00_11_139 as record_id, HDSD00_11_006 as patient_num, RECORD_ID as hisRecordId,\n" +
                                "'1001500' as hospit_code,'无锡市第九人民医院' as hospit_name,HDSD00_11_006||HDSD00_11_139 as hospit_num,HDSD00_11_110 as patient_name,\n" +
                                " decode(HDSD00_11_109,'1','男'，'女' )as patient_sex,HDSD00_11_014 as birthday,HDSD00_11_079 as age , '1' as auto,1 as emr_status,HDSD00_11_075 as  diagnosis_code,\n" +
                                " HDSD00_11_076 as  diagnosis_name,HDSD00_11_141 as hospit_doctor,  '' as record_score,\n" +
                                "(select ITEMNAME from ek_thdhanyun.his_dict where  ITEMCODE =  HDSD00_11_084) as inpatient_area,\n" +
                                "HDSD00_11_084 AS inpatient_code,(select ITEMNAME from ek_thdhanyun.his_dict where ITEMCODE = HDSD00_11_083) as department_name,\n" +
                                "HDSD00_11_083 AS department_code,HDSD00_11_019 as hospit_time,HDSD00_11_085 as in_time,HDSD00_11_019 as out_time\n" +
                                "from ek_thdhanyun.his_ddinp_basy where HDSD00_11_019 is null";
                        temp = executeSqlByPage(druidDataSource, emrSql, EmrRecord.class, " order by out_time ",
                                pager, beginDate, endDate);
                    } else if (beginDate != null) {
                        emrSql = "select * from (" + emrSql + ") a where a.out_time>=?";
                        temp = executeSqlByPage(druidDataSource, emrSql, EmrRecord.class, " order by out_time ",
                                pager, beginDate);
                    } else if (endDate != null) {
                        emrSql = "select * from (" + emrSql + ") a where a.out_time<=?";
                        temp = executeSqlByPage(druidDataSource, emrSql, EmrRecord.class, " order by out_time ",
                                pager, endDate);
                    } else { // 查询全部记录
                        temp = executeSqlByPage(druidDataSource, emrSql, EmrRecord.class, " order by out_time ",
                                pager);
                    }
                    if (!CollectionUtils.isEmpty(temp)) {
                        result.addAll(temp);
                    }
                }
            }
        }
        return result;
    }

    // 根据结束时间获取一天的病例数据
    @Override
    public Date getDefaultBeginDate(String hospitalCode) {
        Date endDate = getDefaultEndDate();
        if (endDate != null) {
            try {
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(new Date());
                calendar.add(Calendar.DATE, -2);
                String dateStr = sdfTime.format(calendar.getTime());
                return sdfTime.parse(dateStr);
            } catch (Exception ignore) {
                return null;
            }
        }

        return null;
    }

    @Override
    public List<EmrContent> selectEmrContentList(EmrRecord record, List<String> logMsg) throws Exception {
        // 一进来就设置不自动转换JSON
        EmrContent.isAutoJson = false;
        List<EmrContent> result = new ArrayList<EmrContent>();
        if (!CollectionUtils.isEmpty(mrqcDatasourceList)) {
            for (MrqcDatasource mrqcDatasource : mrqcDatasourceList) {
                DruidDataSource druidDataSource = CommonUtil.getDruidDataSource(mrqcDatasource);
                if (druidDataSource == null) {
                    continue;
                }

                String emrType = mrqcDatasource.getEmrType();
                String oneEmrSql = mrqcDatasource.getOneEmrSql();
                String recordId = record.getRecordId();
                if ("EmrContent".equalsIgnoreCase(emrType)) { // 获取病历文书
                    String[] params = recordId.split("_");
                    List<EmrContent> temp = executeSql(druidDataSource, oneEmrSql, EmrContent.class, params[0], params[1]);
                    if (!CollectionUtils.isEmpty(temp)) {
                        result.addAll(temp);
                    }
                } else if ("EmrPageRecord".equalsIgnoreCase(emrType)) {  // 获取病案首页文书
                    // 病案首页进行了升级，所以这里recordId，用字段searchValue字段代替
                    List<EmrContent> temp = extractEmrPageRecord(druidDataSource, oneEmrSql, recordId);
                    if (!CollectionUtils.isEmpty(temp)) {
                        result.addAll(temp);
                    }
//                } else if ("EmrNursingRecord".equalsIgnoreCase(emrType)) {  // 获取护理文书
//                    List<EmrContent> temp = extractEmrNursingRecord(druidDataSource, oneEmrSql, recordId);
//                    if (!CollectionUtils.isEmpty(temp)) {
//                        result.addAll(temp);
//                    }
                } else if ("EmrMedicalRecord".equalsIgnoreCase(emrType)) { // 获取医嘱
                    List<EmrContent> temp = extractEmrMedicalRecord(druidDataSource, oneEmrSql, recordId);
                    if (!CollectionUtils.isEmpty(temp)) {
                        result.addAll(temp);
                    }
                } else if ("EmrLisRecord".equalsIgnoreCase(emrType)) { // 获取检验报告
                    List<EmrContent> temp = extractEmrLISRecord(druidDataSource, oneEmrSql, recordId);
                    if (!CollectionUtils.isEmpty(temp)) {
                        result.addAll(temp);
                    }
                } else if ("EmrPacsRecord".equalsIgnoreCase(emrType)) { // 获取检查报告
                    List<EmrContent> temp = extractEmrPacsRecord(druidDataSource, oneEmrSql, recordId);
                    if (!CollectionUtils.isEmpty(temp)) {
                        result.addAll(temp);
                    }
//                } else if ("EmrLisApplyRecord".equalsIgnoreCase(emrType)) { // 获取检验申请单
//                    List<EmrContent> temp = extractEmrLisApplyRecord(druidDataSource, oneEmrSql, recordId);
//                    if (!CollectionUtils.isEmpty(temp)) {
//                        result.addAll(temp);
//                    }
//                } else if ("EmrPacsApplyRecord".equalsIgnoreCase(emrType)) { // 获取检查申请单
//                    List<EmrContent> temp = extractEmrPacsApplyRecord(druidDataSource, oneEmrSql, recordId);
//                    if (!CollectionUtils.isEmpty(temp)) {
//                        result.addAll(temp);
//                    }
//                } else if ("EmrOpeNoticeRecord".equalsIgnoreCase(emrType)) { // 获取手术通知单
//                    List<EmrContent> temp = extractEmrOpeNoticeRecord(druidDataSource, oneEmrSql, recordId);
//                    if (!CollectionUtils.isEmpty(temp)) {
//                        result.addAll(temp);
//                    }
//                } else if ("EmrConsultationRecord".equalsIgnoreCase(emrType)) { // 获取会诊申请
//                    List<EmrContent> temp = extractEmrConsultationRecord(druidDataSource, oneEmrSql, recordId);
//                    if (!CollectionUtils.isEmpty(temp)) {
//                        result.addAll(temp);
//                    }
                } else if ("EmrSurgeryRecord".equalsIgnoreCase(emrType)) { // 获取手麻系统数据
                    // 手麻系统对接
                    List<EmrContent> smlist = getSmEmrContentList(druidDataSource, oneEmrSql, record);
                    if (!CollectionUtils.isEmpty(smlist)) {
                        result.addAll(smlist);
                    }
                }
            }
        }
        LOGGER.info("emrrecord" + JSONUtil.toJsonStr(record));
        // 重症系统对接
        List<EmrContent> zzlist = getZzEmrContentList(record);
        if (!CollectionUtils.isEmpty(zzlist)) {
            result.addAll(zzlist);
        }



        return result;
    }

    /**
     * 手麻系统接口
     * @param record
     * @return
     */
    public static List<EmrContent> getSmEmrContentList(DruidDataSource druidDataSource, String emrSql, EmrRecord record) {
        List<EmrContent> result = new ArrayList<>();

        CloseableHttpClient httpclient = HttpClients.createDefault();
        String hisVisitId = record.getHospitNum().replace(record.getPatientNum(),"");
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        HttpPost post = new HttpPost("http://129.1.1.213/EmrView/Emr/GetEmrList?MrClass=麻醉&PatientID=" + record.getPatientNum()
                + "&VisitID=" + hisVisitId + "&StrStartDate=&StrEndDate=");

        try {
            CloseableHttpResponse httpResponse = httpclient.execute(post);
            int statusCode = httpResponse.getStatusLine().getStatusCode();
            HttpEntity resEntity = httpResponse.getEntity();
            if (resEntity != null && 200 == statusCode) {
                String str = EntityUtils.toString(resEntity, "UTF-8");
                JSONObject json1 = JSON.parseObject(str);
                Object resValue = json1.get("resValue");
                if(null != resValue){
                    JSONArray arry = JSONArray.parseArray(resValue.toString());
                    // 手麻接口返回的文书列表
                    List<Map<String, Object>> resultList = (List) arry;
                    int j = 0;
                    for(Map<String, Object> tempMap : resultList){
                        String archiveDate = tempMap.get("ARCHIVE_DATE").toString();
                        String fileName = tempMap.get("FileNAME").toString();
                        String path = tempMap.get("PATH").toString();

                        EmrContent emrContent = new EmrContent();
                        emrContent.setContentId("sm_" + record.getRecordId() + j);
                        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd");
                        emrContent.setContentTime(sdf.parse(archiveDate));
                        emrContent.setTitle(fileName);
                        emrContent.setRecordId(record.getRecordId());
                        emrContent.setCreateBy("admin");
                        emrContent.setCreateTime(sdf.parse(archiveDate));
                        emrContent.setShowOrder("1");
                        emrContent.setUpdateBy("admin");
                        emrContent.setUpdateTime(sdf.parse(archiveDate));
                        if(fileName.contains("同意书")){
                            emrContent.setEmrId("emr05");
                        } else if(fileName.contains("麻醉记录单") || fileName.contains("访视")) {
                            emrContent.setEmrId("emr07");

                            // 麻醉记录StructJson，其他手麻系统的文书没有StructJson
                            if(fileName.contains("麻醉记录单")){
                                Map<String, List<Map<String, Object>>> map = Maps.newHashMap();
                                // 查询麻醉记录
                                List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, record.getRecordId());
                                if(null != allColumnNameAndValue && allColumnNameAndValue.size() > 0){
                                    for(int idx = 0; idx < allColumnNameAndValue.size(); idx++){
                                        // 查询麻醉记录对应的备注栏数据
                                        Map<String, Object> mzjl = allColumnNameAndValue.get(idx);
                                        emrSql = "select * from med_anesthesia_event t where t.patient_id = ? and t.visit_id = ?";
                                        if(null != mzjl.get("手术ID")){
                                            emrSql = emrSql + " and t.oper_id = '" + mzjl.get("手术ID") + "'";
                                        }
                                        List<Map<String, Object>> bzList = getALLColumnNameAndValue(druidDataSource, emrSql, record.getRecordId());
                                        if(null != bzList && bzList.size() > 0){
                                            mzjl.put("bzContent", bzList);
                                        }
                                    }
                                }

                                map.put("datas", allColumnNameAndValue);
                                emrContent.setStructJson(JSON.toJSONString(map));
                            }
                        } else {
                            emrContent.setEmrId("emr06");
                        }

                        String htmlHead = ContentConstant.HTML_HEAD;
                        StringBuffer jyTableHtml = new StringBuffer();
                        String uri = "http://129.1.1.213/EmrView/View?FilePath=" + path + "&dateTime=1623033444000";
                        jyTableHtml.append("<iframe id='' src='" + uri + "' style='width:100%;height:100%;'></iframe>");
                        emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jyTableHtml.toString())));
                        result.add(emrContent);

                        j++;
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 重症系统接口调用
     * @param record
     * @return
     */
    public static List<EmrContent> getZzEmrContentList(EmrRecord record) {
        List<EmrContent> result = new ArrayList<>();
        CloseableHttpClient httpclient = HttpClients.createDefault();
        // String hisVisitId = record.getHospitNum().replace(record.getPatientNum(),"");
        String[] split = record.getRecordId().split("_");
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        HttpGet httpget = new HttpGet("http://129.1.1.220:8089/api/WorkStation/Document/GetDocumentDataByHisPatientId?hisPatientId=" + split[0]+"_"+split[1]
                + "&hisVisitId=" + split[1] + "&documentCode=CARE_DOC_WXJYF&startTime=" + format.format(record.getInTime()) + "&endTime=" + format.format(null == record.getOutTime()? new Date() : record.getOutTime()));
//        HttpGet httpget = new HttpGet("http://129.1.1.220:8089/api/WorkStation/Document/GetDocumentDataByHisPatientId?hisPatientId=202112733&hisVisitId=1&documentCode=CARE_DOC_WXJYF&startTime=2021-06-06&endTime=2021-06-07");

        try {
            CloseableHttpResponse httpResponse = httpclient.execute(httpget);
            int statusCode = httpResponse.getStatusLine().getStatusCode();
            HttpEntity resEntity = httpResponse.getEntity();
            if (resEntity != null && 200 == statusCode) {
                String str = EntityUtils.toString(resEntity, "UTF-8");
                JSONObject json1 = JSON.parseObject(str);
                if(null != json1.get("RetObj")){
                    String retObj = json1.get("RetObj").toString();
                    JSONObject jsonRetObj = JSON.parseObject(retObj);
                    Object NursingMemoList = jsonRetObj.get("NursingMemoList");
                    if(null != NursingMemoList){
                        JSONArray arry = JSONArray.parseArray(NursingMemoList.toString());
                        List<Map<String, Object>> nursingList = (List) arry;

                        EmrContent emrContent = new EmrContent();
                        emrContent.setContentId("zz_" + record.getRecordId());
                        Date date = Calendar.getInstance().getTime();
                        emrContent.setContentTime(date);
                        emrContent.setTitle("病重（病危）护理记录");
                        emrContent.setRecordId(record.getRecordId());
                        emrContent.setCreateBy("admin");
                        emrContent.setCreateTime(date);
                        emrContent.setEmrId("emr06");
                        emrContent.setShowOrder("1");
                        emrContent.setUpdateBy("admin");
                        emrContent.setUpdateTime(date);
                        Map<String, List<Map<String, Object>>> map = Maps.newHashMap();
                        map.put("datas", nursingList);
                        emrContent.setStructJson(JSON.toJSONString(map));

                        String htmlHead = ContentConstant.HTML_HEAD;
                        StringBuffer jyTableHtml = new StringBuffer(ContentConstant.EMRZZRECORD_HTML_TITLE);
                        int j = 0;
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
                        for (Map<String, Object> tempMap : nursingList) {
                            jyTableHtml.append( "			<tr data-index=\"" + j + "\">\r\n");
                            jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                            jyTableHtml.append( tempMap.get("TimePoint").toString().substring(0,10) + "</td>\r\n");
                            jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                            jyTableHtml.append( tempMap.get("TimePoint").toString().substring(11,16) + "</td>\r\n");
                            jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                            jyTableHtml.append( tempMap.get("NursingDesc")+ "</td>\r\n");
                            jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                            jyTableHtml.append( tempMap.get("CreatorName") + "</td>\r\n" + "			</tr>");
                            j++;
                        }
                        jyTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
                        emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jyTableHtml.toString())));
                        result.add(emrContent);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 组装护理记录单
     *
     * @param druidDataSource
     * @param oneEmrSql
     * @param recordId
     * @return
     */
    private List<EmrContent> extractEmrNursingRecord(DruidDataSource druidDataSource, String oneEmrSql, String recordId) {
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, oneEmrSql, recordId);
//        LOGGER.info("抽取护理记录单: 记录编号 = {}, 护理记录条数 = {}", recordId, allColumnNameAndValue.size());
        List<EmrContent> result = null;

        if (!CollectionUtils.isEmpty(allColumnNameAndValue)) {
            result = new ArrayList<>();
            Map<String, String> emrContentMap = Maps.newHashMap();

            for ( int k = 0; k < allColumnNameAndValue.size(); k++) {
                Map<String, Object> map = allColumnNameAndValue.get(k);

                StringBuilder htmlTr = new StringBuilder();
                htmlTr.append("<tr data-index=\"" + k + "\">\r\n");
                htmlTr.append("	<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("HDSD00_08_052")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("HDSD00_08_075")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("HDSD00_08_041")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("HDSD00_08_025")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("血压")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("HDSD00_08_081")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("意识")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("HDSD00_08_080")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("伤口敷料外观")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("瞳孔大小左")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("瞳孔大小右")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("瞳孔对光反应左")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("瞳孔对光反应右")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("入量名称")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("入量量")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("出量名称")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("出量量")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("管道名称")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("管道量")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("病情观察与护理")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("HDSD00_08_033")) + "</td>\r\n");
                htmlTr.append("</tr>\r\n");

                String title = map.get("THEME_CODE").toString();
                if (emrContentMap.containsKey(title)) {
                    String contentText = emrContentMap.get(title) + htmlTr.toString();
                    emrContentMap.put(title, contentText);
                } else {
                    emrContentMap.put(title, htmlTr.toString());
                }
            }

            String htmlHead = ContentConstant.HTML_HEAD;
            String jcTableHtml = ContentConstant.NURSING_RECORDS_HTML_BODY;
            int j = 1;
            for (Map.Entry<String, String> entry : emrContentMap.entrySet()) {
                EmrContent content = new EmrContent();
                Date date = new Date();
                content.setContentId("hljld_" + recordId + '_' + j);
                content.setRecordId(recordId);
                content.setContentTime(date);
                content.setCreateBy("admin");
                content.setCreateTime(date);
                content.setEmrId("emr06");
                content.setShowOrder("1");
                content.setUpdateBy("admin");
                content.setUpdateTime(date);
                Map<String, List<Map<String, Object>>> maps = Maps.newHashMap();
                maps.put("datas", allColumnNameAndValue);
                content.setStructJson(JSON.toJSONString(maps));
                content.setTitle(entry.getKey());// 护理记录单\科室护理记录单
                String htmlTbody = jcTableHtml.replaceAll("\\$\\{htmlTbody\\}", Matcher.quoteReplacement(entry.getValue()));//htmlTr.toString()
                content.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", htmlTbody));
                result.add(content);

                htmlTbody = null;
                maps = null;
                j++;
            }

        }

        return result;
    }

    /**
     * 默认获取昨天00点为取数开始时间
     *
     * @return
     */
//    @Override
    public Date getDefaltBeginDate(String hospitalCode) {
        try {

            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date());
            calendar.add(Calendar.MONTH, -1);
            String dateStr = sdfDay.format(calendar.getTime()) + " 00:00:00";
            return sdfTime.parse(dateStr);
        } catch (Exception e) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(new Date());
            calendar.add(Calendar.MONTH, -1);
            String dateStr = sdfDay.format(calendar.getTime()) + " 00:00:00";
            try {
                return sdfTime.parse(dateStr);
            } catch (ParseException e1) {
                e1.printStackTrace();
            }
        }
        return null;
    }

    @Override
    public void writeEndDate(Date endDate) {
        String hospitalCode = hospitalUSCC;// 根据条件批量更新
        String dateStr = sdfTime.format(endDate);
        try {
            FileUtils.writeStringToFile(CommonUtil.getGxsjFile(hospitalCode), dateStr, "utf-8");
            System.out.println("设置最大更新时间：" + CommonUtil.getGxsjFile(hospitalCode).getAbsolutePath());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 动态分页执行sql语句
     *
     * @param <T>
     * @param druidDataSource
     * @param executeSql
     * @param clazz
     * @param parameters
     * @return
     * @throws Exception
     */
    private <T> List<T> executeSqlByPage(DruidDataSource druidDataSource, String executeSql, Class<T> clazz,
                                         String orderBy, PageRequest pager, Object... parameters) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        DruidPooledConnection conn = null;
        Integer startIndex = pager.getStartIndex();
        Integer pageSize = pager.getPageSize();
        try {
            PageParam pp = new PageParam(executeSql, orderBy, startIndex.intValue(), pageSize.intValue());
            String pageSql = pp.getPageSqlByDbType(druidDataSource.getDbType());
            return executeSql(druidDataSource, pageSql, clazz, parameters);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                JdbcUtils.close(rs);
                JdbcUtils.close(stmt);
                JdbcUtils.close(conn);
            } catch (Exception e) {
                System.out.println(e.getMessage());
            }
        }
        return null;
    }

    /**
     * 动态执行sql语句
     *
     * @param <T>
     * @param druidDataSource
     * @param executeSql
     * @param clazz
     * @param parameters
     * @return
     * @throws Exception
     */
    public <T> List<T> executeSql(DruidDataSource druidDataSource, String executeSql, Class<T> clazz,
                                  Object... parameters) throws Exception {
        List<T> ret = new ArrayList<T>();

        PreparedStatement stmt = null;
        ResultSet rs = null;
        DruidPooledConnection conn = null;
        try {
            conn = druidDataSource.getConnection();
            stmt = conn.prepareStatement(executeSql);
            setParameters(stmt, parameters);
//            LOGGER.info("抽取病历文书：executeSql: {}, 参数：{} , preparedSql: {}", executeSql, Arrays.asList(parameters), stmt.toString());
            rs = stmt.executeQuery();
            ResultSetMetaData rsMeta = rs.getMetaData();
            Set<String> pkids = new HashSet<String>();
            while (rs.next()) {
                Map<String, Object> row = new LinkedHashMap<String, Object>();

                boolean isContinue = false;
                for (int i = 0, size = rsMeta.getColumnCount(); i < size; ++i) {
                    String columName = rsMeta.getColumnLabel(i + 1);
                    Object value = rs.getObject(i + 1);
                    if (value instanceof Clob) {
                        value = CommonUtil.ClobToString((Clob) value);
                    }
                    if(value instanceof BLOB){
                        // 无语了都两个不是同一个字符集
                        if(StringUtils.endsWithIgnoreCase(columName,"content_xml")){
                            value = CommonUtil.BLOBToString((BLOB) value, StandardCharsets.UTF_8);
                        } else {  // content_text
                            value = CommonUtil.BLOBToString((BLOB) value, Charset.forName("GBK"));
                            // 要对HTML格式进行校验并修正，否则展示可能有问题
                            value = HtmlUtils.formatFix((String) value);
                        }
                    }

                    if ("outTime".equals(CommonUtil.lineToHump(columName)) || "inTime".equals(CommonUtil.lineToHump(columName)) || "hospitTime".equals(CommonUtil.lineToHump(columName))) {
                        if (ObjectUtil.isNotEmpty(value)){
                            value = DateUtil.parse(value.toString(), "yyyy-MM-dd HH:mm:ss");
                        }
                    }
                    // 临时用一下，这个是His登记的患者ID，暂时用searchValue来进行代替
                    if ("hisRecordId".equals(CommonUtil.lineToHump(columName))) {
                        if (ObjectUtil.isNotEmpty(value)) {
                            row.put(CommonUtil.lineToHump("searchValue"), value);
                            continue;
                        }
                    }
                    if ("birthday".equals(CommonUtil.lineToHump(columName))) {
                        if (ObjectUtil.isNotEmpty(value)) {
                            value = DateUtil.parse(value.toString());
                        }
                    }
                    if (value == null && columName.equalsIgnoreCase("content_text")) {
                        isContinue = true;
                        break;
                    }
                    row.put(CommonUtil.lineToHump(columName), value);
                }
                if (isContinue) {
                    continue;
                }
                T bean = clazz.newInstance();
                try {
                    ConvertUtils.register(new DateConverter(null), Date.class);
                    ConvertUtils.register(new BigDecimalConverter(null), BigDecimal.class);
                    BeanUtils.populate(bean, row);
                } catch (Exception e) {
                    e.printStackTrace();
                    continue;
                }
                boolean hasId = false;
                if (bean instanceof EmrTree) {
                    if (pkids.contains(((EmrTree) bean).getEmrId())) {
                        hasId = true;
                    } else {
                        pkids.add(((EmrTree) bean).getEmrId());
                    }
                } else if (bean instanceof EmrRecord) {
                    if (pkids.contains(((EmrRecord) bean).getRecordId())) {
                        hasId = true;
                    } else {
                        pkids.add(((EmrRecord) bean).getRecordId());
                    }
                } else if (bean instanceof EmrContent) {
                    if (pkids.contains(((EmrContent) bean).getContentId())) {
                        hasId = true;
                    } else {
                        pkids.add(((EmrContent) bean).getContentId());
                    }
                }
                if (!hasId) {
                    ret.add(bean);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        } finally {
            try {
                JdbcUtils.close(rs);
                JdbcUtils.close(stmt);
                JdbcUtils.close(conn);
            } catch (Exception e) {
                LOGGER.error(e.getMessage());
            }
        }
        return ret;
    }

    private void setParameters(PreparedStatement stmt, Object[] parameters) throws SQLException {
        for (int i = 0, size = parameters.length; i < size; ++i) {
            Object param = parameters[i];
            if (param instanceof Date) {
                stmt.setObject(i + 1, new Timestamp(((Date) param).getTime()));
            } else {
                stmt.setObject(i + 1, param);
            }
        }
    }


    /**
     * 抽取病程记录--病案首页
     *
     * @param
     * @return
     */

    private List<EmrContent> extractEmrPageRecord(DruidDataSource druidDataSource, String emrSql, String recordId) {
        List<EmrContent> emrContents = Lists.newArrayList();

        List<Map<String, Object>> emrPage = getEmrHomePage(druidDataSource, emrSql, recordId);
//        LOGGER.info("抽取病案首页: 记录编号 = {}, 记录条数 = {}", recordId, emrPage.size());

        if (!CollectionUtils.isEmpty(emrPage)) {
            EmrContent emrContent = new EmrContent();
            emrContent.setContentId("emrPage_" + recordId);
            Date date = Calendar.getInstance().getTime();
            emrContent.setContentTime(date);
            emrContent.setTitle("病案首页");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(date);
            emrContent.setEmrId("emr01");
            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(date);
            Map<String, List<Map<String, Object>>> map = Maps.newHashMap();
            map.put("datas", emrPage);
            emrContent.setStructJson(JSON.toJSONString(map));
            emrContents.add(emrContent);
        }
        return emrContents;
    }



    /**
     * 抽取病程记录--医嘱
     *
     * @param druidDataSource
     * @param emrSql
     * @param recordId
     * @return
     */
    private List<EmrContent> extractEmrMedicalRecord(DruidDataSource druidDataSource, String emrSql, String recordId ) {
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, recordId);
//        LOGGER.info("抽取病程记录--医嘱: 记录编号 = {}, 医嘱条数 = {}", recordId, allColumnNameAndValue.size());
        List<EmrContent> result = null;

        if (!CollectionUtils.isEmpty(allColumnNameAndValue)) {
            result = new ArrayList<>();
            Map<String, String> emrContentMap = Maps.newHashMap();
            int j = 0;
            for ( int k = 0; k < allColumnNameAndValue.size(); k++) {
                Map<String, Object> map = allColumnNameAndValue.get(k);

                // 医嘱开立日期事件  HDSD00_15_020
                // 医嘱内容  	      HDSD00_15_028
                // 医嘱开立医生  	  HDSD00_15_021
                // 核对医嘱护士签名  HDSD00_15_006
                // 医嘱停止日期时间  HDSD00_15_026,
                // 停止医嘱者签名 	  HDSD00_15_013,
                // 医嘱执行者签名    HDSD00_15_031,
                StringBuilder htmlTr = new StringBuilder();

                htmlTr.append("			<tr data-index=\"" + k + "\">\r\n");
                htmlTr.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("医嘱计划开始日期时间")) + "</td>\r\n");
                htmlTr.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("医嘱项目内容")) + "" + nullFilter(map.get("医嘱备注信息")) + "</td>\r\n");
                htmlTr.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("医嘱开立者签名")) + "</td>\r\n");
                htmlTr.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("医嘱执行者签名")) + "</td>\r\n");
                htmlTr.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("医嘱停止日期时间")) + "</td>\r\n");
                htmlTr.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("停止医嘱者签名")) + "</td>\r\n");
                htmlTr.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                htmlTr.append(nullFilter(map.get("医嘱执行者签名")) + "</td>\r\n" + "			</tr>");

                // 当前只有长期医嘱
                String title = map.get("医嘱类型").toString();
                if (emrContentMap.containsKey(title)) {
                    String contentText = emrContentMap.get(title) + htmlTr.toString();
                    emrContentMap.put(title, contentText);
                } else {
                    emrContentMap.put(title, htmlTr.toString());
                }
            }
            j = 1;
            String htmlHead = ContentConstant.HTML_HEAD;
            String jcTableHtml = ContentConstant.EMRMEDICALRECORD_HTML_BODY;
            for (Map.Entry<String, String> entry : emrContentMap.entrySet()) {
                EmrContent content = new EmrContent();
                Date date = new Date();
                content.setContentId("yz_" + recordId + '_' + j);
                content.setRecordId(recordId);
                content.setContentTime(date);
                content.setCreateBy("admin");
                content.setCreateTime(date);
                content.setEmrId("emr03");
                content.setShowOrder("1");
                content.setUpdateBy("admin");
                content.setUpdateTime(date);
                Map<String, List<Map<String, Object>>> maps = Maps.newHashMap();
                maps.put("datas", allColumnNameAndValue);
                content.setStructJson(JSON.toJSONString(maps));
                content.setTitle(entry.getKey());
                String htmlTbody = jcTableHtml.replaceAll("\\$\\{htmlTbody\\}", Matcher.quoteReplacement(entry.getValue()));
                content.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", htmlTbody));
                result.add(content);

                htmlTbody = null;
                maps = null;
                j++;
            }
        }
        return result;
    }

    /**
     * 检查报告
     * @param druidDataSource
     * @param emrSql
     * @param recordId
     * @return
     */
    private List<EmrContent> extractEmrPacsRecord(DruidDataSource druidDataSource, String emrSql, String recordId ) {
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, recordId);
//        LOGGER.info("抽取病程记录--检查报告: 记录编号 = {}, 条数 = {}", recordId, allColumnNameAndValue.size());
        List<EmrContent> result = new ArrayList<EmrContent>();

        if (!CollectionUtils.isEmpty(allColumnNameAndValue)) {
            EmrContent emrContent = new EmrContent();
            Date date = Calendar.getInstance().getTime();
            emrContent.setContentTime(date);
            emrContent.setContentId("jc_" + recordId);
            emrContent.setTitle("检查报告单");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(date);
            emrContent.setEmrId("emr06");

            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(date);
            Map<String, List<Map<String, Object>>> maps = Maps.newHashMap();
            maps.put("datas", allColumnNameAndValue);
            emrContent.setStructJson(JSON.toJSONString(maps));

            String htmlHead = ContentConstant.HTML_HEAD;
            StringBuffer jcTableHtml = new StringBuffer(ContentConstant.EMRRISRECORD_HTML_BODY);
            int j = 0;
            for (Map<String, Object> map : allColumnNameAndValue) {

                // 检查类别  JCLB;检查项目  JCXM;检查医生  JCYS ; 检查结果客观  JCJG;检查结果主观  JCJG
                // 检查报告日期时间  JCRQSJ
                jcTableHtml.append("			<tr data-index=\"" + j + "\">\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("EXAM_CLASS")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("EXAM_SUB_CLASS")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("PERFORMED_BY")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("REPORT_TEXT1")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("REPORT_TEXT")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("EXAM_DATE_TIME")) + "</td>\r\n" + "			</tr>");
                j++;

            }
            jcTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jcTableHtml.toString())));
            result.add(emrContent);
            jcTableHtml = null;
        }
        return result;
    }
    /**
     * 检验文书
     * @param druidDataSource
     * @param emrSql
     * @return
     */
    private List<EmrContent> extractEmrLISRecord(DruidDataSource druidDataSource, String emrSql, String recordId){
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, recordId);
//        LOGGER.info("抽取病程记录--检验文书: 记录编号 = {}, 检验项目条数 = {}", recordId, allColumnNameAndValue.size());

        List<EmrContent> result = Lists.newArrayList();
        if (!org.apache.commons.collections4.CollectionUtils.isEmpty(allColumnNameAndValue)) {
            EmrContent emrContent = new EmrContent();
            emrContent.setContentId("jy_" + recordId);
            Date date = Calendar.getInstance().getTime();
//            emrContent.setContentTime(date);
            emrContent.setTitle("检验报告单");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(date);
            emrContent.setEmrId("emr06");
            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(date);
            Map<String, List<Map<String, Object>>> map = Maps.newHashMap();
            map.put("datas", allColumnNameAndValue);
            emrContent.setStructJson(JSON.toJSONString(map));

            int j = 0;
            String htmlHead = ContentConstant.HTML_HEAD;
            String EMRLISRECORD_HTML_TITLE
                    = "<div class=\"fixed-table-body\">\r\n"
                    + "	<table id=\"bootstrap-table\" class=\"table table-hover\">\r\n" + "		<thead>\r\n"
                    + "			<tr>\r\n"
                    + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userId\" tabindex=\"0\" nowrap><div\r\n"
                    + "						class=\"th-inner \" style='width:100px' nowrap>检验类别</div>\r\n"
                    + "					<div class=\"fht-cell\"></div></th>\r\n"
                    + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userName\" tabindex=\"0\" nowrap><div\r\n"
                    + "						class=\"th-inner \" nowrap>检验项目</div>\r\n"
                    + "					<div class=\"fht-cell\"></div></th>\r\n"
                    + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userName\" tabindex=\"0\" nowrap><div\r\n"
                    + "						class=\"th-inner \" nowrap>检验结果</div>\r\n"
                    + "					<div class=\"fht-cell\"></div></th>\r\n"
                    + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userPhone\" tabindex=\"0\" nowrap><div\r\n"
                    + "						class=\"th-inner \" nowrap>执行医生</div>\r\n"
                    + "					<div class=\"fht-cell\"></div></th>\r\n"
                    + "				<th style=\"text-align: center;border: 1px solid #2f4050!important;\" data-field=\"userEmail\" tabindex=\"0\" nowrap><div\r\n"
                    + "						class=\"th-inner \" style='width:100px'  nowrap>检验日期时间</div>\r\n"
                    + "					<div class=\"fht-cell\"></div></th>\r\n" + "			</tr>\r\n"
                    + "		</thead>" + "		<tbody>";
            StringBuffer jyTableHtml = new StringBuffer(EMRLISRECORD_HTML_TITLE);
            for (Map<String, Object> tempMap : allColumnNameAndValue) {
                // SFXM,	--收费项目
                // JYXM,	--检验项目
                // jyjg,    --检验结果
                // MZZD,	--门诊诊断
                // --ZXYS,	--执行医生
                // JYRQSJ,	--检验日期时间
                jyTableHtml.append( "			<tr data-index=\"" + j + "\">\r\n");
                jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append( nullFilter(tempMap.get("标本")) + "</td>\r\n");
                jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append( nullFilter(tempMap.get("检验项目名称")) + "</td>\r\n");
                jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append( nullFilter(tempMap.get("结果"))+ "</td>\r\n");
                jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append( nullFilter(tempMap.get("检验人员")) + "</td>\r\n");
                jyTableHtml.append( "				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jyTableHtml.append( nullFilter(tempMap.get("检验日期")) + "</td>\r\n" + "			</tr>");
                j++;
            }
            jyTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jyTableHtml.toString())));
            result.add(emrContent);
            jyTableHtml = null;
        }
        return result;
    }

    /**
     * 获取检验申请单
     * @param druidDataSource
     * @param emrSql
     * @param recordId
     * @return
     */
    private List<EmrContent> extractEmrLisApplyRecord(DruidDataSource druidDataSource, String emrSql, String recordId ) {
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, recordId);
//        LOGGER.info("抽取病程记录--检验申请单: 记录编号 = {}, 条数 = {}", recordId, allColumnNameAndValue.size());
        List<EmrContent> result = new ArrayList<EmrContent>();

        if (!CollectionUtils.isEmpty(allColumnNameAndValue)) {
            EmrContent emrContent = new EmrContent();
            Date date = Calendar.getInstance().getTime();
            emrContent.setContentTime(date);
            emrContent.setContentId("jysqd_" + recordId);
            emrContent.setTitle("检验申请单");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(date);
            emrContent.setEmrId("emr03");
            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(date);

            Map<String, List<Map<String, Object>>> maps = Maps.newHashMap();
            maps.put("datas", allColumnNameAndValue);
            emrContent.setStructJson(JSON.toJSONString(maps));

            String htmlHead = ContentConstant.HTML_HEAD;
            StringBuffer jcTableHtml = new StringBuffer(ContentConstant.EMRLisApplyRECORD_HTML_TITLE);
            int j = 0;
            for (Map<String, Object> map : allColumnNameAndValue) {

                // 检查类别  JCLB;检查项目  JCXM;检查医生  JCYS ; 检查结果客观  JCJG;检查结果主观  JCJG
                // 检查报告日期时间  JCRQSJ
                jcTableHtml.append("			<tr data-index=\"" + j + "\">\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("标本")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("项目名称")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("临床印象")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("申请医师")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("申请时间")) + "</td>\r\n" + "			</tr>");
                j++;
            }
            jcTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jcTableHtml.toString())));
            result.add(emrContent);
            jcTableHtml = null;
        }
        return result;
    }

    /**
     * 获取检查申请单
     * @param druidDataSource
     * @param emrSql
     * @param recordId
     * @return
     */
    private List<EmrContent> extractEmrPacsApplyRecord(DruidDataSource druidDataSource, String emrSql, String recordId ) {
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, recordId);
//        LOGGER.info("抽取病程记录--检查申请单: 记录编号 = {}, 条数 = {}", recordId, allColumnNameAndValue.size());
        List<EmrContent> result = new ArrayList<EmrContent>();

        if (!CollectionUtils.isEmpty(allColumnNameAndValue)) {
            EmrContent emrContent = new EmrContent();
            Date date = Calendar.getInstance().getTime();
            emrContent.setContentTime(date);
            emrContent.setContentId("jcsqd_" + recordId);
            emrContent.setTitle("检查申请单");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(date);
            emrContent.setEmrId("emr03");
            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(date);

            Map<String, List<Map<String, Object>>> maps = Maps.newHashMap();
            maps.put("datas", allColumnNameAndValue);
            emrContent.setStructJson(JSON.toJSONString(maps));

            String htmlHead = ContentConstant.HTML_HEAD;
            StringBuffer jcTableHtml = new StringBuffer(ContentConstant.EMRPacsApplyRECORD_HTML_TITLE);
            int j = 0;
            for (Map<String, Object> map : allColumnNameAndValue) {

                // 检查类别  JCLB;检查项目  JCXM;检查医生  JCYS ; 检查结果客观  JCJG;检查结果主观  JCJG
                // 检查报告日期时间  JCRQSJ
                jcTableHtml.append("			<tr data-index=\"" + j + "\">\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("PACS类别")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("项目名称")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("临床印象")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("病例摘要")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("申请医师")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("申请时间")) + "</td>\r\n" + "			</tr>");
                j++;
            }
            jcTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jcTableHtml.toString())));
            result.add(emrContent);
            jcTableHtml = null;
        }
        return result;
    }

    /**
     * 获取手术通知单
     * @param druidDataSource
     * @param emrSql
     * @param recordId
     * @return
     */
    private List<EmrContent> extractEmrOpeNoticeRecord(DruidDataSource druidDataSource, String emrSql, String recordId ) {
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, recordId);
//        LOGGER.info("抽取知情同意书--手术通知单: 记录编号 = {}, 条数 = {}", recordId, allColumnNameAndValue.size());
        List<EmrContent> result = new ArrayList<EmrContent>();

        if (!CollectionUtils.isEmpty(allColumnNameAndValue)) {
            EmrContent emrContent = new EmrContent();
            Date date = Calendar.getInstance().getTime();
            emrContent.setContentTime(date);
            emrContent.setContentId("sstzd_" + recordId);
            emrContent.setTitle("手术通知单");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(date);
            emrContent.setEmrId("emr05");
            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(date);

            Map<String, List<Map<String, Object>>> maps = Maps.newHashMap();
            maps.put("datas", allColumnNameAndValue);
            emrContent.setStructJson(JSON.toJSONString(maps));

            String htmlHead = ContentConstant.HTML_HEAD;
            StringBuffer jcTableHtml = new StringBuffer(ContentConstant.EMROpeNoticeRECORD_HTML_TITLE);
            int j = 0;
            for (Map<String, Object> map : allColumnNameAndValue) {

                jcTableHtml.append("			<tr data-index=\"" + j + "\">\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("手术名称")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("手术等级")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("切口等级")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("申请医生")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("术前诊断")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("麻醉方法")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("手术医师")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("手术日期")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("I助")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("II助")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("麻醉医师")) + "</td>\r\n" + "			</tr>");
                j++;
            }
            jcTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jcTableHtml.toString())));
            result.add(emrContent);
            jcTableHtml = null;
        }
        return result;
    }

    /**
     * 获取会诊记录
     * @param druidDataSource
     * @param emrSql
     * @param recordId
     * @return
     */
    private List<EmrContent> extractEmrConsultationRecord(DruidDataSource druidDataSource, String emrSql, String recordId ) {
        List<Map<String, Object>> allColumnNameAndValue = getALLColumnNameAndValue(druidDataSource, emrSql, recordId);
//        LOGGER.info("病程记录--会诊申请: 记录编号 = {}, 条数 = {}", recordId, allColumnNameAndValue.size());
        List<EmrContent> result = new ArrayList<EmrContent>();

        if (!CollectionUtils.isEmpty(allColumnNameAndValue)) {
            EmrContent emrContent = new EmrContent();
            Date date = Calendar.getInstance().getTime();
            emrContent.setContentTime(date);
            emrContent.setContentId("hzjl_" + recordId);
            emrContent.setTitle("会诊申请");
            emrContent.setRecordId(recordId);
            emrContent.setCreateBy("admin");
            emrContent.setCreateTime(date);
            emrContent.setEmrId("emr03");
            emrContent.setShowOrder("1");
            emrContent.setUpdateBy("admin");
            emrContent.setUpdateTime(date);

            Map<String, List<Map<String, Object>>> maps = Maps.newHashMap();
            maps.put("datas", allColumnNameAndValue);
            emrContent.setStructJson(JSON.toJSONString(maps));

            String htmlHead = ContentConstant.HTML_HEAD;
            StringBuffer jcTableHtml = new StringBuffer(ContentConstant.EMRConsultationRECORD_HTML_TITLE);
            int j = 0;
            for (Map<String, Object> map : allColumnNameAndValue) {

                jcTableHtml.append("			<tr data-index=\"" + j + "\">\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("病历摘要")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("申请会诊目的")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("会诊时间")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("主诉")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("辅助检查")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("诊疗过程")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("会诊科室")) + "</td>\r\n");
                jcTableHtml.append("				<td style=\"text-align: center;border: 1px solid #2f4050!important;\">");
                jcTableHtml.append(nullFilter(map.get("会诊地点")) + "</td>\r\n" + "			</tr>");
                j++;
            }
            jcTableHtml.append("		</tbody>\r\n" + "	</table>\r\n" + "</div>");
            emrContent.setContentText(htmlHead.replaceAll("\\$\\{htmlBody\\}", Matcher.quoteReplacement(jcTableHtml.toString())));
            result.add(emrContent);
            jcTableHtml = null;
        }
        return result;
    }

    /**
     * 获取SQL所有字段和值
     *
     * @param druidDataSource 数据源
     * @param sql             SQL语句
     * @param recordId
     * @return List<Map < String, Object>> [{columnName：value}]
     * @author ouyang0810@foxmail.com
     * @Date 2020/6/4 16:58
     */
    public static List<Map<String, Object>> getALLColumnNameAndValue(DruidDataSource druidDataSource, String sql, String recordId) {
        List<Map<String, Object>> result = new ArrayList<>();
        ResultSet rs = null;
        PreparedStatement pst = null;
        Connection conn = null;
        try {
            conn = druidDataSource.getConnection();
            pst = conn.prepareStatement(sql);
            String[] params = recordId.split("_");
            pst.setString(1, params[0]);
            pst.setString(2, params[1]);
            rs = pst.executeQuery();
            ResultSetMetaData rsMeta = rs.getMetaData();
            while (rs.next()) {
                Map<String, Object> map = Maps.newHashMap();
                for (int i = 0, size = rsMeta.getColumnCount(); i < size; ++i) {
                    String columName = rsMeta.getColumnLabel(i + 1).trim(); // 移除两边的空格
                    Object value = rs.getObject(i + 1);
                    if (null != value && value instanceof Timestamp) {
                        value = DateUtils.dateToString(DateUtils.stringToDate(value.toString(), DateUtils.DATE_TIME_FORMAT), DateUtils.DATE_TIME_FORMAT);
                    }
                    map.put(columName, value);
                }
                result.add(map);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("连接失败");
        } finally {
            try {
                if (rs != null) {
                    rs.close();
                }
                if (pst != null) {
                    pst.close();
                }
                if (conn != null) {
                    conn.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
        return result;
    }


    public static List<Map<String, Object>> getEmrHomePage(DruidDataSource druidDataSource, String sql, String recordId) {
        String[] param = {recordId};
        List<Map<String, Object>> result = new ArrayList<>();
        Map<String, String> diag;
        Map<String, String> oper;
        Map<String, String> ddzzfy;
        List<Map<String, String>> list = null;
        ResultSet rs = null;
        PreparedStatement pst = null;
        Connection conn = null;
        try {
            conn = druidDataSource.getConnection();
            pst = conn.prepareStatement(sql);
            pst.setString(1, recordId);
            rs = pst.executeQuery();
            LOGGER.info("查询病案首页数据sql {}, {}", sql, recordId);
            ResultSetMetaData rsMeta = rs.getMetaData();
            while (rs.next()) {
                // 加载病案首页数据
                Map<String, Object> map = Maps.newLinkedHashMap();
                for (int i = 0, size = rsMeta.getColumnCount(); i < size; ++i) {
                    String columnName = rsMeta.getColumnLabel(i + 1).trim();
                    Object value = rs.getObject(i + 1);
                    if (value instanceof Timestamp) {
                        value = DateUtils.dateToString(DateUtils.stringToDate(value.toString(), DateUtils.DATE_TIME_FORMAT), DateUtils.DATE_TIME_FORMAT);
                    }
                    map.put(columnName, value);
                }

                // 加载诊断数据
                ResultSet rs1 = null;
                PreparedStatement pst1 = null;
                Connection conn1 = null;
                try {
                    String diagInfoSql = "SELECT * FROM	ek_thdhanyun.his_ddinp_diag t WHERE t.record_id = ?";
                    conn1 = druidDataSource.getConnection();
                    pst1 = conn1.prepareStatement(diagInfoSql);
                    pst1.setString(1, param[0]);

                    rs1 = pst1.executeQuery();
                    list = new ArrayList<Map<String, String>>();
                    ResultSetMetaData rsMeta1 = rs1.getMetaData();
                    while (rs1.next()) {
                        diag = new LinkedHashMap<String, String>();
                        for (int i = 1, size = rsMeta1.getColumnCount(); i <= size; i++) {
                            String columnName = rsMeta1.getColumnLabel(i).trim();
                            String value = rs1.getString(i);
                            diag.put(lineToHump(columnName), value);
                        }
                        list.add(diag);
                    }
                } catch (SQLException e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (rs1 != null) {
                            rs1.close();
                        }
                        if (pst1 != null) {
                            pst1.close();
                        }
                        if (conn1 != null) {
                            conn1.close();
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                map.put("zdxxs", list);

                // 手术信息
                ResultSet rs2 = null;
                PreparedStatement pst2 = null;
                Connection conn2 = null;
                try {
                    String operInfoSql = "select * from ek_thdhanyun.his_ddinp_oper t WHERE t.record_id = ? ORDER BY t.hdsd00_11_090_order ";
                    conn2 = druidDataSource.getConnection();
                    pst2 = conn2.prepareStatement(operInfoSql);
                    pst2.setString(1, param[0]);
                    rs2 = pst2.executeQuery();
                    list = new ArrayList<Map<String, String>>();
                    ResultSetMetaData rsMeta2 = rs2.getMetaData();
                    while (rs2.next()) {
                        oper = new LinkedHashMap<String, String>();
                        for (int i = 1, size = rsMeta2.getColumnCount(); i <= size; i++) {
                            String columnName = rsMeta2.getColumnLabel(i).trim();
                            String value = rs2.getString(i);
                            oper.put(lineToHump(columnName), value);
                        }
                        list.add(oper);
                    }
                } catch (SQLException e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (rs2 != null) {
                            rs2.close();
                        }
                        if (pst2 != null) {
                            pst2.close();
                        }
                        if (conn2 != null) {
                            conn2.close();
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                map.put("ssxxs", list);

                // 重症监护室
                ResultSet rs3 = null;
                PreparedStatement pst3 = null;
                Connection conn3 = null;
                try {
                    String ddzzfyInfoSql = "select * from ek_thdhanyun.his_ddinp_icu t WHERE t.record_id = ? ORDER BY t.HDSD00_11_151 ";
                    conn3 = druidDataSource.getConnection();
                    pst3 = conn3.prepareStatement(ddzzfyInfoSql);
                    pst3.setString(1, param[0]);
                    rs3 = pst3.executeQuery();
                    list = new ArrayList<Map<String, String>>();
                    ResultSetMetaData rsMeta3 = rs3.getMetaData();
                    while (rs3.next()) {
                        ddzzfy = new LinkedHashMap<String, String>();
                        for (int i = 1, size = rsMeta3.getColumnCount(); i <= size; i++) {
                            String columnName = rsMeta3.getColumnLabel(i).trim();
                            String value = rs3.getString(i);
                            ddzzfy.put(lineToHump(columnName), value);
                        }
                        list.add(ddzzfy);
                    }
                } catch (SQLException e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (rs3 != null) {
                            rs3.close();
                        }
                        if (pst3 != null) {
                            pst3.close();
                        }
                        if (conn3 != null) {
                            conn3.close();
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
                map.put("zzjhs", list);
                result.add(map);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("连接失败");
        } finally {
            try {
                if (rs != null) {
                    rs.close();
                }
                if (pst != null) {
                    pst.close();
                }
                if (conn != null) {
                    conn.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    private static final char UNDERLINE = '_';

    private String nullFilter(Object o) {
        if (o == null) {
            return "";
        } else if (o instanceof String) {
            return (String) o;
        } else {
            return String.valueOf(o);
        }
    }

    public static String lineToHump(String param) {
        if (StringUtils.isBlank(param)) {
            return "";
        }
        param = param.toLowerCase();
        int len = param.length();
        StringBuilder sb = new StringBuilder(len);
        for (int i = 0; i < len; i++) {
            char c = Character.toLowerCase(param.charAt(i));
            if (c == UNDERLINE) {
                if (++i < len) {
                    sb.append(Character.toUpperCase(param.charAt(i)));
                }
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }

    /**
     * 基础数据抽取
     * @param mrqcEtlService
     * @param objectType
     * @throws Exception
     */
    @Override
    public void etlMdb(IMrqcEtlService mrqcEtlService, String objectType) throws Exception {
        /*
         * 科室数据同步
         */
        if (org.springframework.util.StringUtils.isEmpty(objectType) || com.ruoyi.common.utils.StringUtils.equals(objectType, "科室")) {
            String ksSql = "select distinct * from hanyun.his_hhdept_dict";
            List<Map<String, Object>> ksList = wxjyJdbcTemplate.queryForList(ksSql);
            if (!CollectionUtils.isEmpty(ksList)) {// 添加科室主数据
                List<MdbObject> objectList = new ArrayList<MdbObject>();
                for (Map<String, Object> ks : ksList) {
                    List<MdbObjectAttr> attrs = new ArrayList<MdbObjectAttr>();
                    MdbObject object = new MdbObject();
                    String orgId = (String) ks.get("科室代码");//????
                    String ksCode = (String) ks.get("科室代码");
                    String ksName = (String) ks.get("科室名称");
                    String objectId = orgId + ksCode;
                    object.setOrgId(orgId);
                    object.setObjectId(objectId);
                    object.setObjectCode(ksCode);
                    object.setObjectName(ksName);
                    object.setObjectType("科室");
                    object.setParentId("0");
                    object.setRemark(ksName);
                    for (String key : ks.keySet()) {
                        MdbObjectAttr attr = new MdbObjectAttr();
                        attr.setAttrId(objectId + "_" + key);
                        attr.setAttrCode(key);
                        attr.setAttrName(key);
                        attr.setAttrValue((String) ks.get(key));
                        attr.setObjectId(objectId);
                        attr.setRemark(key);
                        attrs.add(attr);
                    }
                    object.setAttrs(attrs);

                    objectList.add(object);
                    if (objectList.size() >= 200) {
                        mrqcEtlService.insertOrUpdateMdbObjects(objectList);
                        objectList.clear();
                    }
                }
                if (objectList.size() >= 1) {
                    mrqcEtlService.insertOrUpdateMdbObjects(objectList);
                    objectList.clear();
                }
                System.out.println("完成主数据同步，科室[" + ksList.size() + "]");
            }
        }

        /*
         *  人员信息同步
         */
        if (org.springframework.util.StringUtils.isEmpty(objectType) || com.ruoyi.common.utils.StringUtils.equals(objectType, "人员")) {
            String rySql = "select distinct * from hanyun.his_hhstaff_dict";
            List<Map<String, Object>> ryList = wxjyJdbcTemplate.queryForList(rySql);
            if (!CollectionUtils.isEmpty(ryList)) {// 添加科室主数据
                List<MdbObject> objectList = new ArrayList<MdbObject>();
                for (Map<String, Object> ry : ryList) {
                    List<MdbObjectAttr> attrs = new ArrayList<MdbObjectAttr>();
                    MdbObject object = new MdbObject();
                    String orgId = (String) ry.get("唯一号");// ???
                    String ryCode = (String) ry.get("唯一号");
                    String ryName = (String) ry.get("NAME");
                    String ksCode = (String) ry.get("DEPT_ID");
                    String objectId = orgId + ryCode;
                    String parentId = orgId + ksCode;
                    object.setOrgId(orgId);
                    object.setObjectId(objectId);
                    object.setObjectCode(ryCode);
                    object.setObjectName(ryName);
                    object.setObjectType("人员");
                    object.setParentId(parentId);
                    object.setRemark(ryName);

                    for (String key : ry.keySet()) {
                        MdbObjectAttr attr = new MdbObjectAttr();
                        attr.setAttrId(objectId + "_" + key);
                        attr.setAttrCode(key);
                        attr.setAttrName(key);
                        attr.setAttrValue(ry.get(key) + "");
                        attr.setObjectId(objectId);
                        attr.setRemark(key);
                        attrs.add(attr);
                    }
                    object.setAttrs(attrs);
                    objectList.add(object);
                    if (objectList.size() >= 200) {
                        mrqcEtlService.insertOrUpdateMdbObjects(objectList);
                        objectList.clear();
                    }
                }
                if (objectList.size() >= 1) {
                    mrqcEtlService.insertOrUpdateMdbObjects(objectList);
                    objectList.clear();
                }
                System.out.println("完成主数据同步，人员[" + ryList.size() + "]");
            }
        }

        /*
         * 床位数据同步
         */
        if (org.springframework.util.StringUtils.isEmpty(objectType) || com.ruoyi.common.utils.StringUtils.equals(objectType, "病床")) {
            String bcSql = "select distinct * from hanyun.his_hhbed_rec";
            List<Map<String, Object>> bcList = wxjyJdbcTemplate.queryForList(bcSql);
            if (!CollectionUtils.isEmpty(bcList)) {// 添加科室主数据
                List<MdbObject> objectList = new ArrayList<MdbObject>();
                for (Map<String, Object> bc : bcList) {
                    List<MdbObjectAttr> attrs = new ArrayList<MdbObjectAttr>();
                    MdbObject object = new MdbObject();
                    String orgId = (String) bc.get("床号"); // ？？？？
                    String bcCode = bc.get("床号") + "";
                    String bcName = (String) bc.get("床号");
                    String bfName = (String) bc.get("房间号");
                    bcCode = org.springframework.util.StringUtils.isEmpty(bfName) ? bcCode : bfName;
                    bfName = org.springframework.util.StringUtils.isEmpty(bfName) ? bcName : bfName;
                    String ksCode = (String) bc.get("科室编码");
                    String objectId = orgId + bcCode;
                    String parentId = orgId + ksCode;
                    object.setOrgId(orgId);
                    object.setObjectId(objectId);
                    object.setObjectCode(bcCode);
                    object.setObjectName(bfName);
                    object.setObjectType("病房");
                    object.setParentId(parentId);
                    object.setRemark(bcName);

                    for (String key : bc.keySet()) {
                        MdbObjectAttr attr = new MdbObjectAttr();
                        attr.setAttrId(objectId + "_" + key);
                        attr.setAttrCode(key);
                        attr.setAttrName(key);
                        attr.setAttrValue(bc.get(key) + "");
                        attr.setObjectId(objectId);
                        attr.setRemark(key);
                        attrs.add(attr);
                    }
                    object.setAttrs(attrs);
                    objectList.add(object);
                    if (objectList.size() >= 200) {
                        mrqcEtlService.insertOrUpdateMdbObjects(objectList);
                        objectList.clear();
                    }
                }
                if (objectList.size() >= 1) {
                    mrqcEtlService.insertOrUpdateMdbObjects(objectList);
                    objectList.clear();
                }
                System.out.println("完成主数据同步，病房[" + bcList.size() + "]");
            }
        }

        /*
         * 基础字典同步
         */
//        if (org.springframework.util.StringUtils.isEmpty(objectType) || com.ruoyi.common.utils.StringUtils.equals(objectType, "字典")) {
//            String zdSql = "select distinct * from \"字典信息视图\"";
//            List<Map<String, Object>> zdList = wxjyJdbcTemplate.queryForList(zdSql);
//            if (!CollectionUtils.isEmpty(zdList)) {// 添加科室主数据
//                List<MdbObject> objectList = new ArrayList<MdbObject>();
//                for (Map<String, Object> zd : zdList) {
//                    List<MdbObjectAttr> attrs = new ArrayList<MdbObjectAttr>();
//                    MdbObject object = new MdbObject();
//                    String orgId = (String) zd.get("ORGID");
//                    String zdCode = zd.get("DICTID") + "";
//                    String zdName = (String) zd.get("DICTNAME");
//                    String zdType = (String) zd.get("TYPENAME");
//                    String objectId = orgId + zdType + zdCode;
//                    String parentId = orgId + zdType + zdCode;
//                    object.setOrgId(orgId);
//                    object.setObjectId(objectId);
//                    object.setObjectCode(zdCode);
//                    object.setObjectName(zdName);
//                    object.setObjectType(zdType);
//                    object.setParentId("0");
//                    object.setRemark(zdName);
//
//                    for (String key : zd.keySet()) {
//                        MdbObjectAttr attr = new MdbObjectAttr();
//                        attr.setAttrId(objectId + "_" + key);
//                        attr.setAttrCode(key);
//                        attr.setAttrName(key);
//                        attr.setAttrValue(zd.get(key) + "");
//                        attr.setObjectId(objectId);
//                        attr.setRemark(key);
//                        attrs.add(attr);
//                    }
//                    object.setAttrs(attrs);
//                    objectList.add(object);
//                    if (objectList.size() >= 200) {
//                        mrqcEtlService.insertOrUpdateMdbObjects(objectList);
//                        objectList.clear();
//                    }
//                }
//                if (objectList.size() >= 1) {
//                    mrqcEtlService.insertOrUpdateMdbObjects(objectList);
//                    objectList.clear();
//                }
//                System.out.println("完成主数据同步，字典数据[" + zdList.size() + "]");
//            }
//        }

    }


}
