package com.hanyun.hip.mrqc.jms.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hanyun.hip.mrqc.common.DateUtils;
import com.hanyun.hip.mrqc.jms.constants.JmsConstant;
import com.hanyun.hip.mrqc.jms.entity.MrqcEmrConfig;
import com.hanyun.hip.mrqc.jms.mapper.AutoAssignMapper;
import com.hanyun.hip.mrqc.rule.constant.Constant;
import com.hanyun.hip.mrqc.service.Component.CommonComp;
import com.hanyun.hip.mrqc.service.entity.*;
import com.hanyun.hip.mrqc.service.manage.AutoAssignEMR;
import com.hanyun.hip.mrqc.service.manage.AutoAssignEMRToolService;
import com.hanyun.hip.mrqc.service.mapper.IEmrRecordMapper;
import com.hanyun.hip.mrqc.service.mapper.MrqcOperatorInfoMapper;
import com.hanyun.hip.mrqc.service.mapper.MrqcOperatorRecordMapper;
import com.hanyun.hip.mrqc.service.service.IMrqcOperatorInfoService;
import com.hanyun.hip.mrqc.service.service.IMrqcReportMonthService;
import com.ruoyi.system.domain.SysConfig;
import com.ruoyi.system.domain.SysUser;
import com.ruoyi.system.mapper.SysConfigMapper;
import com.ruoyi.system.mapper.SysDeptMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.ModelMap;

import java.util.*;

/**
 *
 * 南通大学附属医院电子病历分配实现
 *
 */
@Component("JmsAutoAssignEMRImpl")
public class JmsAutoAssignEMRImpl extends AutoAssignEMR {

	private static final Logger logger = LoggerFactory.getLogger(JmsAutoAssignEMRImpl.class);

	private static final String PREFIX_ZJSY_SYSCONFIG_RULE_KEY = "mrqc.rule.getEmr.tdfy.";

	private final AutoAssignEMRToolService assignEMRToolService;
	private final IMrqcReportMonthService mrqcReportMonthService;
	private final MrqcOperatorRecordMapper mrqcOperatorRecordMapper;
	private final SysConfigMapper sysConfigMapper;
	private final SysUserMapper sysUserMapper;
	private final SysDeptMapper sysDeptMapper;
	private final IEmrRecordMapper emrRecordMapper;
	private final AutoAssignMapper autoAssignMapper;


	@Autowired
	private IMrqcEmrConfigService mrqcEmrConfigService;

	@Autowired
	private IMrqcOperatorInfoService mrqcOperatorInfoServicel;

	@Autowired
	private MrqcOperatorInfoMapper mrqcOperatorInfoMapper;

	@Autowired
	private CommonComp commonComp;


	public JmsAutoAssignEMRImpl(IEmrRecordMapper emrRecordMapper, SysDeptMapper sysDeptMapper,
								 SysUserMapper sysUserMapper, SysConfigMapper sysConfigMapper,
								 MrqcOperatorRecordMapper mrqcOperatorRecordMapper, IMrqcReportMonthService mrqcReportMonthService,
								 AutoAssignEMRToolService assignEMRToolService,AutoAssignMapper autoAssignMapper) {

		this.emrRecordMapper = emrRecordMapper;
		this.sysDeptMapper = sysDeptMapper;
		this.sysUserMapper = sysUserMapper;
		this.sysConfigMapper = sysConfigMapper;
		this.mrqcOperatorRecordMapper = mrqcOperatorRecordMapper;
		this.mrqcReportMonthService = mrqcReportMonthService;
		this.assignEMRToolService = assignEMRToolService;
		this.autoAssignMapper = autoAssignMapper;
	}

	@Override
	public void pageParameters(ModelMap modelMap) {
		commonComp.getLoginUserInfo(modelMap);
		List<MrqcEmrConfig> list=mrqcEmrConfigService.selectMrqcEmrConfigList(null);
		List<QcUser> qcUsers=mrqcOperatorInfoServicel.selectMrqcOeprInfoByList(null, null);
		modelMap.put("list",list);
		modelMap.put("qcUsers",qcUsers);
	}

	@Override
	public List<MrqcOperatorInfo> obtainOperator(String qcMonth) {
		List<MrqcOperatorInfo> operatorInfoList = new ArrayList<>();
		HashMap<String,MrqcOperatorInfo> hashMap = new HashMap<>();
		//规则中配置的质控员
		JSONArray jsonArray =JSONArray.parseArray(mrqcEmrConfigService.selectMrqcEmrConfigByKey("emrs").getConfigValue());
		for(int i=0;i<jsonArray.size();i++){
			JSONObject jsonObject = jsonArray.getJSONObject(i);
			String departmentSel = jsonObject.getString("departmentSel");
			String inpatientAreaSel = jsonObject.getString("inpatientAreaSel");
			String zkySel = jsonObject.getString("zkySel");
			MrqcOperatorInfo operatorInfo = new MrqcOperatorInfo();
			operatorInfo.setUserId(Long.parseLong(zkySel));
			operatorInfo.setOperationDept(departmentSel);
			operatorInfo.setMedicalGroup(inpatientAreaSel);
			//判断是否存在
			MrqcOperatorInfo userInfo = mrqcOperatorInfoMapper.selectMrqcOperatorInfoById(operatorInfo.getUserId()+"");
			if(userInfo != null) {
				if(com.ruoyi.common.utils.StringUtils.isNotBlank(departmentSel) || com.ruoyi.common.utils.StringUtils.isNotBlank(inpatientAreaSel)){
					mrqcOperatorInfoMapper.updateMrqcOperatorInfo(operatorInfo);
				}
			}else {
				// INFO表中插入一条质控员的数据
				mrqcOperatorInfoMapper.insertMrqcOperatorInfo(operatorInfo);
			}
			//用map判断质控员是否已存在
			hashMap.put(zkySel,operatorInfo);
			operatorInfoList.add(operatorInfo);
		}
		//选择的质控员
		// 获取上个月的质控员，作为这个月的质控员列表
		String controlMonth = assignEMRToolService.calculateQualityControlMonth(qcMonth);
		MrqcReportMonth mrqcReportMonth = new MrqcReportMonth();
		mrqcReportMonth.setQcMonth(controlMonth);
		List<MrqcReportMonth> reportMonths = mrqcReportMonthService.selectMrqcReportMonthList(mrqcReportMonth);
		if (CollectionUtils.isNotEmpty(reportMonths)) {
			for (MrqcReportMonth month : reportMonths) {
				MrqcOperatorInfo operatorInfo = new MrqcOperatorInfo();
				if(!hashMap.containsKey(month.getUserId())){
					//不存在,添加
					operatorInfo.setUserId(month.getUserId());
					operatorInfoList.add(operatorInfo);
				}
			}
		}

		return operatorInfoList;
	}

	@Override
	public List<EmrRecord> assignEmrToOperator(MrqcOperatorInfo currOperator, MrqcReportMonth reportMonth,
											   List<MrqcOperatorInfo> mrqcOpts) {
		// 每个质控员质控病例数，默认每个质控员20份
		int emrsPerMonth=Integer.parseInt(mrqcEmrConfigService.selectMrqcEmrConfigByKey(JmsConstant.EMRS_PERDAY).getConfigValue());

		// 获取该质控员需要质控的科室
		SysUser sysUser = sysUserMapper.selectUserById(currOperator.getUserId());
		if(sysUser == null) {
			return null;
		}

		MrqcOperatorRecord record = new MrqcOperatorRecord();
		record.setUserId(currOperator.getUserId());
		record.setReportId(reportMonth.getReportId());
		List<MrqcOperatorRecord> records = mrqcOperatorRecordMapper.selectMrqcOperatorRecordList(record);
		// if (CollectionUtils.isNotEmpty(records) && records.size() >= emrsPerMonth) {
		// 	return new ArrayList<>();
		// } else {
		// 	emrsPerMonth = emrsPerMonth - records.size();
		// }

		List<EmrRecord> result = new ArrayList<>();

		String dataScope = " AND (u.inpatient_area = '"+currOperator.getOperationDept()+"') ";
		if(StringUtils.isNotBlank(currOperator.getMedicalGroup())){
			//病区不为空
			dataScope = "  AND (u.inpatient_area = '"+currOperator.getOperationDept()+"') AND department_name = '"+currOperator.getMedicalGroup()+"' ";
		}

		List<Map<String,Object>> list = autoAssignMapper.selecAllDeptsurgicalOperateList(reportMonth.getQcMonth(),sysUser.getUserId(),dataScope);

		if (CollectionUtils.isNotEmpty(list)) {
			if (list.size() > emrsPerMonth) {
				for(int i=0;i<emrsPerMonth;i++) {
					Map<String,Object> map = list.get(i);
					String recordId = (String)map.get("record_id");
					String inTime = String.valueOf(map.get("in_time"));
					String outTime = String.valueOf(map.get("out_time"));
					String hospitDoctor = (String)map.get("hospit_Doctor");
					EmrRecord emrRecord = new EmrRecord();
					emrRecord.setRecordId(recordId);
					try {
						emrRecord.setInTime(DateUtils.stringToDate(inTime, DateUtils.DATE_TIME_FORMAT));
						emrRecord.setOutTime(DateUtils.stringToDate(outTime, DateUtils.DATE_TIME_FORMAT));
					} catch (Exception e) {
						e.printStackTrace();
					}
					emrRecord.setHospitDoctor(hospitDoctor);
					result.add(emrRecord);
				}
			} else {
				for(Map<String,Object> map:list) {
					String recordId = (String)map.get("record_id");
					String inTime = String.valueOf(map.get("in_time"));
					String outTime = String.valueOf(map.get("out_time"));
					String hospitDoctor = (String)map.get("hospit_Doctor");
					EmrRecord emrRecord = new EmrRecord();
					emrRecord.setRecordId(recordId);
					try {
						emrRecord.setInTime(DateUtils.stringToDate(inTime, DateUtils.DATE_TIME_FORMAT));
						emrRecord.setOutTime(DateUtils.stringToDate(outTime, DateUtils.DATE_TIME_FORMAT));
					} catch (Exception e) {
						e.printStackTrace();
					}
					emrRecord.setHospitDoctor(hospitDoctor);
					result.add(emrRecord);
				}
			}
		}
		return result;
	}

	private static int getRandomNumberInRange(int min, int max) {
		if (min >= max) {
			throw new IllegalArgumentException("max must be greater than min");
		}
		Random r = new Random();
		return r.nextInt((max - min) + 1) + min;
	}

	private String getConfigValue(List<SysConfig> configs, String configKey, String defaultValue) {
		for (SysConfig config : configs) {
			if (configKey.equals(config.getConfigKey())) {
				return StringUtils.isNotBlank(config.getConfigValue()) ? config.getConfigValue() : defaultValue;
			}
		}
		return defaultValue;
	}

	@Override
	public List<MrqcOperatorRecord> builderMrqcOperatorRecord(MrqcOperatorInfo currOperator,
															  MrqcReportMonth reportMonth, List<EmrRecord> emrRecords) {
		List<MrqcOperatorRecord> recordList = new ArrayList<>();
		if(emrRecords != null) {
			for (EmrRecord emrRecord : emrRecords) {
				MrqcOperatorRecord record = new MrqcOperatorRecord();
				record.setReportId(reportMonth.getReportId());
				record.setRecordId(emrRecord.getRecordId());
				record.setUserId(reportMonth.getUserId());
				record.setApproved(Constant.UN_APPROVED);
				record.setCreater(reportMonth.getUserId() + "");
				record.setCreateTime(new Date());
				record.setAdmissionTime(simpleFormatter.format(emrRecord.getInTime()));
				record.setDischargeTime(simpleFormatter.format(emrRecord.getOutTime()));
				record.setHomepagePhysician(emrRecord.getHospitDoctor());

				recordList.add(record);
			}
		}

		return recordList;
	}

	@Override
	@Transactional
	public void autoAssignEMRTemplate(String qcMonth) {
		logger.info("开始进行通大病例数据分配任务...");
		String controlMonth = super.obtainQualityControlMonth(qcMonth);
		//质控员列表
		List<MrqcOperatorInfo> mrqcOpts = obtainOperator(qcMonth);
		if(CollectionUtils.isNotEmpty(mrqcOpts)){
			for(MrqcOperatorInfo operator : mrqcOpts){
				// 记录质控员的质控月信息
				MrqcReportMonth reportMonth = assignEMRToolService.recordQCmonthlyInformation(operator, qcMonth);
				if (operator.getUserId() != null){
					if(hook(operator)){
						// 获取质控员的质控病历列表
						List<EmrRecord> emrRecords = assignEmrToOperator(operator, reportMonth, mrqcOpts);
						// 组建质控病历记录信息
						List<MrqcOperatorRecord> records = builderMrqcOperatorRecord(operator, reportMonth, emrRecords);
						// 记录质控病历列表
						assignEMRToolService.recordMonthQcEmrList(records);
					}
				}
			}
		}
	}





}
