package com.hanyun.hip.mrqc.jms.service.extractor;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.hanyun.hip.mrqc.rule.entity.fact.MrqcStructData;
import com.hanyun.hip.mrqc.service.entity.EmrContent;
import com.hanyun.hip.mrqc.service.entity.MrqcStructExtract;
import com.hanyun.hip.mrqc.service.service.CustomStructExtracter;
import com.hanyun.hip.mrqc.service.util.JsonUtil;
import com.hanyun.hip.mrqc.service.util.UUidUtil;
import com.jayway.jsonpath.JsonPath;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * ------------------------------------------------------------------------<br>
 *  (or): 表示由多个表达式时，如果有一个有值就退出，否则一直往下取值
 *  --> 表示对结果集需要做额外的出来：
 *     - [join] 表示对结果集进行全部拼接，获取的值是数组的时候，多个值拼接成一个
 *     - [PRE]  表示要在结构的前面添加额外的标识，该标识由参数固定
 *     - [END]  与[PRE]相对，表示要在结果的末尾添加额外的标识，该标识由参数固定
 *     - [DEL]  后续的表达式，以逗号隔开，进行遍历删除，删除首尾
 *     - [INDEX[1]] 若取值为列表时，获取指定下标的值,下标从 0 开始
 *  (and): 表示由多个表达式同时取出结果，并进行结果的拼接
 $.ScatterData.Component.Section[?(@.Code.Code=="S004")].Composite[?(@.Code.Code=="V006")].StyleText-->[join](and)$.ScatterData.Component.Section[?(@.Code.Code=="S004")].Composite[?(@.Code.Code=="V006")].StyleText
 [AAA,BBB] = AAABBB
 *
 * @描述: JaywayJsonPathExtractor
 * <br>---------------------------------------------
 * @时间: 2024/3/26
 * @版本: 1.0
 * @创建人: Yurl
 */
public class JaywayJsonPathExtractor implements CustomStructExtracter {

    private static final String RESULT_ORIENTED_CHARACTERS = "-->";

    private static final String MULTIPLE_EXPRESSION = "(and)";

    private static final String OR_EXPRESSION = "(or)";

    @Override
    public MrqcStructData extract(MrqcStructExtract struct, EmrContent content, MrqcStructData parent) {
        try {
            parent = parent == null ? new MrqcStructData() : parent;
            String structJson = content.getStructJson();
            String jsonPath = struct.getExtScript();
            if (StringUtils.isEmpty(jsonPath) || StringUtils.isEmpty(structJson)) {
                return parent;
            }

            List<String> expressList = new ArrayList<>();
            if (jsonPath.contains(OR_EXPRESSION)){
                expressList = Arrays.asList(jsonPath.split("\\(or\\)"));
            } else {
                expressList.add(jsonPath);
            }

            String resultObj = "";
            for (String express : expressList){
                List<String> jsonPathList = new ArrayList<>();
                if (express.contains(MULTIPLE_EXPRESSION)) {
                    jsonPathList = Arrays.asList(express.split("\\(and\\)"));
                } else {
                    jsonPathList.add(express);
                }
                resultObj = getJsonPathValue(structJson, jsonPathList);
                // 去除转义等字符
                resultObj = StringEscapeUtils.unescapeJava(resultObj);
                if (StrUtil.isNotBlank(resultObj) && !StrUtil.equalsIgnoreCase("null", resultObj)){
                    break;
                }
            }
            parent.setDataValue(resultObj);
            parent.setDataId(UUidUtil.getUUID());
            parent.setContentId(content.getContentId());
            parent.setRecordId(content.getRecordId());
            parent.setStructId(struct.getStructId());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return parent;
    }

    public static String getJsonPathValue(String structJson, List<String> jsonPathList) {

        StringBuilder resultHandler = new StringBuilder();
        for (String jsonPath : jsonPathList) {
            String resultHandPath = null;
            if (jsonPath.contains(RESULT_ORIENTED_CHARACTERS)) {
                String[] split = jsonPath.split(RESULT_ORIENTED_CHARACTERS);
                jsonPath = split[0];
                resultHandPath = split[1];
            }
            // 格式化
            jsonPath = JsonUtil.transferToJsonPath(jsonPath);
            // 读取JSON数据
            Object resultObj = JsonPath.read(structJson, jsonPath);

            String result = arrayResultHandler(resultObj, resultHandPath);
            resultHandler.append(result);
        }
        return resultHandler.toString();
    }

    private static String arrayResultHandler(Object resultObj, String resultHandPath) {

        // 默认去掉首尾空格、双引号
        if (ObjectUtil.isEmpty(resultObj)) {
            return "";
        }
        String resultString = null;
        if (resultObj instanceof String) {
            resultString = resultObj.toString();
            resultString = valueFilter(resultString);
        } else if (resultObj instanceof ArrayList) {
            ArrayList arrayList = (ArrayList) resultObj;
            if (StrUtil.equalsIgnoreCase(resultHandPath, "[join]")) {
                StringBuilder stringBuilder = new StringBuilder();
                for (Object result : arrayList) {
                    stringBuilder.append("" + valueFilter(StrUtil.toString(result)));
                }
                resultString = stringBuilder.toString();
            } else if (StrUtil.startWith(resultHandPath, "[INDEX[")){
                int index = Integer.parseInt(resultHandPath.substring(7, resultHandPath.length() - 2));
                resultString = valueFilter(arrayList.size() < index ? "" : arrayList.get(index).toString());
            } else {
                resultString = valueFilter(StrUtil.toString(arrayList.get(0)));
            }
        } else {
            resultString = StrUtil.toString(resultObj);
        }
        if (StrUtil.isNotBlank(resultHandPath)) {
            if (resultHandPath.startsWith("[PRE]")) {
                // 添加后面的字符
                resultString = resultHandPath.substring(5) + resultString;
            } else if (resultHandPath.startsWith("[END]")) {
                // 添加后面的字符
                resultString = resultString + resultHandPath.substring(5);
            } else if (resultHandPath.startsWith("[DEL]")) {
                String handExpress = resultHandPath.substring(5);
                for (String express : handExpress.split(",")) {
                    resultString = StrUtil.removeSuffix(resultString, express);
                    resultString = StrUtil.removePrefix(resultString, express);
                }
            }else if(resultHandPath.startsWith("[SUBSTR]")){
                String handExpress = resultHandPath.substring(8);
                String[] flag = handExpress.split(",");
                if(flag.length>=2){
                    String start = flag[0];
                    String end = flag[1];
                    resultString =  resultString.substring(resultString.indexOf(start)+start.length(),resultString.lastIndexOf(end));
                }else {
                    resultString = resultString.substring(resultString.indexOf(flag[0])+flag[0].length(),resultString.length());
                }

            }
        }

        return resultString;
    }

    private static String valueFilter(String resultString) {
        if (StrUtil.isBlank(resultString)) {
            return "";
        }
        resultString = StrUtil.removeSuffix(resultString, "\"");
        resultString = StrUtil.removePrefix(resultString, "\"");
        if (StrUtil.equalsIgnoreCase("null", StrUtil.trim(resultString))) {
            return "";
        }
        return resultString;
    }
}
