package com.hanyun.hip.mrqc.jms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.hanyun.hip.mrqc.jms.entity.HomePigeonhole;
import com.hanyun.hip.mrqc.jms.entity.HomePigeonholeSum;
import com.hanyun.hip.mrqc.jms.entity.ReportRecordDTO;
import com.hanyun.hip.mrqc.jms.mapper.MrqcEmrConfigMapper;
import com.hanyun.hip.mrqc.service.entity.EmrRecord;
import com.hanyun.hip.mrqc.service.mapper.IEmrRecordMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * @ClassName：HomePigeonholeService
 * @author: ouyang0810@foxmail.com
 * @CreateDate: 2024/11/5
 * @UpdateUser: ouyang0810@foxmail.com
 * @UpdateDate: 2024/11/5
 * @UpdateRemark:
 * @Description:
 * @Version: [V1.0]
 */
@Component
public class HomePigeonholeService {

    @Autowired
    private IEmrRecordMapper iEmrRecordMapper;

    @Autowired
    private MrqcEmrConfigMapper mrqcEmrConfigMapper;

    public List<HomePigeonhole> getHomePigeonholeList(String s1, String s2, Map<String, Object> map) {
        List<HomePigeonhole> homePigeonholes = new ArrayList<>();
        EmrRecord emrRecord = new EmrRecord();
        emrRecord.setParams(map);
        String beginDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String departmentName = (String) map.get("departmentName");
        String doctorName = (String) map.get("doctorName");
        if (ObjectUtil.isEmpty(beginDate) && ObjectUtil.isEmpty(endDate)) {
            emrRecord.setBeginTime(DateUtil.beginOfMonth(new Date()));
            emrRecord.setEndTime(DateUtil.endOfMonth(new Date()));
        }
        if (ObjectUtil.isNotEmpty(beginDate)) {
            emrRecord.setBeginTime(DateUtil.parse(beginDate));
        }
        if (ObjectUtil.isNotEmpty(endDate)) {
            emrRecord.setEndTime(DateUtil.parse(endDate));
        }

        if (ObjectUtil.isNotEmpty(departmentName)) {
            emrRecord.setInpatientArea(departmentName);
        }

        List<EmrRecord> emrRecords = mrqcEmrConfigMapper.selectEmrRecordList(emrRecord);
        Map<String, List<EmrRecord>> collect = emrRecords.stream().collect(Collectors.groupingBy(EmrRecord :: getInpatientArea));
        AtomicInteger twoDayCountSum = new AtomicInteger();
        AtomicInteger twoDayNotCountSum = new AtomicInteger();
        AtomicInteger threeDayCountSum = new AtomicInteger();
        AtomicInteger fourDayCountSum = new AtomicInteger();
        AtomicInteger fiveDayCountSum = new AtomicInteger();
        AtomicInteger moreThanSevenDayCountSum = new AtomicInteger();
        AtomicInteger sevenDayCountSum = new AtomicInteger();
        for (Map.Entry<String, List<EmrRecord>> entry : collect.entrySet()) {
            List<EmrRecord> records = collect.get(entry.getKey());
            HomePigeonhole homePigeonhole = new HomePigeonhole();
            homePigeonhole.setBeginDate(DateUtil.formatDate(emrRecord.getBeginTime()));
            homePigeonhole.setEndDate(DateUtil.formatDate(emrRecord.getEndTime()));

            homePigeonhole.setInpatientArea(entry.getKey());
            homePigeonhole.setHospitCount(records.size());
            int twoDayCount = 0;
            int twoDayNotCount = 0;
            int threeDayCount = 0;
            int fourDayCount = 0;
            int fiveDayCount = 0;
            int moreThanSevenDayCount = 0;
            int sevenDayCount = 0;
            for (EmrRecord record : records) {
                //归档时间
                Date etlTime = record.getEtlTime();
                Date outTime = record.getOutTime();
                if (ObjectUtil.isNotEmpty(etlTime)) {
                    long day = DateUtil.betweenDay(etlTime, outTime, false);
                    if (day <= 2) {
                        twoDayCount++;
                        twoDayCountSum.getAndIncrement();
                    }
                    if (day > 2) {
                        twoDayNotCount++;
                        twoDayNotCountSum.getAndIncrement();
                    }
                    if (day <= 3) {
                        threeDayCount++;
                        threeDayCountSum.getAndIncrement();
                    }
                    if (day <= 4) {
                        fourDayCount++;
                        fourDayCountSum.getAndIncrement();
                    }
                    if (day <= 5) {
                        fiveDayCount++;
                        fiveDayCountSum.getAndIncrement();
                    }
                    if (day <= 7) {
                        sevenDayCount++;
                        sevenDayCountSum.getAndIncrement();
                    }
                    if (day > 7) {
                        moreThanSevenDayCount++;
                        moreThanSevenDayCountSum.getAndIncrement();
                    }
                }else {
                    twoDayNotCount++;
                    twoDayNotCountSum.getAndIncrement();
                }
            }
            homePigeonhole.setTwoDayCount(twoDayCount);
            homePigeonhole.setTwoDayPigeonhole(BigDecimal.valueOf(twoDayCount * 100).divide(BigDecimal.valueOf(records.size()), 2, RoundingMode.HALF_UP).toString());
            homePigeonhole.setTwoDayNotCount(twoDayNotCount);
            homePigeonhole.setThreeDayCount(threeDayCount);
            homePigeonhole.setThreeDayPigeonhole(BigDecimal.valueOf(threeDayCount * 100).divide(BigDecimal.valueOf(records.size()), 2, RoundingMode.HALF_UP).toString());
            homePigeonhole.setFourDayCount(fourDayCount);
            homePigeonhole.setFourDayPigeonhole(BigDecimal.valueOf(fourDayCount * 100).divide(BigDecimal.valueOf(records.size()), 2, RoundingMode.HALF_UP).toString());
            homePigeonhole.setFiveDayCount(fiveDayCount);
            homePigeonhole.setFiveDayPigeonhole(BigDecimal.valueOf(fiveDayCount * 100).divide(BigDecimal.valueOf(records.size()), 2, RoundingMode.HALF_UP).toString());
            homePigeonhole.setSevenDayCount(sevenDayCount);
            homePigeonhole.setSevenDayPigeonhole(BigDecimal.valueOf(sevenDayCount * 100).divide(BigDecimal.valueOf(records.size()), 2, RoundingMode.HALF_UP).toString());
            homePigeonhole.setMoreThanSevenDayCount(moreThanSevenDayCount);
            homePigeonhole.setMoreThanDayPigeonhole(BigDecimal.valueOf(moreThanSevenDayCount * 100).divide(BigDecimal.valueOf(records.size()), 2, RoundingMode.HALF_UP).toString());
            homePigeonholes.add(homePigeonhole);
        }
        return homePigeonholes;
    }

    public List<HomePigeonholeSum> getHomePigeonholeSum(String s1, String s2, Map<String, Object> map) {
        List<HomePigeonholeSum> homePigeonholeSums = new ArrayList<>();
        HomePigeonholeSum homePigeonholeSum = new HomePigeonholeSum();
        EmrRecord emrRecord = new EmrRecord();
        emrRecord.setParams(map);
        String beginDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String departmentName = (String) map.get("departmentName");
        String doctorName = (String) map.get("doctorName");
        if (ObjectUtil.isEmpty(beginDate) && ObjectUtil.isEmpty(endDate)) {
            emrRecord.setBeginTime(DateUtil.beginOfMonth(new Date()));
            emrRecord.setEndTime(DateUtil.endOfMonth(new Date()));
        }
        if (ObjectUtil.isNotEmpty(beginDate)) {
            emrRecord.setBeginTime(DateUtil.parse(beginDate));
        }
        if (ObjectUtil.isNotEmpty(endDate)) {
            emrRecord.setEndTime(DateUtil.parse(endDate));
        }

        if (ObjectUtil.isNotEmpty(departmentName)) {
            emrRecord.setInpatientArea(departmentName);
        }

        List<EmrRecord> emrRecords = mrqcEmrConfigMapper.selectEmrRecordList(emrRecord);
        if (CollUtil.isEmpty(emrRecords)) {
            return homePigeonholeSums;
        }
        Map<String, List<EmrRecord>> collect = emrRecords.stream().collect(Collectors.groupingBy(EmrRecord :: getInpatientArea));
        AtomicInteger twoDayCountSum = new AtomicInteger();
        AtomicInteger twoDayNotCountSum = new AtomicInteger();
        AtomicInteger threeDayCountSum = new AtomicInteger();
        AtomicInteger fourDayCountSum = new AtomicInteger();
        AtomicInteger fiveDayCountSum = new AtomicInteger();
        AtomicInteger moreThanSevenDayCountSum = new AtomicInteger();
        AtomicInteger sevenDayCountSum = new AtomicInteger();
        for (Map.Entry<String, List<EmrRecord>> entry : collect.entrySet()) {
            List<EmrRecord> records = collect.get(entry.getKey());
            Map<String, List<EmrRecord>> dateRecords = records.stream().collect(Collectors.groupingBy(o -> DateUtil.format(o.getOutTime(), "yyyy-MM")));
            dateRecords.forEach((k, v) -> {
                int twoDayCount = 0;
                int twoDayNotCount = 0;
                int threeDayCount = 0;
                int fourDayCount = 0;
                int fiveDayCount = 0;
                int moreThanSevenDayCount = 0;
                int sevenDayCount = 0;
                for (EmrRecord record : v) {
                    //归档时间
                    Date etlTime = record.getEtlTime();
                    Date outTime = record.getOutTime();
                    if (ObjectUtil.isNotEmpty(etlTime)) {
                        long day = DateUtil.betweenDay(etlTime, outTime, false);
                        if (day <= 2) {
                            twoDayCount++;
                            twoDayCountSum.getAndIncrement();
                        }
                        if (day > 2) {
                            twoDayNotCount++;
                            twoDayNotCountSum.getAndIncrement();
                        }
                        if (day <= 3) {
                            threeDayCount++;
                            threeDayCountSum.getAndIncrement();
                        }
                        if (day <= 4) {
                            fourDayCount++;
                            fourDayCountSum.getAndIncrement();
                        }
                        if (day <= 5) {
                            fiveDayCount++;
                            fiveDayCountSum.getAndIncrement();
                        }
                        if (day <= 7) {
                            sevenDayCount++;
                            sevenDayCountSum.getAndIncrement();
                        }
                        if (day > 7) {
                            moreThanSevenDayCount++;
                            moreThanSevenDayCountSum.getAndIncrement();
                        }
                    }else {
                        twoDayNotCount++;
                        twoDayNotCountSum.getAndIncrement();
                    }
                }

            });


        }
        homePigeonholeSum.setTwoDayCountSum(twoDayCountSum.toString());
        homePigeonholeSum.setTwoDayNotCountSum(twoDayNotCountSum.toString());
        homePigeonholeSum.setThreeDayCountSum(threeDayCountSum.toString());
        homePigeonholeSum.setFourDayCountSum(fourDayCountSum.toString());
        homePigeonholeSum.setFiveDayCountSum(fiveDayCountSum.toString());
        homePigeonholeSum.setSevenDayCountSum(sevenDayCountSum.toString());
        homePigeonholeSum.setMoreThanSevenDayCountSum(moreThanSevenDayCountSum.toString());
        homePigeonholeSum.setHospitCountSum(emrRecords.size() + "");
        homePigeonholeSum.setTwoDayPigeonholeSum(BigDecimal.valueOf(twoDayCountSum.get() * 100).divide(BigDecimal.valueOf(emrRecords.size()), 2, RoundingMode.HALF_UP).toString());
        homePigeonholeSum.setThreeDayPigeonholeSum(String.valueOf(BigDecimal.valueOf(threeDayCountSum.get() * 100).divide(BigDecimal.valueOf(emrRecords.size()), 2, RoundingMode.HALF_UP)));
        homePigeonholeSum.setFourDayPigeonholeSum(String.valueOf(BigDecimal.valueOf(fourDayCountSum.get() * 100).divide(BigDecimal.valueOf(emrRecords.size()), 2, RoundingMode.HALF_UP)));
        homePigeonholeSum.setFiveDayPigeonholeSum(String.valueOf(BigDecimal.valueOf(fiveDayCountSum.get() * 100).divide(BigDecimal.valueOf(emrRecords.size()), 2, RoundingMode.HALF_UP)));
        homePigeonholeSum.setSevenDayPigeonholeSum(String.valueOf(BigDecimal.valueOf(sevenDayCountSum.get() * 100).divide(BigDecimal.valueOf(emrRecords.size()), 2, RoundingMode.HALF_UP)));
        homePigeonholeSum.setMoreThanDayPigeonholeSum(String.valueOf(BigDecimal.valueOf(moreThanSevenDayCountSum.get() * 100).divide(BigDecimal.valueOf(emrRecords.size()), 2, RoundingMode.HALF_UP)));
        homePigeonholeSums.add(homePigeonholeSum);

        return homePigeonholeSums;
    }

    public List<ReportRecordDTO> getreportRecordList(String s1, String s2, Map<String, Object> map) {
        List<ReportRecordDTO> reportRecordDTOS = new ArrayList<>();
        List<HomePigeonhole> homePigeonholes = new ArrayList<>();
        EmrRecord emrRecord = new EmrRecord();
        emrRecord.setParams(map);
        String beginDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String departmentName = (String) map.get("departmentName");
        String doctorName = (String) map.get("doctorName");
        if ("null".equals(beginDate)) {
            beginDate = null;
        }
        if ("null".equals(endDate)) {
            endDate = null;
        }
        String type = (String) map.get("type");
        if (ObjectUtil.isEmpty(beginDate) && ObjectUtil.isEmpty(endDate)) {
            emrRecord.setBeginTime(DateUtil.beginOfMonth(new Date()));
            emrRecord.setEndTime(DateUtil.endOfMonth(new Date()));
        }
        if (ObjectUtil.isNotEmpty(beginDate)) {
            emrRecord.setBeginTime(DateUtil.parse(beginDate));
        }
        if (ObjectUtil.isNotEmpty(endDate)) {
            emrRecord.setEndTime(DateUtil.parse(endDate));
        }

        if (ObjectUtil.isNotEmpty(departmentName)) {
            emrRecord.setInpatientArea(departmentName);
        }

        List<EmrRecord> emrRecords = mrqcEmrConfigMapper.selectEmrRecordNotLikeList(emrRecord);
        for (EmrRecord record : emrRecords) {
            Date etlTime = record.getEtlTime();
            Date outTime = record.getOutTime();
            long day = - 1;
            if (ObjectUtil.isNotEmpty(etlTime)) {
                day = DateUtil.betweenDay(etlTime, outTime, false);
            }
            ReportRecordDTO reportRecordDTO = new ReportRecordDTO();

            reportRecordDTO.setRecordId(record.getRecordId());
            reportRecordDTO.setPatientName(record.getPatientName());
            reportRecordDTO.setInpatientArea(record.getInpatientArea());
            reportRecordDTO.setDiagnosisName(record.getDiagnosisName());
            reportRecordDTO.setHospitNum(record.getHospitNum());
            reportRecordDTO.setHospitDoctor(record.getHospitDoctor());
            reportRecordDTO.setInTime(DateUtil.formatDateTime(record.getInTime()));
            reportRecordDTO.setOutTime(DateUtil.formatDateTime(record.getOutTime()));

            switch (type) {
                case "1":
                    reportRecordDTOS.add(reportRecordDTO);
                    break;

                case "2":
                    if (day != - 1 && day <= 2) {
                        reportRecordDTOS.add(reportRecordDTO);
                    }
                    break;
                case "3":
                    if ( day > 2) {
                        reportRecordDTOS.add(reportRecordDTO);
                    }
                    break;
                case "4":
                    if (day != - 1 && day <= 3) {
                        reportRecordDTOS.add(reportRecordDTO);
                    }
                    break;
                case "5":
                    if (day != - 1 && day <= 4) {
                        reportRecordDTOS.add(reportRecordDTO);
                    }
                    break;
                case "6":
                    if (day != - 1 && day <= 5) {
                        reportRecordDTOS.add(reportRecordDTO);
                    }
                    break;
                case "7":
                    if (day != - 1 && day <= 7) {
                        reportRecordDTOS.add(reportRecordDTO);
                    }
                    break;
                case "8":
                    if (day != - 1 && day > 7) {
                        reportRecordDTOS.add(reportRecordDTO);
                    }
                    break;
                default:
                    break;
            }
        }
        return reportRecordDTOS;
    }

    public List<ReportRecordDTO> getRecordCommitList(String s1, String s2, Map<String, Object> map) {
        List<ReportRecordDTO> reportRecordDTOS = new ArrayList<>();
        List<HomePigeonhole> homePigeonholes = new ArrayList<>();
        EmrRecord emrRecord = new EmrRecord();
        emrRecord.setParams(map);
        String beginDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String departmentName = (String) map.get("departmentName");
        String doctorName = (String) map.get("doctorName");
        if ("null".equals(beginDate)) {
            beginDate = null;
        }
        if ("null".equals(endDate)) {
            endDate = null;
        }
        String type = (String) map.get("type");
        if (ObjectUtil.isEmpty(beginDate) && ObjectUtil.isEmpty(endDate)) {
            emrRecord.setBeginTime(DateUtil.beginOfMonth(new Date()));
            emrRecord.setEndTime(DateUtil.endOfMonth(new Date()));
        }
        if (ObjectUtil.isNotEmpty(beginDate)) {
            emrRecord.setBeginTime(DateUtil.parse(beginDate));
        }
        if (ObjectUtil.isNotEmpty(endDate)) {
            emrRecord.setEndTime(DateUtil.parse(endDate));
        }

        if (ObjectUtil.isNotEmpty(departmentName)) {
            emrRecord.setInpatientArea(departmentName);
        }
        if (ObjectUtil.isNotEmpty(doctorName)) {
            emrRecord.setHospitDoctor(doctorName);
        }
        List<EmrRecord> emrRecords = mrqcEmrConfigMapper.selectEmrRecordCommitList(emrRecord);
        for (EmrRecord record : emrRecords) {
            Date etlTime = record.getEtlTime();
            Date outTime = record.getOutTime();
            if (ObjectUtil.isEmpty(etlTime)) {
                continue;
            }
            ReportRecordDTO reportRecordDTO = new ReportRecordDTO();

            reportRecordDTO.setRecordId(record.getRecordId());
            reportRecordDTO.setPatientName(record.getPatientName());
            reportRecordDTO.setInpatientArea(record.getInpatientArea());
            reportRecordDTO.setDiagnosisName(record.getDiagnosisName());
            reportRecordDTO.setHospitNum(record.getHospitNum());
            reportRecordDTO.setHospitDoctor(record.getHospitDoctor());
            reportRecordDTO.setInTime(DateUtil.formatDateTime(record.getInTime()));
            reportRecordDTO.setOutTime(DateUtil.formatDateTime(record.getOutTime()));
            reportRecordDTO.setEtlTime(DateUtil.formatDateTime(record.getEtlTime()));
            reportRecordDTOS.add(reportRecordDTO);
        }
        return reportRecordDTOS;
    }
}
