package com.hanyun.hip.mrqc.jms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.hanyun.hip.mrqc.common.EmrETLApi;
import com.hanyun.hip.mrqc.common.task.AdmissionAndDaysDischargeETLTask;
import com.hanyun.hip.mrqc.jms.api.JmsEmrETLApi;
import com.hanyun.hip.mrqc.rule.constant.Constant;
import com.hanyun.hip.mrqc.rule.util.SpringUtil;
import com.hanyun.hip.mrqc.service.service.IMrqcEtlService;
import com.hanyun.hip.mrqc.service.util.XmlTool;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import org.apache.commons.collections.CollectionUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.util.EntityUtils;
import org.dom4j.DocumentException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import javax.net.ssl.SSLContext;
import java.nio.charset.Charset;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;


@Component("JmsCommonTask")
public class JmsCommonTask {

    private static final Logger logger = LoggerFactory.getLogger(JmsCommonTask.class);

    @Autowired
    IMrqcEtlService mrqcEtlService;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    private static final String WEBSERVICE_XML = "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:dhcc=\"http://www.dhcc.com.cn\">\n" +
            "<soapenv:Header/>\n" +
            "<soapenv:Body>\n" +
            "    <dhcc:GetHISInfo>\n" +
            "        <dhcc:action>%s</dhcc:action>\n" +
            "        <dhcc:Input><![CDATA[%s]]></dhcc:Input>\n" +
            "    </dhcc:GetHISInfo>\n" +
            "</soapenv:Body>\n" +
            "</soapenv:Envelope>";

    private static String doPostSoap(String postUrl, String requestCode, String requestParam, String soapAction) {
        int socketTimeout = 60000;// 请求超时时间
        int connectTimeout = 60000;// 传输超时时间
        String retStr = "";
        // 创建HttpClientBuilder
        HttpClientBuilder httpClientBuilder = HttpClientBuilder.create();
        try {
            //忽略SSL证书(https时需要证书)
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, (arg0, arg1) -> true).build();
            httpClientBuilder.setSSLContext(sslContext);
            httpClientBuilder.setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE);
        } catch (KeyManagementException | NoSuchAlgorithmException | KeyStoreException e) {
            throw new RuntimeException("Could not disable SSL on HttpClient builder.", e);
        }
        // HttpClient
        CloseableHttpClient closeableHttpClient = httpClientBuilder.build();
        HttpPost httpPost = new HttpPost(postUrl);
        //  设置请求和传输超时时间
        RequestConfig requestConfig = RequestConfig.custom()
                .setSocketTimeout(socketTimeout)
                .setConnectTimeout(connectTimeout).build();
        httpPost.setConfig(requestConfig);
        try {
            httpPost.setHeader("Content-Type", "text/xml;charset=UTF-8");
            httpPost.setHeader("SOAPAction", soapAction);
            StringEntity data = new StringEntity(String.format(WEBSERVICE_XML, requestCode, requestParam), Charset.forName("UTF-8"));
            httpPost.setEntity(data);
            CloseableHttpResponse response = closeableHttpClient
                    .execute(httpPost);
            HttpEntity httpEntity = response.getEntity();
            if (httpEntity != null) {
                // 打印响应内容
                retStr = EntityUtils.toString(httpEntity, "UTF-8");
                //LOGGER.info("response:" + retStr);
            }
            // 释放资源
            closeableHttpClient.close();
        } catch (Exception e) {
            logger.error("文书编号：{}；获取文书请求出错！ 异常信息：{}", e.getMessage());
        }

        return retStr;
    }
    public void refreshEtlTime() {
        String querySql = "SELECT record_id FROM mrqc_record WHERE auto is null or auto = ''";

        List<String> recordIds = jdbcTemplate.queryForList(querySql, String.class);
        List<Map<String,String>> mapList = new ArrayList<>();
        for (String recordId : recordIds) {
            String response = doPostSoap("http://172.17.0.206/csp/hsb/DHC.Published.PUB0010.BS.PUB0010.CLS", "", recordId, "http://www.dhcc.com.cn/DHC.Published.PUB0010.BS.PUB0010.GetHISInfo");
            try {
                JSONObject jsonObject = XmlTool.documentToJSONObject(response);
                JSONObject serviceResponse = jsonObject.getJSONObject("Body").getJSONObject("GetHISInfoResponse");
                if (null != serviceResponse && serviceResponse.size() != 0) {
                    String resultResponse = serviceResponse.getString("GetHISInfoResult");
                    JSONObject admFirstPage = XmlTool.documentToJSONObject(resultResponse);
                    String gdsj = admFirstPage.getString("GDSJ");
                    if (StrUtil.isNotEmpty(gdsj)) {
                        Map<String, String> map = new HashMap<>();
                        map.put("recordId", recordId);
                        map.put("gdsj", gdsj);
                        mapList.add(map);
                    }
                }
            }catch (DocumentException e) {
                logger.error("数据抽提, 病历recordId: {} ,病案数据XML转换异常! ExceptionMsg: {}", recordId, e.getMessage());
            }
        }
        if (CollUtil.isNotEmpty(mapList)) {
            String sql = "UPDATE mrqc_record SET etl_time = ? WHERE record_id = ?";

            jdbcTemplate.batchUpdate(sql, new BatchPreparedStatementSetter() {
                @Override
                public void setValues(PreparedStatement ps, int i) throws SQLException {
                    Map<String, String> map = mapList.get(i);
                    ps.setString(1, map.get("recordId"));           // 假设id是主键
                    ps.setString(2, map.get("gdsj")); // 假设columnName是要更新的字段
                }

                @Override
                public int getBatchSize() {
                    return mapList.size();
                }
            });
        }
    }

    @Log(title = "抽提未质控数据", businessType = BusinessType.OTHER)
    public void etlAutoisNullTask() {


        try {
            if (CollectionUtils.isNotEmpty(Constant.ETL_APIS)) {
                for (String etlApi : Constant.ETL_APIS) {
                    EmrETLApi emrETLApi = (EmrETLApi) SpringUtil.getBean(etlApi);
                    if (emrETLApi != null) {
                        //  先增量更新
                        String querySql = "SELECT record_id FROM mrqc_record WHERE auto is null or auto = ''";

                        List<String> recordIds = jdbcTemplate.queryForList(querySql, String.class);
                        if (CollectionUtils.isNotEmpty(recordIds)) {
                            ExecutorService executorService = Executors
                                    .newFixedThreadPool(10);
                            final CountDownLatch countDownLatch = new CountDownLatch(recordIds.size());
                            try {
                                for (int i = 0; i < recordIds.size(); i++) {
                                    String recordId = recordIds.get(i);
                                    try {
                                        emrETLApi.etlEmr(null, recordId, null, null, null, null, true, null, mrqcEtlService, 1, null);
                                        logger.info("==================》增量抽取未质控病历，完成进度：{}/{}", i, recordIds.size());
                                    } catch (Exception e) {
                                        logger.error("电子病历文书抽取异常，recordID={}", recordId);
                                    } finally {
                                        countDownLatch.countDown();
                                    }
                                }
                                // 保证所有的检查都完毕后才退出
                                countDownLatch.await();
                            } catch (InterruptedException ignore) {
                                logger.error("抽取未质控电子病历文书被中断。");
                            } finally {
                                executorService.shutdownNow();
                            }
                        }
                    }
                }
            }
        } catch (Exception e1) {
            e1.printStackTrace();
        }
    }
}
