package com.hanyun.hip.mrqc.jms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.hanyun.hip.mrqc.jms.entity.MrqcEmrConfig;
import com.hanyun.hip.mrqc.jms.entity.ProcessDTO;
import com.hanyun.hip.mrqc.jms.entity.ProcessYWKDTO;
import com.hanyun.hip.mrqc.jms.mapper.MrqcEmrConfigMapper;
import com.hanyun.hip.mrqc.jms.service.IMrqcEmrConfigService;
import com.hanyun.hip.mrqc.service.entity.MrqcProcess;
import com.ruoyi.common.core.text.Convert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * 电子病历规则分配 服务层实现
 *
 * @author ruoyi
 * @date 2024-04-22
 */
@Service
public class MrqcEmrConfigServiceImpl implements IMrqcEmrConfigService {
    @Autowired
    private MrqcEmrConfigMapper mrqcEmrConfigMapper;

    @Override
    public MrqcEmrConfig selectMrqcEmrConfigByKey(String configKey) {
        return mrqcEmrConfigMapper.selectMrqcEmrConfigByKey(configKey);
    }

    /**
     * 查询电子病历规则分配信息
     *
     * @param configId 电子病历规则分配ID
     * @return 电子病历规则分配信息
     */
    @Override
    public MrqcEmrConfig selectMrqcEmrConfigById(Long configId) {
        return mrqcEmrConfigMapper.selectMrqcEmrConfigById(configId);
    }

    /**
     * 查询电子病历规则分配列表
     *
     * @param mrqcEmrConfig 电子病历规则分配信息
     * @return 电子病历规则分配集合
     */
    @Override
    public List<MrqcEmrConfig> selectMrqcEmrConfigList(MrqcEmrConfig mrqcEmrConfig) {
        return mrqcEmrConfigMapper.selectMrqcEmrConfigList(mrqcEmrConfig);
    }

    /**
     * 新增电子病历规则分配
     *
     * @param mrqcEmrConfig 电子病历规则分配信息
     * @return 结果
     */
    @Override
    public int insertMrqcEmrConfig(MrqcEmrConfig mrqcEmrConfig) {
        return mrqcEmrConfigMapper.insertMrqcEmrConfig(mrqcEmrConfig);
    }

    /**
     * 修改电子病历规则分配
     *
     * @param mrqcEmrConfig 电子病历规则分配信息
     * @return 结果
     */
    @Override
    public int updateMrqcEmrConfig(MrqcEmrConfig mrqcEmrConfig) {
        return mrqcEmrConfigMapper.updateMrqcEmrConfig(mrqcEmrConfig);
    }

    /**
     * 删除电子病历规则分配对象
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteMrqcEmrConfigByIds(String ids) {
        return mrqcEmrConfigMapper.deleteMrqcEmrConfigByIds(Convert.toStrArray(ids));
    }


    @Override
    public List<ProcessYWKDTO> getProcessListByYWK(MrqcProcess mrqcProcess) {
        List<ProcessYWKDTO> processListByYWK = mrqcEmrConfigMapper.getProcessListByYWK(mrqcProcess);
        return processListByYWK;
    }

    @Override
    public List<ProcessDTO> getProcessList(MrqcProcess mrqcProcess) {

        List<ProcessDTO> processList = mrqcEmrConfigMapper.getProcessList(mrqcProcess);
        if (CollUtil.isNotEmpty(processList)) {
            for (ProcessDTO process : processList) {
                String commitTime = process.getCommitTime();
                String auditTime = process.getAuditTime();
                if (StrUtil.isEmpty(auditTime) || StrUtil.isEmpty(commitTime)) {
                    continue;
                }
                long diffInMillies = DateUtil.parse(commitTime).getTime() - DateUtil.parse(auditTime).getTime();
                long diff = Math.abs(diffInMillies);

                long days = TimeUnit.MILLISECONDS.toDays(diff);
                long hours = TimeUnit.MILLISECONDS.toHours(diff) % 24;
                long minutes = TimeUnit.MILLISECONDS.toMinutes(diff) % 60;
                String sign = diffInMillies >= 0 ? "+" : "-";
                process.setDiffTime("时间差: " + sign + days + " 天, " + hours + " 小时, " + minutes + " 分钟");
            }
        }

        return processList;
    }


}
