package com.hanyun.hip.mrqc.jms.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.hanyun.hip.mrqc.jms.entity.*;
import com.hanyun.hip.mrqc.jms.mapper.UReportExportMapper;
import com.hanyun.hip.mrqc.service.entity.EmrRecord;
import com.hanyun.hip.mrqc.service.mapper.IEmrRecordMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

@Component
public class UReportExportService {

    @Autowired
    private IEmrRecordMapper iEmrRecordMapper;

    @Autowired
    private UReportExportMapper uReportExportMapper;


    public List<UReportDeathDisplayDTO> getHomePigeonholeList(String s1, String s2, Map<String, Object> map) {

        List<UReportDeathDisplayDTO> result = new ArrayList<>();
        EmrRecord emrRecord = new EmrRecord();
        emrRecord.setParams(map);
        String beginDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String departmentName = (String) map.get("departmentName");
        String doctorName = (String) map.get("doctorName");
        if (ObjectUtil.isEmpty(beginDate) && ObjectUtil.isEmpty(endDate)) {
            emrRecord.setBeginTime(DateUtil.beginOfMonth(new Date()));
            emrRecord.setEndTime(DateUtil.endOfMonth(new Date()));
        }
        if (ObjectUtil.isNotEmpty(beginDate)) {
            emrRecord.setBeginTime(DateUtil.parse(beginDate));
        }
        if (ObjectUtil.isNotEmpty(endDate)) {
            emrRecord.setEndTime(DateUtil.parse(endDate));
        }

        if (ObjectUtil.isNotEmpty(departmentName)) {
            emrRecord.setInpatientArea(departmentName);
        }

        if (ObjectUtil.isNotEmpty(doctorName)) {
            emrRecord.setHospitDoctor(doctorName);
        }
        String type = (String) map.get("type");
        List<UReportDeathDTO> homePigeonholes = uReportExportMapper.selectDeathList(emrRecord);
        if (CollUtil.isEmpty(homePigeonholes)) {
            return new ArrayList<>();
        }
        if (StrUtil.isEmpty(type) || "全院".equals(type)) {
            long labelCount = homePigeonholes.stream()
                    .filter(homePigeonhole -> "27".equals(homePigeonhole.getLabelId()))
                    .count();
            long approvedCount = homePigeonholes.stream()
                    .filter(homePigeonhole -> "1".equals(homePigeonhole.getApproved()))
                    .count();
            long sumCount = homePigeonholes.size();
            UReportDeathDisplayDTO deathDisplayDTO = new UReportDeathDisplayDTO();
            deathDisplayDTO.setLabelCount(labelCount);
            deathDisplayDTO.setApproveCount(approvedCount);
            deathDisplayDTO.setSumCount(sumCount);
            deathDisplayDTO.setName("全院");
            deathDisplayDTO.setBeginDate(beginDate);
            deathDisplayDTO.setEndDate(endDate);
            deathDisplayDTO.setNameType(type);
            result.add(deathDisplayDTO);
        }
        if (StrUtil.isNotEmpty(type) && "科室".equals(type)) {
            Map<String, List<UReportDeathDTO>> collect = homePigeonholes.stream().collect(Collectors.groupingBy(UReportDeathDTO :: getInpatientArea));
            collect.forEach((key, value) -> {
                long labelCount = value.stream()
                        .filter(homePigeonhole -> "27".equals(homePigeonhole.getLabelId()))
                        .count();
                long approvedCount = value.stream()
                        .filter(homePigeonhole -> "1".equals(homePigeonhole.getApproved()))
                        .count();
                long sumCount = value.size();
                UReportDeathDisplayDTO deathDisplayDTO = new UReportDeathDisplayDTO();
                deathDisplayDTO.setLabelCount(labelCount);
                deathDisplayDTO.setApproveCount(approvedCount);
                deathDisplayDTO.setSumCount(sumCount);
                deathDisplayDTO.setName(key);
                deathDisplayDTO.setBeginDate(beginDate);
                deathDisplayDTO.setEndDate(endDate);
                deathDisplayDTO.setNameType(type);
                result.add(deathDisplayDTO);
            });
        }
        if (StrUtil.isNotEmpty(type) && "个人".equals(type)) {
            Map<String, List<UReportDeathDTO>> collect = homePigeonholes.stream().collect(Collectors.groupingBy(UReportDeathDTO :: getDoctorName));
            collect.forEach((key, value) -> {
                long labelCount = value.stream()
                        .filter(homePigeonhole -> "27".equals(homePigeonhole.getLabelId()))
                        .count();
                long approvedCount = value.stream()
                        .filter(homePigeonhole -> "1".equals(homePigeonhole.getApproved()))
                        .count();
                long sumCount = value.size();
                UReportDeathDisplayDTO deathDisplayDTO = new UReportDeathDisplayDTO();
                deathDisplayDTO.setLabelCount(labelCount);
                deathDisplayDTO.setApproveCount(approvedCount);
                deathDisplayDTO.setSumCount(sumCount);
                deathDisplayDTO.setName(key);
                deathDisplayDTO.setBeginDate(beginDate);
                deathDisplayDTO.setEndDate(endDate);
                deathDisplayDTO.setNameType(type);
                result.add(deathDisplayDTO);
            });
        }
        return result;
    }

    public List<ReportRecordDTO> getreportRecordList(String s1, String s2, Map<String, Object> map) {
        List<ReportRecordDTO> reportRecordDTOS = new ArrayList<>();
        EmrRecord emrRecord = new EmrRecord();
        emrRecord.setParams(map);
        String beginDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String name = (String) map.get("name");
        String nameType = (String) map.get("nameType");
        if ("null".equals(beginDate)) {
            beginDate = null;
        }
        if ("null".equals(endDate)) {
            endDate = null;
        }
        String type = (String) map.get("type");
        if (ObjectUtil.isEmpty(beginDate) && ObjectUtil.isEmpty(endDate)) {
            emrRecord.setBeginTime(DateUtil.beginOfMonth(new Date()));
            emrRecord.setEndTime(DateUtil.endOfMonth(new Date()));
        }
        if (ObjectUtil.isNotEmpty(beginDate)) {
            emrRecord.setBeginTime(DateUtil.parse(beginDate));
        }
        if (ObjectUtil.isNotEmpty(endDate)) {
            emrRecord.setEndTime(DateUtil.parse(endDate));
        }


        if (ObjectUtil.isNotEmpty(name)) {
            if ("科室".equals(nameType)) {
                emrRecord.setInpatientArea(name);
            }
            if ("个人".equals(nameType)) {
                emrRecord.setHospitDoctor(name);
            }
        }
        emrRecord.setDateType(type);
        List<EmrRecord> emrRecords = new ArrayList<>();
        if ("2".equals(type)) {
            emrRecords = uReportExportMapper.selectEmrRecordListByLabelId(emrRecord);
        }
        if ("3".equals(type)) {
            emrRecords = uReportExportMapper.selectEmrRecordListByApproved(emrRecord);
        }
        if ("1".equals(type)) {
            emrRecords = uReportExportMapper.selectEmrRecordList(emrRecord);
        }
        for (EmrRecord record : emrRecords) {
            ReportRecordDTO reportRecordDTO = new ReportRecordDTO();

            reportRecordDTO.setRecordId(record.getRecordId());
            reportRecordDTO.setPatientName(record.getPatientName());
            reportRecordDTO.setInpatientArea(record.getInpatientArea());
            reportRecordDTO.setDiagnosisName(record.getDiagnosisName());
            reportRecordDTO.setHospitNum(record.getHospitNum());
            reportRecordDTO.setHospitDoctor(record.getHospitDoctor());
            reportRecordDTO.setInTime(DateUtil.formatDateTime(record.getInTime()));
            reportRecordDTO.setOutTime(DateUtil.formatDateTime(record.getOutTime()));
            reportRecordDTOS.add(reportRecordDTO);
        }
        return reportRecordDTOS;
    }

    public List<UReportRectificationDisplayDTO> getRectificationList(String s1, String s2, Map<String, Object> map) {

        List<UReportRectificationDisplayDTO> result = new ArrayList<>();
        EmrRecord emrRecord = new EmrRecord();
        emrRecord.setParams(map);
        String beginDate = (String) map.get("startDate");
        String endDate = (String) map.get("endDate");
        String departmentName = (String) map.get("departmentName");
        String doctorName = (String) map.get("doctorName");
        String type = (String) map.get("type");
        if (ObjectUtil.isEmpty(beginDate) && ObjectUtil.isEmpty(endDate)) {
            emrRecord.setBeginTime(DateUtil.beginOfMonth(new Date()));
            emrRecord.setEndTime(DateUtil.endOfMonth(new Date()));
        }
        if (ObjectUtil.isNotEmpty(beginDate)) {
            emrRecord.setBeginTime(DateUtil.parse(beginDate));
        }
        if (ObjectUtil.isNotEmpty(endDate)) {
            emrRecord.setEndTime(DateUtil.parse(endDate));
        }

        if (ObjectUtil.isNotEmpty(departmentName)) {
            emrRecord.setInpatientArea(departmentName);
        }

        if (ObjectUtil.isNotEmpty(doctorName)) {
            emrRecord.setHospitDoctor(doctorName);
        }
        List<UReportRectificationDTO> rectificationDTOS = uReportExportMapper.selectRectificationList(emrRecord);
        if (StrUtil.isNotEmpty(type) && "个人".equals(type)) {

            Map<String, List<UReportRectificationDTO>> collect = rectificationDTOS.stream().collect(Collectors.groupingBy(UReportRectificationDTO :: getHospitDoctor));
            collect.forEach((key, value) -> {
                setDisplay(result, beginDate, endDate, key, value);

            });
        }
        if (StrUtil.isNotEmpty(type) && "科室".equals(type)) {
            Map<String, List<UReportRectificationDTO>> collect = rectificationDTOS.stream().collect(Collectors.groupingBy(UReportRectificationDTO :: getInpatientArea));
            collect.forEach((key, value) -> {
                setDisplay(result, beginDate, endDate, key, value);

            });
        }
        if (StrUtil.isEmpty(type) || "全院".equals(type)) {
            setDisplay(result, beginDate, endDate, "全院", rectificationDTOS);
        }
        return result.stream().sorted(Comparator.comparing(UReportRectificationDisplayDTO::getSort).reversed()).collect(Collectors.toList());

    }

    private static void setDisplay(List<UReportRectificationDisplayDTO> result, String beginDate, String endDate, String key, List<UReportRectificationDTO> value) {
        if (CollUtil.isEmpty(value)) {
            return;
        }
        UReportRectificationDisplayDTO dto = new UReportRectificationDisplayDTO();
        dto.setName(key);
        dto.setBeginDate(beginDate);
        dto.setEndDate(endDate);
        dto.setSunCount(value.size());
        long rectificationCount = value.stream().filter(o -> o.getHistoryCount() - (o.getNowPCount() + o.getNowRCount()) > 0).count();
        dto.setRectificationCount(rectificationCount);
        dto.setSort((double)rectificationCount / (double) value.size());
        dto.setRectificationRate(new BigDecimal(rectificationCount).divide(new BigDecimal(value.size()), 2, RoundingMode.HALF_UP).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP).toString() + "%");
        result.add(dto);
    }

}
